@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_anim_c =
- *   "$Id: anim.c,v 1.12 2002/07/15 04:25:39 mwedel Exp $";
+ *   "$Id: anim.c,v 1.16 2003/09/13 05:01:26 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2002-2003 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -143,21 +143,33 @@ int find_animation(char *name)
 /*
  * animate_object(object) updates the face-variable of an object.
  * If the object is the head of a multi-object, all objects are animated.
+ * op is the object to animate.
+ * dir is the direction the object is facing.  This is generally same as
+ *    op->direction, but in some cases, op->facing is used instead - the
+ *    caller has a better idea which one it really wants to be using,
+ *    so let it pass along the right one.
  */
 
-void animate_object(object *op) {
+void animate_object(object *op, int dir) {
     int max_state;  /* Max animation state object should be drawn in */
     int base_state; /* starting index # to draw from */
-    int	dir=op->direction;
 
     if(!op->animation_id || !NUM_ANIMATIONS(op)) {
 	LOG(llevError,"Object lacks animation.\n");
 	dump_object(op);
 	return;
     }
-    ++op->state;    /* increase draw state */
+    if (op->head) {
+	dir=op->head->direction;
 
-    if (op->head) dir=op->head->direction;
+	if (NUM_ANIMATIONS(op) == NUM_ANIMATIONS(op->head))
+	    op->state = op->head->state;
+	else
+	    ++op->state;
+    }
+    else {
+	++op->state;    /* increase draw state */
+    }
 
     /* If object is turning, then max animation state is half through the
      * animations.  Otherwise, we can use all the animations.
@@ -180,7 +192,7 @@ void animate_object(object *op) {
     }
     else if (NUM_FACINGS(op)==8) {
 	if (dir==0) base_state=0;
-	else base_state = (dir-1)*(NUM_ANIMATIONS(op)/4); 
+	else base_state = (dir-1)*(NUM_ANIMATIONS(op)/8);
     }
 
     /* If beyond drawable states, reset */
@@ -190,9 +202,13 @@ void animate_object(object *op) {
 
     if(op->face==blank_face)
 	op->invisible=1;
-#if 1
-    /* Anyone know what this code here is for?  MSW 980503 */
-    else if(QUERY_FLAG((&op->arch->clone),FLAG_ALIVE)) {
+
+    /* This block covers monsters (eg, pixies) which are supposed to
+     * cycle from visible to invisible and back to being visible.
+     * as such, disable it for players, as then players would become
+     * visible.
+     */
+    else if(op->type != PLAYER && QUERY_FLAG((&op->arch->clone),FLAG_ALIVE)) {
 	if(op->face->number==0) {
 	    op->invisible=1;
 	    CLEAR_FLAG(op, FLAG_ALIVE);
@@ -201,9 +217,10 @@ void animate_object(object *op) {
 	    SET_FLAG(op, FLAG_ALIVE);
 	}
     }
-#endif
+
     if(op->more)
-	animate_object(op->more);
+	animate_object(op->more, dir);
+
     /* update_object will also recursively update all the pieces.
      * as such, we call it last, and only call it for the head
      * piece, and not for the other tail pieces.
