@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_arch_c =
- *   "$Id: arch.c,v 1.23 2002/07/15 04:57:11 mwedel Exp $";
+ *   "$Id: arch.c,v 1.29 2003/10/27 07:24:13 mwedel Exp $";
  */
 
 /*
@@ -39,6 +39,14 @@ int arch_cmp=0;		/* How many strcmp's */
 int arch_search=0;	/* How many searches */
 int arch_init;		/* True if doing arch initialization */
 
+/* The naming of these functions is really poor - they are all
+ * pretty much named '.._arch_...', but they may more may not
+ * return archetypes.  Some make the arch_to_object call, and thus
+ * return an object.  Perhaps those should be called 'archob' functions
+ * to denote they return an object derived from the archetype.
+ * MSW 2003-04-29
+ */
+
 /**
  * GROS -  This function retrieves an archetype given the name that appears
  * during the game (for example, "writing pen" instead of "stylus").
@@ -51,17 +59,53 @@ int arch_init;		/* True if doing arch initialization */
  * - the archetype found or null if nothing was found.
  */
 archetype *find_archetype_by_object_name(char *name) {
-  archetype *at;
+    archetype *at;
 
-  if (name == NULL)
-    return (archetype *) NULL;
+    if (name == NULL)
+	return (archetype *) NULL;
 
-  for(at = first_archetype;at!=NULL;at=at->next)
-  {
-        if (!strcmp(at->clone.name, name))
-                return at;
-  };
-  return NULL;
+    for(at = first_archetype;at!=NULL;at=at->next) {
+	if (!strcmp(at->clone.name, name))
+		return at;
+    }
+    return NULL;
+}
+
+/* This is a lot like the above function.  Instead, we are trying to match
+ * the arch->skill values.  type is the type of object to match
+ * against (eg, to only match against skills or only skill objects for example).
+ * If type is -1, ew don't match on type.
+ */
+object *get_archetype_by_skill_name(char *skill, int type) {
+    archetype *at;
+
+    if (skill == NULL)
+	return NULL;
+
+    for(at = first_archetype;at!=NULL;at=at->next) {
+	if ( ((type == -1) || (type == at->clone.type)) &&
+	     (!strcmp(at->clone.skill, skill)))
+		return arch_to_object(at);
+    }
+    return NULL;
+}
+
+/* similiar to above - this returns the first archetype
+ * that matches both the type and subtype.  type and subtype
+ * can be -1 to say ignore, but in this case, the match it does
+ * may not be very useful.  This function is most useful when
+ * subtypes are known to be unique for a particular type
+ * (eg, skills)
+ */
+archetype *get_archetype_by_type_subtype(int type, int subtype) {
+    archetype *at;
+
+    for(at = first_archetype;at!=NULL;at=at->next) {
+	if ( ((type == -1) || (type == at->clone.type)) &&
+	     (subtype == -1 || subtype == at->clone.subtype))
+		return at;
+    }
+    return NULL;
 }
 
 /**
@@ -167,29 +211,44 @@ int item_matched_string(object *pl, object *op, char *name)
 
 	if (!cp || cp[0]=='\0' || count<0) return 0;
 
-	/* base name matched - not bad */
-	if (strcasecmp(cp,op->name)==0 && !count) return 4;
 
-	else if (count>1) {	/* Need to plurify name for proper match */
-	    if (!strcasecmp(cp,op->name_pl)) {
-		pl->contr->count=count;	/* May not do anything */
-		return 6;
-	    }
-	}
-	else if (count==1) {
-	    if (!strcasecmp(op->name,cp)) {
-		pl->contr->count=count;	/* May not do anything */
-		return 6;
-	    }
-	}
+	/* The code here should go from highest retval to lowest.  That
+	 * is because of the 'else' handling - we don't want to match on
+	 * something and set a low retval, even though it may match a higher retcal
+	 * later.  So keep it in descending order here, so we try for the best
+	 * match first, and work downward.
+	 */
 	if (!strcasecmp(cp,query_name(op))) retval=20;
 	else if (!strcasecmp(cp,query_short_name(op))) retval=18;
 	else if (!strcasecmp(cp,query_base_name(op,0))) retval=16;
 	else if (!strcasecmp(cp,query_base_name(op,1))) retval=16;
+	else if (op->custom_name && !strcasecmp(cp,op->custom_name)) retval=15;
 	else if (!strncasecmp(cp,query_base_name(op,0),
-			      MIN(strlen(cp),strlen(query_base_name(op,0))))) retval=14;
+			      strlen(cp))) retval=14;
 	else if (!strncasecmp(cp,query_base_name(op,1),
-			      MIN(strlen(cp),strlen(query_base_name(op,1))))) retval=14;
+			      strlen(cp))) retval=14;
+
+	/* Do substring checks, so things like 'Str+1' will match.
+	 * retval of these should perhaps be lower - they are lower
+	 * then the specific strcasecmp aboves, but still higher than
+	 * some other match criteria.
+	 */
+	else if (strstr(query_base_name(op,1), cp)) retval = 12;
+	else if (strstr(query_base_name(op,0), cp)) retval = 12;
+	else if (strstr(query_short_name(op), cp)) retval = 12;
+
+	/* Check against plural/non plural based on count. */
+	else if (count>1 && !strcasecmp(cp,op->name_pl)) {
+		retval=6;
+	}
+	else if (count==1 && !strcasecmp(op->name,cp)) {
+		retval=6;
+	}
+	/* base name matched - not bad */
+	else if (strcasecmp(cp,op->name)==0 && !count) retval=4;
+	/* Check for partial custom name, but give a real low priority */
+	else if (op->custom_name && strstr(op->custom_name, cp)) retval = 3;
+
 	if (retval) {
 	    if (pl->type == PLAYER)
 		pl->contr->count=count;
@@ -422,7 +481,7 @@ void second_arch_pass(FILE *fp) {
       if(at!=NULL) {
         treasurelist *tl=find_treasurelist(argument);
         if(tl==NULL)
-          LOG(llevError,"Failed to link treasure to arch: %s\n",variable);
+          LOG(llevError,"Failed to link treasure to arch (%s): %s\n",at->name, argument);
         else
           at->clone.randomitems=tl;
       }
