@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_button_c =
- *   "$Id: button.c,v 1.18 2002/09/05 06:19:27 mwedel Exp $";
+ *   "$Id: button.c,v 1.24 2004/04/18 07:59:36 mwedel Exp $";
  */
 
 /*
@@ -41,80 +41,113 @@
  */
 
 void push_button(object *op) {
-  object *tmp;
-  objectlink *ol;
+    object *tmp;
+    objectlink *ol;
 
 /*   LOG(llevDebug, "push_button: %s (%d)\n", op->name, op->count); */
-  for (ol = get_button_links(op); ol; ol = ol->next) {
-    if (!ol->ob || ol->ob->count != ol->id) {
-      LOG(llevError, "Internal error in push_button (%s).\n", op->name);
-      continue;
-    }
-    /* a button link object can become freed when the map is saving.  As
-     * a map is saved, objects are removed and freed, and if an object is
-     * on top of a button, this function is eventually called.  If a map
-     * is getting moved out of memory, the status of buttons and levers
-     * probably isn't important - it will get sorted out when the map is
-     * re-loaded.  As such, just exit this function if that is the case.
-     */
+    for (ol = get_button_links(op); ol; ol = ol->next) {
+	if (!ol->ob || ol->ob->count != ol->id) {
+	    LOG(llevError, "Internal error in push_button (%s).\n", op->name);
+	    continue;
+	}
+	/* a button link object can become freed when the map is saving.  As
+	 * a map is saved, objects are removed and freed, and if an object is
+	 * on top of a button, this function is eventually called.  If a map
+	 * is getting moved out of memory, the status of buttons and levers
+	 * probably isn't important - it will get sorted out when the map is
+	 * re-loaded.  As such, just exit this function if that is the case.
+	 */
     
-    if (QUERY_FLAG(ol->ob, FLAG_FREED)) return;
-    tmp = ol->ob;
-    switch(tmp->type) {
-    case GATE:
-    case HOLE:
-      tmp->value=tmp->stats.maxsp?!op->value:op->value;
-      tmp->speed=0.5;
-      update_ob_speed(tmp);
-      break;
-    case CF_HANDLE:
-	SET_ANIMATION(tmp, (tmp->value=tmp->stats.maxsp?!op->value:op->value));
-	update_object(tmp,UP_OBJ_FACE);
-	break;
-    case SIGN:
-      if (!tmp->stats.food || tmp->last_eat < tmp->stats.food) {
-        (*info_map_func)(NDI_UNIQUE | NDI_NAVY,tmp->map,tmp->msg);
-        if (tmp->stats.food) tmp->last_eat++;
-      }
-      break;
-    case ALTAR:
-	tmp->value = 1;
-	SET_ANIMATION(tmp, tmp->value);
-	update_object(tmp,UP_OBJ_FACE);
-	break;
-    case BUTTON:
-    case PEDESTAL:
-	tmp->value=op->value;
-	SET_ANIMATION(tmp, tmp->value);
-	update_object(tmp,UP_OBJ_FACE);
-	break;
-    case MOOD_FLOOR:
-	do_mood_floor(tmp, op);
-	break;
-    case TIMED_GATE:
-	tmp->speed = tmp->arch->clone.speed;
-	update_ob_speed(tmp);  /* original values */
-        tmp->value = tmp->arch->clone.value;
-        tmp->stats.sp = 1;
-        tmp->stats.hp = tmp->stats.maxhp;
-        break;
-    case DIRECTOR:
-    case FIREWALL:
+	if (QUERY_FLAG(ol->ob, FLAG_FREED)) return;
+	tmp = ol->ob;
+
+	/* if the criteria isn't appropriate, don't do anything */
+	if (op->value && !QUERY_FLAG(tmp, FLAG_ACTIVATE_ON_PUSH)) continue;
+	if (!op->value && !QUERY_FLAG(tmp, FLAG_ACTIVATE_ON_RELEASE)) continue;
+
+	switch(tmp->type) {
+
+	    case GATE:
+	    case HOLE:
+		tmp->value=tmp->stats.maxsp?!op->value:op->value;
+		tmp->speed=0.5;
+		update_ob_speed(tmp);
+		break;
+
+	    case CF_HANDLE:
+		SET_ANIMATION(tmp, (tmp->value=tmp->stats.maxsp?!op->value:op->value));
+		update_object(tmp,UP_OBJ_FACE);
+		break;
+
+	    case SIGN:
+		if (!tmp->stats.food || tmp->last_eat < tmp->stats.food) {
+		    (*info_map_func)(NDI_UNIQUE | NDI_NAVY,tmp->map,tmp->msg);
+		    if (tmp->stats.food) tmp->last_eat++;
+		}
+	    break;
+
+	    case ALTAR:
+		tmp->value = 1;
+		SET_ANIMATION(tmp, tmp->value);
+		update_object(tmp,UP_OBJ_FACE);
+		break;
+
+	    case BUTTON:
+	    case PEDESTAL:
+		tmp->value=op->value;
+		SET_ANIMATION(tmp, tmp->value);
+		update_object(tmp,UP_OBJ_FACE);
+		break;
+
+	    case MOOD_FLOOR:
+		do_mood_floor(tmp, op);
+		break;
+
+	    case TIMED_GATE:
+		tmp->speed = tmp->arch->clone.speed;
+		update_ob_speed(tmp);  /* original values */
+		tmp->value = tmp->arch->clone.value;
+		tmp->stats.sp = 1;
+		tmp->stats.hp = tmp->stats.maxhp;
+		/* Handle multipart gates.  We copy the value for the other parts
+		 * from the head - this ensures that the data will consistent
+		 */
+		for (tmp=tmp->more; tmp!=NULL; tmp=tmp->more) {
+		    tmp->speed = tmp->head->speed;
+		    tmp->value = tmp->head->value;
+		    tmp->stats.sp = tmp->head->stats.sp;
+		    tmp->stats.hp = tmp->head->stats.hp;
+		    update_ob_speed(tmp);
+		}
+	    break;
+
+	    case DIRECTOR:
+	    case FIREWALL:
 		if(!QUERY_FLAG(tmp,FLAG_ANIMATE)&&tmp->type==FIREWALL) (*move_firewall_func)(tmp);
 		else {
-        if ((tmp->stats.sp += tmp->stats.maxsp) > 8) /* next direction */
-          tmp->stats.sp = ((tmp->stats.sp-1)%8)+1;
-        animate_turning(tmp);
+		    if ((tmp->stats.sp += tmp->stats.maxsp) > 8) /* next direction */
+			tmp->stats.sp = ((tmp->stats.sp-1)%8)+1;
+		    animate_turning(tmp);
 		}
-        break;
-    case TELEPORTER:
-	(*move_teleporter_func)(tmp);
-	break;
-    case CREATOR:
-	(*move_creator_func)(tmp);
-	break;
+		break;
+
+	    case TELEPORTER:
+		(*move_teleporter_func)(tmp);
+		break;
+
+	    case CREATOR:
+		(*move_creator_func)(tmp);
+		break;
+		
+		case TRIGGER_MARKER:
+		(*move_trigger_marker_func)(tmp);
+		break;
+
+	    case DUPLICATOR:
+		(*move_duplicator_func)(tmp);
+		break;
+	}
     }
-  }
 }
 
 /*
@@ -299,7 +332,10 @@ void trigger_move (object *op, int state) /* 1 down and 0 up */
     op->stats.wc = state;
     if (state) {
 	use_trigger(op);
-	op->speed = 1.0 / op->arch->clone.stats.exp;
+	if (op->stats.exp > 0) /* check sanity  */
+		op->speed = 1.0 / op->stats.exp;
+	else
+		op->speed = 1.0;
 	update_ob_speed(op);
 	op->speed_left = -1;
     } else {
@@ -564,8 +600,10 @@ void do_mood_floor(object *op, object *op2) {
 			 * dereference a null value
 			 */
 			if (tmp->type == GOLEM && tmp->owner && tmp->owner->type==PLAYER &&
-			    tmp->owner->contr->golem==tmp)
-				tmp->owner->contr->golem=NULL;
+			    tmp->owner->contr->ranges[range_golem]==tmp) {
+				tmp->owner->contr->ranges[range_golem]=NULL;
+				tmp->owner->contr->golem_count = 0;
+			}
 			tmp->owner = 0;
 		}
 		break;
@@ -617,14 +655,20 @@ object * check_inv_recursive(object *op, object *trig)
 {
     object *tmp,*ret=NULL;
 
+    /* First check the object itself. */
+    if((trig->stats.hp && (op->type == trig->stats.hp)) 
+	|| (trig->slaying && (op->slaying == trig->slaying)) 
+	|| (trig->race && (op->arch->name == trig->race))) 
+	    return op;
+
     for(tmp=op->inv; tmp; tmp=tmp->below) {
 	if (tmp->inv) {
 	    ret=check_inv_recursive(tmp, trig);
 	    if (ret) return ret;
 	}
-	else if((trig->stats.hp && tmp->type == trig->stats.hp) 
-		|| tmp->slaying == trig->slaying 
-		|| tmp->arch->name == trig->race) 
+	else if((trig->stats.hp && (tmp->type == trig->stats.hp)) 
+		|| (trig->slaying && (tmp->slaying == trig->slaying)) 
+		|| (trig->race && (tmp->arch->name == trig->race))) 
 		    return tmp;
     }
     return NULL;
