@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_image_c =
- *   "$Id: image.c,v 1.6 2002/03/26 07:18:56 mwedel Exp $";
+ *   "$Id: image.c,v 1.10 2004/02/28 10:40:56 ryo_saeba Exp $";
  */
 
 /*
@@ -75,6 +75,13 @@ New_Face *blank_face, *dark_faces[3], *potion_face, *empty_face;
  */
 int nroffiles = 0, nrofpixmaps=0;
 
+struct smoothing {
+    uint16 id;
+    uint16 smooth;
+};
+
+static struct smoothing *smooth=NULL;
+int nrofsmooth=0;
 /* the only thing this table is used for now is to
  * translate the colorname in the magicmap field of the
  * face into a numeric index that is then sent to the
@@ -100,6 +107,13 @@ char *colorname[NUM_COLORS] = {
 static int compar (struct bmappair *a, struct bmappair *b) {
     return strcmp (a->name, b->name);
 }
+static int compar_smooth (struct smoothing *a, struct smoothing *b) {
+    if (a->id<b->id)
+        return -1;
+    if (b->id<a->id)
+        return 1;
+    return 0;
+}
 
 
 /*
@@ -185,7 +199,7 @@ static void ReadFaceData()
     fclose(fp);
 }
 
-/* This reads the bmaps.paths file to get all the bitmap names and
+/* This reads the bmaps file to get all the bitmap names and
  * stuff.  It only needs to be done once, because it is player
  * independent (ie, what display the person is on will not make a
  * difference.)
@@ -301,7 +315,6 @@ int ReadBmapNames () {
     return nrofpixmaps;
 }
 
-
 /* This returns an the face number of face 'name'.  Number is constant
  * during an invocation, but not necessarily between versions (this
  * is because the faces are arranged in alphabetical order, so
@@ -339,6 +352,92 @@ int FindFace (char *name, int error) {
 
     return bp ? bp->number : error;
 }
+/* According to docs, faces are saved in an array
+ * The number associated with each face is it's
+ * position in the array. Right, this should
+ * be easy to convert a facenumber to a facename.
+ * Hope this is so. Tchize
+ * Damn, this is not, have to work around a little. Tchize
+ */
+char* FindFaceName (int facenbr, char* error) {
+    int i=0;
+    if ( (facenbr>=nroffiles) ||(facenbr<0)) 
+        return error;
+    /*xbm[facenbr] is 'near' the face we are seeking*/
+    if (xbm[facenbr].number==facenbr)
+        return xbm[facenbr].name;
+    for (i=1;;i++){
+        if ( ((facenbr-i)<0) && ((facenbr+i)>nroffiles))
+            return error;
+        if (xbm[facenbr-i].number==facenbr){
+            return xbm[facenbr-i].name;
+        }
+        if (xbm[facenbr+i].number==facenbr){
+            return xbm[facenbr+i].name;
+        }
+    }
+}
+/* Reads the smooth file to know how to smooth datas.
+ * the smooth file if made of 2 elements lines.
+ * lines starting with # are comment
+ * the first element of line is face to smooth
+ * the next element is the 16x2 faces picture
+ * used for smoothing
+ */
+int ReadSmooth () {
+    char buf[MAX_BUF], *p, *q;
+    FILE *fp;
+    int smoothcount = 0;
+
+    bmaps_checksum=0;
+    sprintf (buf,"%s/smooth", settings.datadir);
+    LOG(llevDebug,"Reading smooth from %s...",buf);
+    if ((fp=fopen(buf,"r"))==NULL) {
+	perror("Can't open smooth file");
+	printf("buf = %s\n", buf);
+	exit(-1);
+    }
+
+    /* First count how many smooth we have, so we can allocate correctly */
+    while (fgets (buf, MAX_BUF, fp)!=NULL)
+	if(buf[0] != '#' && buf[0] != '\n' )
+	    smoothcount++;
+    rewind(fp);
+
+    smooth = (struct smoothing *) malloc(sizeof(struct smoothing) * (smoothcount));
+    memset (smooth, 0, sizeof (struct smoothing) * (smoothcount));
+
+    while(fgets (buf, MAX_BUF, fp)!=NULL) {
+        if (*buf == '#')
+            continue;
+        p=strchr(buf,' ');
+        if (!p)
+            continue;
+        *p='\0';
+        q=buf;
+        smooth[nrofsmooth].id=FindFace(q,0);
+        q=p+1;
+        smooth[nrofsmooth].smooth=FindFace(q,0);
+        nrofsmooth++;
+    }
+    fclose(fp);
+
+    LOG(llevDebug,"done (got %d smooth entries)\n",nrofsmooth);
+    qsort (smooth, nrofsmooth, sizeof(struct smoothing), (int (*)())compar_smooth);
+    return nrofsmooth;
+}
+
+int FindSmooth (uint16 face, uint16* smoothed) {
+    struct smoothing *bp, tmp;
+
+    tmp.id = face;
+    bp = (struct smoothing *)bsearch 
+        (&tmp, smooth, nrofsmooth, sizeof(struct smoothing), (int (*)())compar_smooth);
+    (*smoothed)=0;
+    if (bp)
+         (*smoothed)=bp->smooth;
+    return bp ? 1 : 0;
+}
 
 void free_all_images()
 {
