@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.18 2002/09/11 06:21:46 mwedel Exp $";
+ *   "$Id: init.c,v 1.34 2004/02/16 20:03:10 tanner Exp $";
  */
 
 /*
@@ -31,11 +31,11 @@
 #include <global.h>
 #include <object.h>
 
-/* You unforunately need to looking in include/structs.h to see what these
+/* You unforunately need to looking in include/global.h to see what these
  * correspond to.
  */
 struct Settings settings = {
-"",			        /* Logfile */
+"/var/log/crossfire/logfile",   /* Logfile */
 CSPORT,				/* Client/server port */
 
 /* Debug level */
@@ -56,15 +56,58 @@ UNIQUE_DIR, TMPDIR,
 STAT_LOSS_ON_DEATH,
 USE_PERMANENT_EXPERIENCE,
 BALANCED_STAT_LOSS,
+NOT_PERMADETH,
 SIMPLE_EXP,
 RESET_LOCATION_TIME,
+SET_TITLE,
+RESURRECTION,
+SEARCH_ITEMS,
+SPELL_ENCUMBRANCE,
+SPELL_FAILURE_EFFECTS,
+CASTING_TIME,
+REAL_WIZ,
+RECYCLE_TMP_MAPS,
+EXPLORE_MODE,
+SPELLPOINT_LEVEL_DEPEND,
+SET_FRIENDLY_FIRE,
+MOTD,
+"",		/* DM_MAIL */
 0,		/* This and the next 3 values are metaserver values */
 "",
 "",
 0,
 "",
-0,0,0,0,0,0,0,  /* worldmap settings*/
-EMERGENCY_MAPPATH, EMERGENCY_X, EMERGENCY_Y
+0,0,0,0,0,0,0,0,  /* worldmap settings*/
+EMERGENCY_MAPPATH, EMERGENCY_X, EMERGENCY_Y,
+0,
+1.0
+};
+
+/* perhaps not the best place for this, but needs to be
+ * in some file in the common area so that standalone
+ * programs, like the random map generator, can be built.
+ */
+char *spellpathnames[NRSPELLPATHS] = {
+ "Protection",
+ "Fire",
+ "Frost",
+ "Electricity",
+ "Missiles",
+ "Self",
+ "Summoning",
+ "Abjuration",
+ "Restoration",
+ "Detonation",
+ "Mind",
+ "Creation",
+ "Teleportation",
+ "Information",
+ "Transmutation",
+ "Transferrence",
+ "Turning",
+ "Wounding",
+ "Death",
+ "Light"
 };
 
 
@@ -128,6 +171,7 @@ void init_library() {
     init_vars();
     init_block();
     ReadBmapNames ();
+    ReadSmooth();
     init_anim();    /* Must be after we read in the bitmaps */
     init_archetypes();	/* Reads all archetypes from file */
     init_dynamic ();
@@ -145,7 +189,6 @@ void init_library() {
 void init_environ() {
     char *cp;
 
-#ifndef SECURE
     cp=getenv("CROSSFIRE_LIBDIR");
     if (cp) settings.datadir=cp;
     cp=getenv("CROSSFIRE_LOCALDIR");
@@ -162,7 +205,6 @@ void init_environ() {
     if (cp) settings.uniquedir=cp;
     cp=getenv("CROSSFIRE_TMPDIR");
     if (cp) settings.tmpdir=cp;
-#endif
 }
     
 
@@ -175,7 +217,7 @@ void init_globals() {
     if (settings.logfilename[0] == 0) {
 	logfile = stderr;
     }
-    else if ((logfile=fopen(settings.logfilename, "w"))==NULL) {
+    else if ((logfile=fopen(settings.logfilename, "a"))==NULL) {
 	fprintf(stderr,"Unable to open %s as the logfile - will use stderr instead\n",
 		settings.logfilename);
 	logfile = stderr;
@@ -398,4 +440,3 @@ void init_attackmess(){
     LOG(llevDebug, "got %d messages in %d categories.\n", total, mess+1);
     close_and_delete(fp, comp);
 }
-	    
