@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c,v 1.26 2002/08/11 07:52:15 mwedel Exp $";
+ *   "$Id: item.c,v 1.43 2004/04/26 05:07:52 mwedel Exp $";
  */
 
 /*
@@ -92,13 +92,13 @@ static char levelnumbers_10[11][20] = {
   "seventieth","eightieth","ninetieth"
 };
 
-
+/*
 materialtype material[NROFMATERIALS] = {
-  /*  		  P  M  F  E  C  C  A  D  W  G  P S P T F  C D D C C G H B  I *
+  *  		  P  M  F  E  C  C  A  D  W  G  P S P T F  C D D C C G H B  I *
    *		  H  A  I  L  O  O  C  R  E  H  O L A U E  A E E H O O O L  N *
    *		  Y  G  R  E  L  N  I  A  A  O  I O R R A  N P A A U D L I  T *
    *		  S  I  E  C  D  F  D  I  P  S  S W A N R  C L T O N   Y N  R *
-   *		  I  C     T     U     N  O  T  O   L      E E H S T P   D  N */
+   *		  I  C     T     U     N  O  T  O   L      E E H S T P   D  N *
   {"paper", 	{15,10,17, 9, 5, 7,13, 0,20,15, 0,0,0,0,0,10,0,0,0,0,0,0,0,0}},
   {"metal", 	{ 2,12, 3,12, 2,10, 7, 0,20,15, 0,0,0,0,0,10,0,0,0,0,0,0,0,0}},
   {"glass", 	{14,11, 8, 3,10, 5, 1, 0,20,15, 0,0,0,0,0, 0,0,0,0,0,0,0,0,0}},
@@ -113,21 +113,21 @@ materialtype material[NROFMATERIALS] = {
   {"bone", 	{10, 9, 4, 5, 3,10,10, 0,20,15, 0,0,0,0,0, 2,0,0,0,0,0,0,0,0}},
   {"ice", 	{14,11,16, 5, 0, 5, 6, 0,20,15, 0,0,0,0,0, 7,0,0,0,0,0,0,0,0}}
 };
-
+*/
 
 /* This curve may be too steep.  But the point is that there should
  * be tough choices - there is no real point to this if everyone can
  * wear whatever they want with no worries.  Perhaps having the steep
  * curve is good (maybe even steeper), but allowing players to
  * have 2 * level instead.  Ideally, top level characters should only be
- * able to use 2-3 of the most power items.
+ * able to use 2-3 of the most powerful items.
  * note that this table is only really used for program generated items -
  * custom objects can use whatever they want.
  */
 static int enc_to_item_power[21] = {
 0, 0, 1, 2, 3, 4,    /* 5 */
 5, 7, 9, 11, 13,    /* 10 */
-16, 18, 21, 24, 27, /* 15 */
+15, 18, 21, 24, 27, /* 15 */
 30, 35, 40, 45, 50  /* 20 */
 };
 
@@ -205,14 +205,7 @@ int calc_item_power(object *op, int flag)
     if(QUERY_FLAG(op,FLAG_SEE_IN_DARK))	    enc += 1;
     if(QUERY_FLAG(op,FLAG_MAKE_INVIS))	    enc += 1;
 
-    if (enc > 20) {
-	LOG(llevDebug,"calc_item_power got %d enchantments for %s\n", enc, op->name);
-	enc = 20;
-    }
-    /* Items only have a positive power rating */
-    if (enc < 0) enc = 0;
-
-    return enc_to_item_power[enc];
+    return get_power_from_ench(enc);
 
 }
 
@@ -328,7 +321,7 @@ char *ring_desc (object *op)
 	}
     }
     if(op->stats.exp)
-	sprintf(buf+strlen(buf), "(speed %+d)", op->stats.exp);
+	sprintf(buf+strlen(buf), "(speed %+lld)", op->stats.exp);
     if(op->stats.wc)
 	sprintf(buf+strlen(buf), "(wc%+d)", op->stats.wc);
     if(op->stats.dam)
@@ -363,9 +356,10 @@ char *ring_desc (object *op)
     DESCRIBE_PATH_SAFE(buf, op->path_attuned, "Attuned", &len, VERY_BIG_BUF);
     DESCRIBE_PATH_SAFE(buf, op->path_repelled, "Repelled", &len, VERY_BIG_BUF);
     DESCRIBE_PATH_SAFE(buf, op->path_denied, "Denied", &len, VERY_BIG_BUF);
-    if(op->item_power)
-	sprintf(buf+strlen(buf), "(item_power %+d)",op->item_power);
 
+    /*    if(op->item_power)
+	sprintf(buf+strlen(buf), "(item_power %+d)",op->item_power);
+    */
     if(buf[0] == 0 && op->type!=SKILL)
 	strcpy(buf,"of adornment");
 
@@ -388,7 +382,7 @@ char *query_short_name(object *op)
     if(!op->nrof && !op->weight && !op->title && !is_magical(op)) 
 	return op->name; /* To speed things up (or make things slower?) */
 
-    if (op->nrof == 1)
+    if (op->nrof <= 1)
 	safe_strcat(buf,op->name, &len, HUGE_BUF);
     else
 	safe_strcat(buf,op->name_pl, &len, HUGE_BUF);
@@ -399,40 +393,28 @@ char *query_short_name(object *op)
     }
 
     switch(op->type) {
-      case SPELLBOOK:
-	if (QUERY_FLAG(op,FLAG_IDENTIFIED)||QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
-	    if(!op->title) {
-		safe_strcat(buf," of ", &len, HUGE_BUF);
-		if(op->slaying) safe_strcat(buf,op->slaying, &len, HUGE_BUF);
-		else
-		  safe_strcat(buf,spells[op->stats.sp].name, &len, HUGE_BUF);
+	case SPELLBOOK:
+	case SCROLL:
+	case WAND:
+	case ROD:
+	    if (QUERY_FLAG(op,FLAG_IDENTIFIED)||QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
+		if(!op->title) {
+		    safe_strcat(buf," of ", &len, HUGE_BUF);
+		    if (op->inv) 
+			safe_strcat(buf,op->inv->name, &len, HUGE_BUF);
+		    else
+			LOG(llevError,"Spellbook %s lacks inventory\n", op->name);
+		}
 		if(op->type != SPELLBOOK) {
 		    sprintf(buf2, " (lvl %d)", op->level);
 		    safe_strcat(buf, buf2, &len, HUGE_BUF);
 		}
 	    }
-	}
-	
-	break;
-      case SCROLL:
-      case WAND:
-      case ROD:
-	if (QUERY_FLAG(op,FLAG_IDENTIFIED)||QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
-	    if(!op->title) {
-		safe_strcat(buf," of ", &len, HUGE_BUF);
-		safe_strcat(buf,spells[op->stats.sp].name, &len, HUGE_BUF);
-		if(op->type != SPELLBOOK) {
-		    sprintf(buf2, " (lvl %d)", op->level);
-		    safe_strcat(buf, buf2, &len, HUGE_BUF);
-		}
-	    }
-	}
-	break;
+	    break;
 
       case SKILL:
       case AMULET:
       case RING:
-#ifdef FULL_RING_DESCRIPTION
 	if (!op->title) {
 	    /* If ring has a title, full description isn't so useful */ 
 	    char *s = ring_desc(op);
@@ -441,7 +423,6 @@ char *query_short_name(object *op)
 		safe_strcat(buf, s, &len, HUGE_BUF);
 	    }
 	}
-#endif
 	break;
       default:
 	if(op->magic && ((QUERY_FLAG(op,FLAG_BEEN_APPLIED) && 
@@ -468,10 +449,23 @@ char *query_name(object *op) {
     static char buf[5][HUGE_BUF];
     static int use_buf=0;
     int len=0;
+#ifdef NEW_MATERIAL_CODE
+    materialtype_t *mt;
+#endif
 
     use_buf++;
     use_buf %=5;
 
+#ifdef NEW_MATERIAL_CODE
+    if ((IS_ARMOR(op) || IS_WEAPON(op)) && op->materialname) {
+	mt = name_to_material(op->materialname);
+	if (mt) {
+	    safe_strcat(buf[use_buf], mt->description, &len, HUGE_BUF);
+	    safe_strcat(buf[use_buf], " ", &len, HUGE_BUF);
+	}
+    }
+#endif
+
     safe_strcat(buf[use_buf], query_short_name(op), &len, HUGE_BUF);
 
     if (QUERY_FLAG(op,FLAG_INV_LOCKED))
@@ -497,7 +491,18 @@ char *query_name(object *op) {
      */
     if (QUERY_FLAG(op,FLAG_KNOWN_MAGICAL) && !QUERY_FLAG(op,FLAG_IDENTIFIED))
 	safe_strcat(buf[use_buf], " (magic)", &len, HUGE_BUF);
-    if(QUERY_FLAG(op,FLAG_APPLIED)) {
+
+#if 0
+    /* item_power will be returned in desribe_item - it shouldn't really
+     * be returned in the name.
+     */
+    if(op->item_power)
+	sprintf(buf[use_buf]+strlen(buf[use_buf]), "(item_power %+d)",
+	    op->item_power);
+
+#endif
+
+    if (QUERY_FLAG(op,FLAG_APPLIED)) {
 	switch(op->type) {
 	  case BOW:
 	  case WAND:
@@ -544,8 +549,9 @@ char *query_name(object *op) {
  * If plural is set, we generate the plural name of this.
  */
 char *query_base_name(object *op, int plural) {
-    static char buf[MAX_BUF];
+    static char buf[MAX_BUF], buf2[MAX_BUF];
     int len;
+    materialtype_t *mt;
 
     if((!plural && !op->name) || (plural && !op->name_pl))
 	return "(null)";
@@ -553,11 +559,30 @@ char *query_base_name(object *op, int plural) {
     if(!op->nrof && !op->weight && !op->title && !is_magical(op)) 
 	return op->name; /* To speed things up (or make things slower?) */
 
-    if (!plural) 
-	strcpy(buf,op->name);
-    else
-	strcpy(buf,op->name_pl);
-    len=strlen(buf);
+    if ((IS_ARMOR(op) || IS_WEAPON(op)) && op->materialname)
+	mt = name_to_material(op->materialname);
+
+#ifdef NEW_MATERIAL_CODE
+    if ((IS_ARMOR(op) || IS_WEAPON(op)) && op->materialname && mt &&
+	op->arch->clone.materialname != mt->name &&
+	!(op->material & M_SPECIAL)) {
+	strcpy(buf, mt->description);
+	len=strlen(buf);
+	safe_strcat(buf, " ", &len, MAX_BUF);
+	if (!plural)
+	    safe_strcat(buf, op->name, &len, MAX_BUF);
+	else
+	    safe_strcat(buf, op->name_pl, &len, MAX_BUF);
+    } else {
+#endif
+	if (!plural)
+	    strcpy(buf, op->name);
+	else
+	    strcpy(buf, op->name_pl);
+	len=strlen(buf);
+#ifdef NEW_MATERIAL_CODE
+    }
+#endif
 
     if (op->title && QUERY_FLAG(op,FLAG_IDENTIFIED)) {
 	safe_strcat(buf, " ", &len, MAX_BUF);
@@ -565,36 +590,29 @@ char *query_base_name(object *op, int plural) {
     }
 
     switch(op->type) {
-      case SPELLBOOK:
-	if (QUERY_FLAG(op,FLAG_IDENTIFIED)||QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
-	    if(!op->title) {
-		safe_strcat(buf," of ", &len, MAX_BUF);
-		if(op->slaying) safe_strcat(buf,op->slaying, &len, MAX_BUF);
-		else
-		  safe_strcat(buf,spells[op->stats.sp].name, &len, MAX_BUF);
-		if(op->type != SPELLBOOK) 
-		    sprintf(buf+strlen(buf), " (lvl %d)", op->level);
+	case SPELLBOOK:
+	case SCROLL:
+	case WAND:
+	case ROD:
+	    if (QUERY_FLAG(op,FLAG_IDENTIFIED)||QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
+		if(!op->title) {
+		    safe_strcat(buf," of ", &len, MAX_BUF);
+		    if (op->inv) 
+			safe_strcat(buf,op->inv->name, &len, MAX_BUF);
+		    else
+			LOG(llevError,"Spellbook %s lacks inventory\n", op->name);
+		}
+		if(op->type != SPELLBOOK) {
+		    sprintf(buf2, " (lvl %d)", op->level);
+		    safe_strcat(buf, buf2, &len, MAX_BUF);
+		}
 	    }
-	}
-	break;
+	    break;
 
-      case SCROLL:
-      case WAND:
-      case ROD:
-	if (QUERY_FLAG(op,FLAG_IDENTIFIED)||QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
-	    if(!op->title) {
-		safe_strcat(buf," of ", &len, MAX_BUF);
-		safe_strcat(buf,spells[op->stats.sp].name, &len, MAX_BUF);
-		if(op->type != SPELLBOOK)
-		    sprintf(buf+strlen(buf), " (lvl %d)", op->level);
-	    }
-	}
-	break;
 
       case SKILL:
       case AMULET:
       case RING:
-#ifdef FULL_RING_DESCRIPTION
 	if (!op->title) {
 	    /* If ring has a title, full description isn't so useful */ 
 	    char *s = ring_desc(op);
@@ -603,7 +621,6 @@ char *query_base_name(object *op, int plural) {
 		safe_strcat (buf, s, &len, MAX_BUF);
 	    }
 	}
-#endif
 	break;
       default:
 	if(op->magic && ((QUERY_FLAG(op,FLAG_BEEN_APPLIED) && 
@@ -694,7 +711,7 @@ static char *describe_monster(object *op) {
 	treasure *t;
 	int first = 1;
 	for(t=op->randomitems->items; t != NULL; t=t->next)
-	    if(t->item && (t->item->clone.type == ABILITY)) {
+	    if(t->item && (t->item->clone.type == SPELL)) {
 		if(first) {
 		    first = 0;
 		    strcat(retbuf,"(Spell abilities:)");
@@ -797,7 +814,10 @@ char *describe_item(object *op, object *owner) {
      * also makes the code easier to read.
      */
     if (!need_identify(op) || QUERY_FLAG(op, FLAG_IDENTIFIED)) identified = 1;
-    else identified = 0;
+    else {
+	strcpy(retbuf,"(unidentified)");
+	identified = 0;
+    }
 
     switch(op->type) {
 	case BOW:
@@ -814,6 +834,7 @@ char *describe_item(object *op, object *owner) {
 	case GIRDLE:
 	case BRACERS:
 	case CLOAK:
+	case SKILL_TOOL:
 	    break;  /* We have more information to do below this switch */
 
 	case FOOD:
@@ -847,12 +868,12 @@ char *describe_item(object *op, object *owner) {
 	case SKILL:
 	case RING:
 	case AMULET:
-#ifdef FULL_RING_DESCRIPTION
+	    if(op->item_power) {
+		sprintf(buf,"(item_power %+d)",op->item_power);
+		strcat(retbuf,buf);
+	    }
 	    if (op->title)
 		strcat (retbuf, ring_desc(op));
-#else
-	    strcat (retbuf, ring_desc(op));
-#endif
 	    return retbuf;
 
 	default:
@@ -873,7 +894,7 @@ char *describe_item(object *op, object *owner) {
 	}
 
 	if(op->stats.exp) {
-	    sprintf(buf,"(speed %+d)",op->stats.exp);
+	    sprintf(buf,"(speed %+lld)",op->stats.exp);
 	    strcat(retbuf,buf);
 	}
 
@@ -906,7 +927,7 @@ char *describe_item(object *op, object *owner) {
 		    sprintf(buf,"(ac%+d)",op->stats.ac);
 		    strcat(retbuf,buf);
 		}
-		if (op->type==WEAPON && op->level>0) {
+		if ((op->type==WEAPON || op->type == BOW) && op->level>0) {
 		    sprintf(buf,"(improved %d/%d)",op->last_eat,op->level);
 		    strcat(retbuf,buf);
 		}
@@ -960,11 +981,10 @@ char *describe_item(object *op, object *owner) {
 		    sprintf(buf,"(Max speed %1.2f)", ARMOUR_SPEED(op) / 10.0);
 		    strcat(retbuf, buf);
 		}
-		/* Do this in all cases - otherwise it gets confusing - does that
-		 * item have no penality, or is it not fully identified for example.
-	         */
-		sprintf(buf,"(Spell regen penalty %d)", ARMOUR_SPELLS(op));
-		strcat(retbuf, buf);
+		if (ARMOUR_SPELLS(op)) {
+		    sprintf(buf,"(Spell regen penalty %d)", ARMOUR_SPELLS(op));
+		    strcat(retbuf, buf);
+		}
 		more_info=1;
 		break;
 
@@ -1110,107 +1130,89 @@ int is_magical(object *op) {
  */
 
 int need_identify(object *op) {
-  switch(op->type) {
-  case RING:
-  case WAND:
-  case ROD:
-  case HORN:
-  case SCROLL:
-  case SKILL:
-  case SKILLSCROLL:
-  case SPELLBOOK:
-  case FOOD:
-  case POTION:
-  case BOW:
-  case ARROW:
-  case WEAPON:
-  case ARMOUR:
-  case SHIELD:
-  case HELMET:
-  case AMULET:
-  case BOOTS:
-  case GLOVES:
-  case BRACERS:
-  case GIRDLE:
-  case CONTAINER:
-  case DRINK:
-  case FLESH:
-  case INORGANIC:
-  case CLOSE_CON:
-  case CLOAK:
-  case GEM:
-  case POWER_CRYSTAL:
-  case POISON:
-  case BOOK:
-    return 1;
-  }
-/* Try to track down some stuff that may show up here.  Thus, the
- * archetype file can be updated, and this function removed.
- */
+    switch(op->type) {
+	case RING:
+	case WAND:
+	case ROD:
+	case HORN:
+	case SCROLL:
+	case SKILL:
+	case SKILLSCROLL:
+	case SPELLBOOK:
+	case FOOD:
+	case POTION:
+	case BOW:
+	case ARROW:
+	case WEAPON:
+	case ARMOUR:
+	case SHIELD:
+	case HELMET:
+	case AMULET:
+	case BOOTS:
+	case GLOVES:
+	case BRACERS:
+	case GIRDLE:
+	case CONTAINER:
+	case DRINK:
+	case FLESH:
+	case INORGANIC:
+	case CLOSE_CON:
+	case CLOAK:
+	case GEM:
+	case POWER_CRYSTAL:
+	case POISON:
+	case BOOK:
+	case SKILL_TOOL:
+	    return 1;
+    }
+    /* Try to track down some stuff that may show up here.  Thus, the
+     * archetype file can be updated, and this function removed.
+     */
 #if 0
-  LOG(llevDebug,"need_identify: %s does not need to be id'd\n", op->name);
+    LOG(llevDebug,"need_identify: %s does not need to be id'd\n", op->name);
 #endif
-  return 0;
+    return 0;
 }
 
 
-/* 
- *  Return the number of the spell that whose name passes the pasesed string
- *  argument.   Return -1 if no such spell name match is found.
- */
-int look_up_spell_name( char * spname ){
-   register int i;
-   for(i=0;i< NROFREALSPELLS;i++){
-      if( strcmp(spname, spells[i].name) == 0) return i;
-   }
-   return -1;
-}
-
 /*
  * Supposed to fix face-values as well here, but later.
  */
 
 void identify(object *op) {
-  object *pl;
+    object *pl;
 
-  SET_FLAG(op,FLAG_IDENTIFIED);
-  CLEAR_FLAG(op, FLAG_KNOWN_MAGICAL);
-  CLEAR_FLAG(op, FLAG_NO_SKILL_IDENT);
+    SET_FLAG(op,FLAG_IDENTIFIED);
+    CLEAR_FLAG(op, FLAG_KNOWN_MAGICAL);
+    CLEAR_FLAG(op, FLAG_NO_SKILL_IDENT);
 
-/*
- * We want autojoining of equal objects:
- */
-  if (QUERY_FLAG(op,FLAG_CURSED) || QUERY_FLAG(op,FLAG_DAMNED))
-    SET_FLAG(op,FLAG_KNOWN_CURSED);
+    /*
+     * We want autojoining of equal objects:
+     */
+    if (QUERY_FLAG(op,FLAG_CURSED) || QUERY_FLAG(op,FLAG_DAMNED))
+	SET_FLAG(op,FLAG_KNOWN_CURSED);
 
-  if (op->type == POTION && op->arch != (archetype *) NULL) {
-    /*op->face = op->arch->clone.face; */
-      free_string(op->name);
-      op->name = add_refcount(op->arch->clone.name);
-  } else if( op->type == SPELLBOOK && op->slaying != NULL){
-       if((op->stats.sp = look_up_spell_name( op->slaying )) <0 ){
-	  char buf[256];
-          op->stats.sp = -1;  
-          sprintf(buf, "Spell forumla for %s", op->slaying);
-	  if(op->name != NULL) 
-		free_string(op->name);
-	  op->name = add_string(buf);
-       } else {
-         /* clear op->slaying since we no longer need it */
-         free_string(op->slaying);
-         op->slaying=NULL;
-       }
-  }
+    if (op->type == POTION) {
+	if (op->inv && op->randomitems) {
+	    if (op->title) free_string(op->title);
+	    op->title = add_refcount(op->inv->name);
+	} else if (op->arch) {
+	    free_string(op->name);
+	    op->name = add_refcount(op->arch->clone.name);
+	    free_string(op->name_pl);
+	    op->name_pl = add_refcount(op->arch->clone.name_pl);
+	}
+    }
 
-  if (op->map) /* The shop identifies items before they hit the ground */
-    /* I don't think identification will change anything but face */
-    update_object(op,UP_OBJ_FACE);
-  else {
-    pl = is_player_inv(op->env);
-    if (pl)
-	/* A lot of the values can change from an update - might as well send
-	 * it all.
-	 */
-	esrv_send_item_func(pl, op);
-  }
+    /* If the object is on a map, make sure we update its face */
+    if (op->map)
+	update_object(op,UP_OBJ_FACE);
+    else {
+	pl = is_player_inv(op->env);
+	if (pl)
+	    /* A lot of the values can change from an update - might as well send
+	     * it all.
+	     */
+	    esrv_send_item_func(pl, op);
+    }
 }
