@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_loger_c =
- *   "$Id: logger.c,v 1.7 2002/07/15 04:25:39 mwedel Exp $ ";
+ *   "$Id: logger.c,v 1.8 2003/09/13 05:01:29 mwedel Exp $ ";
  */
 
 /*
@@ -70,10 +70,6 @@ void LOG (LogLevel logLevel, char *format, ...)
     exiting = 1;
     if (!trying_emergency_save)
       (*emergency_save_func) (0);
-#if 0
-    if (!editor)
-	fatal(TOO_MANY_ERRORS);
-#endif
   }
   va_end(ap);
 }
