@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_porting_c =
- *   "$Id: porting.c,v 1.14 2002/09/07 05:55:18 mwedel Exp $";
+ *   "$Id: porting.c,v 1.23 2004/03/26 22:41:43 ryo_saeba Exp $";
  */
 
 /*
@@ -58,6 +58,7 @@
 #include <unistd.h>
 #endif
 
+#include <stdarg.h>
 /* Has to be after above includes so we don't redefine some values */
 #include "global.h"
 
@@ -116,20 +117,21 @@ void remove_directory(const char *path)
 	struct dirent *de;
 
 	for (de=readdir(dirp); de; de = readdir(dirp)) {
-	    status=stat(de->d_name, &statbuf);
-	    /* Linus actually has a type field in the dirent structure,
-	     * but that is not portable - stat should be portable
-	     */
-	    if ((status!=-1) && (S_ISDIR(statbuf.st_mode))) {
-		sprintf(buf,"%s/%s", path, de->d_name);
-		remove_directory(buf);
-		continue;
-	    }
 	    /* Don't remove '.' or '..'  In  theory we should do a better 
 	     * check for .., but the directories we are removing are fairly
 	     * limited and should not have dot files in them.
 	     */
 	    if (de->d_name[0] == '.') continue;
+
+	    /* Linux actually has a type field in the dirent structure,
+	     * but that is not portable - stat should be portable
+	     */
+	    status=stat(de->d_name, &statbuf);
+	    if ((status!=-1) && (S_ISDIR(statbuf.st_mode))) {
+		sprintf(buf,"%s/%s", path, de->d_name);
+		remove_directory(buf);
+		continue;
+	    }
 	    sprintf(buf,"%s/%s", path, de->d_name);
 	    if (unlink(buf)) {
 		LOG(llevError,"Unable to remove directory %s\n", path);
@@ -137,7 +139,7 @@ void remove_directory(const char *path)
 	}
 	closedir(dirp);
     }
-    if (unlink(path)) {
+    if (rmdir(path)) {
 	LOG(llevError,"Unable to remove directory %s\n", path);
     }
 }
@@ -283,7 +285,7 @@ long strtol_local(str, ptr, base)
 
 /* This seems to be lacking on some system */
 #if !defined(HAVE_STRNCASECMP)
-int strncasecmp(char *s1, char *s2, int n)
+int strncasecmp(const char *s1, const char *s2, int n)
 {
   register int c1, c2;
 
@@ -303,7 +305,7 @@ int strncasecmp(char *s1, char *s2, int n)
 #endif
 
 #if !defined(HAVE_STRCASECMP)
-int strcasecmp(char *s1, char*s2)
+int strcasecmp(const char *s1, const char*s2)
 {
   register int c1, c2;
 
@@ -321,6 +323,41 @@ int strcasecmp(char *s1, char*s2)
 }
 #endif
 
+char *strcasestr_local(const char *s, const char *find)
+{
+    char c, sc;
+    size_t len;
+
+    if ((c = *find++) != 0) {
+		c = tolower(c);
+        len = strlen(find);
+        do {
+            do {
+                 if ((sc = *s++) == 0)
+                     return NULL;
+            } while (tolower(sc) != c);
+        } while (strncasecmp(s, find, len) != 0);
+        s--;
+     }
+     return (char *)s;
+}
+
+#if !defined(HAVE_SNPRINTF)
+
+int snprintf(char *dest, int max, const char *format, ...)
+{
+    va_list var;
+    int ret;
+
+    va_start(var, format);
+    ret = vsprintf(dest, format, var);
+    va_end(var);
+    if (ret > max) abort();
+
+    return ret;
+}
+#endif
+
 
 /* This takes an err number and returns a string with a description of
  * the error.
@@ -383,6 +420,11 @@ char *ltostr10(signed long n) {
     *(--cp)='-';
   return cp;
 }
+char *doubletostr10(double v){
+  static char tbuf[200];
+  sprintf(tbuf,"%f",v);
+  return tbuf;
+}
 
 /*
  * A fast routine which appends the name and decimal number specified
@@ -390,25 +432,33 @@ char *ltostr10(signed long n) {
  * Could be faster, though, if the strcat()s at the end could be changed
  * into alternate strcat which returned a pointer to the _end_, not the
  * start!
+ *
+ * Hey good news, it IS faster now, according to changes in get_ob_diff
+ * Completly redone prototype and made define in loader.l. See changes there.
+ * Didn't touch those for speed reason (don't use them anymore) .
+ *                                                             Tchize
  */
 
 void save_long(char *buf, char *name, long n) {
-#if 0 /* This doesn't work, since buf is always the beginning */
-  char *cp, *var;
-  for(cp=buf;*name!='\0';)
-    *cp++ = *name++;
-  *cp++=' ';
-  for(var=ltostr10(n);*var!='\0';)
-    *cp++ = *name++;
-  *cp='\0';
+    char buf2[MAX_BUF];
+    strcpy(buf2,name);
+    strcat(buf2," ");
+    strcat(buf2,ltostr10(n));
+    strcat(buf2,"\n");
+    strcat(buf,buf2);
+}
+
+
+
+void save_long_long(char *buf, char *name, sint64 n) {
+    char buf2[MAX_BUF];
+
+#ifndef WIN32
+    sprintf(buf2,"%s %lld\n", name, n);
 #else
-  char buf2[MAX_BUF];
-  strcpy(buf2,name);
-  strcat(buf2," ");
-  strcat(buf2,ltostr10(n));
-  strcat(buf2,"\n");
-  strcat(buf,buf2);
+    sprintf(buf2,"%s %I64d\n", name, n);
 #endif
+    strcat(buf,buf2);
 }
 
 /* This is a list of the suffix, uncompress and compress functions.  Thus,
@@ -545,7 +595,7 @@ void make_path_to_file (char *filename)
 #endif
 	if (stat(buf, &statbuf) || !S_ISDIR (statbuf.st_mode)) {
 	    LOG(llevDebug, "Was not dir...");
-	    if (mkdir (buf, 0777)) {
+	    if (mkdir (buf, SAVE_DIR_MODE)) {
 		perror ("Couldn't make path to file");
 		return;
 	    }
