@@ -45,9 +45,13 @@ static recipe *get_empty_formula() {
   t->index = 0;
   t->transmute = 0;
   t->yield=0;
+  t->diff=0;
+  t->exp=0;
   t->keycode = 0;
   t->title = NULL;
   t->arch_name = NULL;
+  t->skill = NULL;
+  t->cauldron = NULL;
   t->ingred = NULL;
   t->next=NULL;
   return t;
@@ -125,20 +129,19 @@ void init_formulae() {
     if (!strncmp(cp, "Object", 6)) {
       formula=get_empty_formula();
       formula->title = add_string(strchr(cp,' ') + 1);
-    }
-    else if (!strncmp(cp, "keycode", 7)) {
+    } else if (!strncmp(cp, "keycode", 7)) {
       formula->keycode = add_string(strchr(cp,' ') + 1);
-    }
-    else if (sscanf(cp, "trans %d", &value)) {
-        formula->transmute = (uint16) value;
-    }
-    else if (sscanf(cp, "yield %d", &value)) {
-	formula->yield = (uint16) value;
-    }
-    else if (sscanf(cp, "chance %d", &value)) {
-        formula->chance = (uint16) value;
-    }
-    else if (!strncmp(cp, "ingred",6)) {
+    } else if (sscanf(cp, "trans %d", &value)) {
+        formula->transmute = (uint16)value;
+    } else if (sscanf(cp, "yield %d", &value)) {
+	formula->yield = (uint16)value;
+    } else if (sscanf(cp, "chance %d", &value)) {
+        formula->chance = (uint16)value;
+    } else if (sscanf(cp, "exp %d", &value)) {
+	formula->exp = (uint16)value;
+    } else if (sscanf(cp, "diff %d", &value)) {
+	formula->diff = (uint16)value;
+    } else if (!strncmp(cp, "ingred",6)) {
       int numb_ingred = 1;
       cp = strchr(cp,' ') + 1;
       do {
@@ -169,6 +172,10 @@ void init_formulae() {
     } else if (!strncmp(cp, "arch",4)) { 
         formula->arch_name = add_string(strchr(cp,' ')+1);
         (void) check_recipe(formula);
+    } else if (!strncmp(cp, "skill", 5)) {
+	formula->skill = add_string(strchr(cp, ' ')+1);
+    } else if (!strncmp(cp, "cauldron", 8)) {
+	formula->cauldron = add_string(strchr(cp, ' ')+1);
     } else
         LOG(llevError,"Unknown input in file %s: %s\n", filename, buf);
   }
@@ -247,7 +254,13 @@ void dump_alchemy( void ) {
 		}
                 fprintf(logfile,"\n");
 		if(tval!=formula->index) fprintf(logfile, "WARNING:ingredient list and formula values not equal.\n");
-             }  
+             }
+	     if (formula->skill != NULL)
+		 fprintf(logfile, "\tSkill Required: %s", formula->skill);
+	     if (formula->cauldron != NULL)
+		 fprintf(logfile, "\tCauldron: %s\n", formula->cauldron);
+	     fprintf(logfile, "\tDifficulty: %d\t Exp: %d\n", formula->diff,
+		 formula->exp);
 	  }
 	} else 
 	   LOG(llevError,"Can't find archetype:%s for formula %s\n", string,
@@ -482,15 +495,6 @@ void dump_alchemy_costs (void)
 	     num_errors);
 }
 
-#if 0
-recipe *get_first_recipe() {
-  recipelist *fl=formulalist;
-  recipe *formula=fl->items;
-
-  return formula;
-}
-#endif
-
 char * ingred_name (char *name) {
   char *cp=name;
  
@@ -591,8 +595,14 @@ void free_all_recipes()
 	for (formula=fl->items; formula!=NULL; formula=next) {
 	    next=formula->next;
       
-	    if (formula->arch_name) free_string(formula->arch_name);
-	    if (formula->title) free_string(formula->title);
+	    if (formula->arch_name)
+		free_string(formula->arch_name);
+	    if (formula->title)
+		free_string(formula->title);
+	    if (formula->skill)
+		free_string(formula->skill);
+	    if (formula->cauldron)
+		free_string(formula->cauldron);
 	    for (lchar=formula->ingred; lchar; lchar=charnext) {
 		charnext=lchar->next;
 		free_string(lchar->name);
