@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_xutil_c =
- *   "$Id: xutil.c,v 1.8 2001/12/16 03:31:03 mwedel Exp $";
+ *   "$Id: xutil.c,v 1.11 2002/07/15 04:25:40 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 
@@ -62,6 +62,8 @@ XChar2b fontindex_to_XChar2b(Fontindex s)
 }
 
 
+#define IMAGE_BUF 65536
+
 /*
  * ReadPixmaps(): When color pixmaps are used instead of fonts, this function
  * does the actual reading of pixmap-file.  This function is based largely on
@@ -89,7 +91,7 @@ int ReadImages(Display *gdisp, Pixmap **pixmaps, Pixmap **masks,
     Window	root = RootWindow (gdisp,DefaultScreen(gdisp));
     int		use_private_cmap=0,num, compressed, len,i;
     FILE	*infile;
-    char	*cp, databuf[HUGE_BUF], filename[MAX_BUF];
+    char	*cp, databuf[IMAGE_BUF], filename[MAX_BUF];
 #ifdef IMAGE_TIME_LOAD
     time_t	start_time = time(NULL);
 #endif
@@ -119,7 +121,7 @@ int ReadImages(Display *gdisp, Pixmap **pixmaps, Pixmap **masks,
     LOG(llevDebug,"Building images...");
 
     if (type==Dm_Png) {
-	sprintf(filename,"%s/crossfire.png",settings.datadir);
+	sprintf(filename,"%s/crossfire.0",settings.datadir);
 #ifdef HAVE_LIBPNG
 	init_pngx_loader(gdisp);
 #endif
@@ -145,7 +147,7 @@ int ReadImages(Display *gdisp, Pixmap **pixmaps, Pixmap **masks,
 	/* Skip accross the number data */
 	for (cp=databuf+6; *cp!=' '; cp++) ;
 	len = atoi(cp);
-	if (len==0 || len>HUGE_BUF) {
+	if (len==0 || len>IMAGE_BUF) {
 	    LOG(llevError,"ReadImages: length not valid: %d\n%s",
                     len,databuf);
                 abort();
