@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_config_h =
- *   "$Id: config.h,v 1.30 2002/09/11 06:21:46 mwedel Exp $";
+ *   "$Id: config.h,v 1.58 2004/02/17 00:35:53 mwedel Exp $";
  */
 
 /*
@@ -44,13 +44,6 @@
  * flags  However, for some things, that would just be too messy.
  */
 
-/* To be removed soon (setable by player) */
-/* IF this is set, then the range type will not switch to skill when
- * you use a melee weapon - this better emulates pre-skill code.
- */
-
-#define NO_AUTO_SKILL_SWITCH
-
 /* There are 4 main sections to this file-
  * Section 1 is feature selection (enabling/disabling certain features)
  *
@@ -74,44 +67,16 @@
  * alphabetically.
  *
  * Short list of features, and what to search for:
- * ALCHEMY - enables alchemy code
- * BALANCED_STAT_LOSS - Based death stat depletion on level etc?
- * CASTING_TIME - makes spells take time to cast
  * CS_LOGSTATS - log various new client/server data.
  * DEBUG - more verbose message logging?
- * EXPLORE_MODE - add an explore mode method of play?
- * FULL_RING_DESCRIPTION - makes rings abilities show in inventory window
  * MAP_CLIENT_X, MAP_CLIENT_Y - determines max size client map will receive
  * MAX_TIME - how long an internal tick is in microseconds
  * MANY_CORES - generate core dumps on gross errors instead of continuing?
- * MULTIPLE_GODS - adds numerous gods to the game, with different powers
- * NO_CONE_PROPOGATE - makes cone spells stop at the first monster
- * NOT_PERMADEATH - should death be the final end or not?
  * PARTY_KILL_LOG - stores party kill information
- * REAL_WIZ - changes wiz stuff to be more mudlike.
- * RECYCLE_TMP_MAPS - use tmp maps across multiple runs?
- * RESURRECTION - lets players bring other dead players back to life.
- * SEARCH_ITEMS - let players search for items on the ground
- * SECURE - Allow overriding values with run time flags?
- * SET_TITLE - lets players set their title
- * SIMPLE_EXP - uses simpler experience system
- * SPELL_* - various spell related options
- * STAT_LOSS_ON_DEATH - toggle between stat loss or stat depletion
- * USE_LIGHTING - enable light/darkness & light sources
- * USE_PERMANENT_EXPERIENCE - allow players to accumulate permanent experience?
- * USE_SWAP_STATS - allows stat swapping for new characters
  * WATCHDOG - allows use of an external watchdog program
  *
  ***********************************************************************/
 
-/* Enables Brian Thomas's new alchemy code.  Might unbalance the game - lets
- * characters make potions and other special items.
- * 0.94.2 note - I think enough work has been done, and even some maps
- * changed that this really should probably be a standard feature now.
- */
-
-#define ALCHEMY
-
 /* Use balanced stat loss code?
  * This code is a little more merciful with repeated stat loss at lower
  * levels. Basically, the more stats you have lost, the less likely that
@@ -144,24 +109,33 @@
  */
 /* GD */
 
-#define BALANCED_STAT_LOSS FALSE
 #define BALSL_LOSS_CHANCE_RATIO    4
 #define BALSL_NUMBER_LOSSES_RATIO  6
 #define BALSL_MAX_LOSS_RATIO       2
 
-/* casting times for spells, if you have this defined then it takes a
- * specific amount of time for a spell to go off. You may attack or
- * be hit during this time, and this will cause you to lose the spell.
- * I commented this out on my copy - I don't like the 'double clutch'
- * needed to cast it (first direction key starts casting, the
- * next actually puts the spell into effect.)  It could perhaps
- * be changed so that the initial direction is where it goes, and it
- * is cast automatically in that direction (so only 1 key is needed.)
- * But this could be undesirable as various things move while you cast it.
- */
-/*
-#define CASTING_TIME
-*/
+
+/* Don't edit these values.  They are configured in lib/settings.  These are
+   Simply the defaults. */
+
+#define BALANCED_STAT_LOSS FALSE
+#define USE_PERMANENT_EXPERIENCE FALSE
+#define SET_TITLE TRUE
+#define SIMPLE_EXP TRUE
+#define SPELLPOINT_LEVEL_DEPEND TRUE
+#define SPELL_ENCUMBRANCE TRUE
+#define SPELL_FAILURE_EFFECTS FALSE
+#define REAL_WIZ TRUE
+#define RECYCLE_TMP_MAPS FALSE
+#define RESURRECTION FALSE
+#define SEARCH_ITEMS TRUE
+#define NOT_PERMADETH TRUE
+#define EXPLORE_MODE FALSE
+#define STAT_LOSS_ON_DEATH FALSE
+#define CASTING_TIME FALSE
+#define SET_FRIENDLY_FIRE 5
+
+/* you can edit the ones below */
+
 
 /* CS_LOGSTATS will cause the server to log various usage stats
  * (number of connections, amount of data sent, amount of data received,
@@ -187,43 +161,6 @@
 #endif
 #endif
 
-
-/* if EXPLORE_MODE is defined, it allows players to enter explore mode,
- * where they can not die.  Unlike other games (nethack for instance) explore
- * modes, the only thing Crossfire explore mode does is prevent death.
- * Characters in explore mode will not be able to go on the scoreboard.  Also,
- * once explore mode is entered, there is no return.  Explore mode can
- * only be entered if the person doing so is the only player in the
- * game, and once done, that person can not add new players to the
- * game.  To get into explore mode, type 'explore
- * Explore mode code added by Mark Wedel (mark@pyramid.com)
- * It's a good idea to turn off EXPLORE_MODE if you will only be running
- * on server, since it would block new players.
- */
-/*
-#define EXPLORE_MODE
-*/
-
-
-/* FULL_RING_DESCRIPTION has crossfire prints out the abilities of
- * rings or amulets in the appropriate windows.  So instead of just
- * have 'ring' get printed in your inventory, ring (Cha+1) or
- * amulet (reflect missiled), etc, will be printed out.  This also applies
- * to items laying on the ground.
- * No more information will be gathered than if you left click the
- * item (so, if the item is not identified, you will see nothing
- * except the item name.)  Tastes may vary on using this option.  It
- * can make the name excessively long, especially for unpaid items,
- * causing the (unpaid) to be cut off.  On the other hand, it can save
- * a lot of time, especially if you have several rings, or are looking
- * for one in a shop with lots of rings.
- * 0.94.2 - this will likely be removed in the not too distant future - 
- * client should really deal/emulate the short names if that is desired.
- */
-
-#define FULL_RING_DESCRIPTION
-
-
 /*
  * This option creates more core files.  In some areas, there are certain
  * checks done to try and make the program more stable (ie, check
@@ -269,37 +206,12 @@
 
 #define MAX_TIME	120000
 
-/* This is a modification to the cone spells.  If a cone spell hits
- * a monster, the spell stops progressing (same is true for parties
- * of players).  This weakens the cones spells a bit.  If it is not
- * define, you get the old behaviour (cone spells cover every square
- * up to their range in a 90 degree arc.
- */
-/*
-#define NO_CONE_PROPOGATE
-*/
-
 /* Polymorph as it currently stands is unbalancing, so by default
  * we have it disabled.  It can be enabled and it works, but
  * it can be abused in various ways.
  */
 #define NO_POLYMORPH
 
-/* NOT_PERMADEATH by Charles Henrich (henrich@crh.cl.msu.edu), April 9, 1993
- *
- * Makes death non permanent.  If enabled and you die, you lose a bunch of
- * exp, a random stat, and go back to starting town.
- * If not defined, if you die, you are dead.  If an a multiplayer server,
- * resurrection may be possible
- *
- * This option changes the game significantly one way or the other - it makes
- * adventuring right at the edge of death something more reasonable to do
- * (death still hurts here).  On the other hand, it certainly makes the
- * game a bit safer and easier.
- */
-
-#define NOT_PERMADEATH
-
 
 /* This determine how many entries are stored in the kill log.  You
  * can see this information with the 'party kills' command.  More entries
@@ -310,151 +222,6 @@
 #define PARTY_KILL_LOG 20
 */
 
-/* Calling this REAL_WIZ is probably not really good.  Something like
- * MUD_WIZ might be a better name.
- *
- * Basically, if REAL_WIZ is define then the WIZ/WAS_WIZ flags for objects
- * are not set - instead, wizard created/manipulated objects appear as
- * normal objects.  This makes the wizard a little more mudlike, since
- * object manipulations will be usable for other objects.
- */
-
-#define REAL_WIZ
-
-
-/*
- * Set this if you want the temporary maps to be saved and reused across
- * crossfire runs.  This can be especially useful for single player
- * servers, but even holds use for multiplayer servers.  The file used
- * is updated each time a temp map is updated.
- * Note that the file used to store this information is stored in
- * the LIB directory.  Running multiple crossfires with the same LIB
- * directory will cause serious problems, simply because in order for
- * this to really work, the filename must be constant so the next run
- * knows where to find the information.
- */
-/*
-#define RECYCLE_TMP_MAPS
-*/
-
-/* define RESURECTION if you want to let players bring other players
- * back to life via some spells.  If RESURRECTION is undefined, then a
- * death is permanent.  NOTE: RESURRECTION (either defined or undefined)
- * has no meaning if NOT_PERMADEATH is defined.  SAVE_PLAYER also needs
- * to be defined for this to have any use
- */
-
-#define RESURRECTION
-
-
-/*
- * Enables the 'search-item command; a method to find equipment
- * in shops. 
- * This is somewhat useful is FULL_RING_DESCRIPTION is enabled (above.)
- * It matches by substrings of the name of the item as it appears in
- * the look window.  Therefor, if FULL_RING_DESCRIPTION is not enabled,
- * you can never match by ring abilities, as these are then not
- * displayed in the look window.
- * Seems like it now works, and doesn't cause the game to hang.
- * 0.94.2 - I think this really should be a client issue - after all, the
- * client has all the info the player does.
- */
-
-#define SEARCH_ITEMS
-
-
-/*
- * If SECURE is defined, crossfire will not accept enviromental variables
- * as changes to the LIBDIR and other defines pointing to specific
- * directories.  The only time this should really be an issue if you are
- * running crossfire setuid/setgid and someone else could change the
- * values and get special priveledges.  If you are running with normal uid
- * privelidges, then this is not any more/less secure (user could compile
- * there own version pointing wherever they want.)  However, having nonsecure
- * can make debugging or other developement much easier, because you can
- * change the paths without forcing a recompile.
- *
- * NOTE: Prior to 0.93.2, the default was for this to be defined (ie,
- * could not change values on the command line/through environmental
- * variables.)
- */
-/*
-#define SECURE
-*/
-
-/*
- * Define the following if you wish to allow players to set their title
- * and to save it on the disk
- * 0.94.2 - this will probably become a default option sometime unless
- * someone has a good reason why it shouldn't
- */
-
-#define SET_TITLE
-
-/*
- * Set to true if you want to use the new (0.95.3) experience system.
- * The system has the following changes:
- *
- * 1) Your stats in your skill categories don't affect the amount of
- *  exp you get.
- * 2) Your level difference doesn't affect the exp you get for killing
- *  monsters.
- *
- * If this is set to FALSE out, you get the old system.
- *
- * This can also be set by using -simple-exp/+simple-exp as a run
- * time option, or update the settings file
- */
-
-#define SIMPLE_EXP TRUE
-
-/*  SPELLPOINT_LEVEL_DEPEND  --  Causes the spellpoint cost
- *  of spells to vary with their power.  Spells that become very
- *  powerful at high level cost more.  The damage/time of
- *  characters increases though.
- */
-
-#define SPELLPOINT_LEVEL_DEPEND
-
-
-/* SPELL_ENCUMBRANCE -- If you're carrying a weapon or wearing heavy armour,
-  you have a chance of fumbling your spellcasting with this on.  Formula is:
-  encumbrance = 3*weapon_weight + armour_weight
-
-  failure if roll from 1-200 is less than encumbrance + sp->level -
-  op->level - 35
-*/
-
-#define SPELL_ENCUMBRANCE
-
-/* SPELL_FAILURE_EFFECTS only has meaing if SPELL_ENCUMBRANCE is defined.
- * What it does, is that when the player fails a spell, various effects
- * will happen (player is paralyzed, confused, wonder spell is cast, etc.)
- * I disabled it because I think it would make life much too hazardous
- * for low level casters.  They either need to wear light armor (which
- * means that they will get pounded on by monsters), or will get
- * confused/paralyzed/other effects often.  High level casters would
- * be mostly unaffected, since they would be casting spells that are
- * below their level.
- * Note-  it seems that you still get some failur effects even with this
- * not defined - most notably when reading scrolls and fail to read
- * them properly.
- */
-
-/* #define SPELL_FAILURE_EFFECTS */
-
-
-/* Set this to FALSE if you don't want characters to loose a random stat when
- * they die - instead, they just get deplete.
- * Setting it to TRUE keeps the old behaviour.  This can be
- * changed at run time via -stat_loss_on_death or +stat_loss_on_death.
- * In theory, this can be changed on a running server, but so glue code
- * in the wiz stuff would need to be added for that to happen.
- */
-
-#define STAT_LOSS_ON_DEATH FALSE
-
-
 /* Use permanent experience code?
  * This code allows players to build up a small amount of 'permanent
  * experience' which reduces the effect of large experience drains, such as
@@ -486,26 +253,10 @@
  */
 /* GD */
 
-#define USE_PERMANENT_EXPERIENCE FALSE
 #define PERM_EXP_MINIMUM_RATIO        0.25f
 #define PERM_EXP_GAIN_RATIO           0.10f
 #define PERM_EXP_MAX_LOSS_RATIO       0.50f
 
-/*
- * SWAP_STATS is fixed now - rgg.
- *
- * Swap stats allows players to swap the values for their stats around
- * in the "Roll again" phase of the character selection.  This should
- * hopefully make character selection quicker, allowing for quicker
- * game start ups. 
- * Bugs:    c.blackwood@rdt.monash.edu.au
- * SORT_ROLLED_STATS - only useful with USE_SWAP_STATS - will order
- * stats from lowest to highest on initial roll.
- */ 
-
-#define	USE_SWAP_STATS
-#define SORT_ROLLED_STATS
-
 /*
  * WATCHDOG lets sends datagrams to port 13325 on localhost
  * in (more-or-less) regular intervals, so an external watchdog
@@ -517,6 +268,17 @@
 #define WATCHDOG
 #endif
 
+
+/* Enable the new material code - it needs some work.  You can
+ * enable this, and things will work, just you'll see a whole
+ * bunch more materials show up, and thus a whole bunch more materials
+ * in your inventory, and the sorting for them isn't really good.
+ */
+
+/*
+#define NEW_MATERIAL_CODE
+*/
+
 /***********************************************************************
  * SECTION 2 - Machine/Compiler specific stuff.
  *
@@ -567,9 +329,6 @@
  * BANFILE - ban certain users/hosts.
  * CSPORT - port to use for new client/server
  * DMFILE - file with dm/wizard access lists
- * DM_MAIL - address of server administrator
- * DUMP_SWITCHES - enable -m? flags for spoiler generation
- * LIBDIR - location of archetypes & other data.
  * LOGFILE - where to log if using -daemon option
  * MAP_ - various map timeout and swapping parameters
  * MAX_OBJECTS - how many objects to keep in memory.
@@ -594,7 +353,6 @@
 #define BANFILE         "ban_file"
 #endif
 
-
 /* CSPORT is the port used for the new client/server code.  Change
  * if desired.  Only of relevance if ERIC_SERVER is set above
  */
@@ -613,26 +371,6 @@
 #endif
 
 
-/*
- * Where to send error-reports.  If you're hacking at the source, define
- * this as your mail-address.  You do not need to define this - you can
- * leave it undefined if you don't want the information available from this
- * and instead use some other mean (motd for example)
- */
-
-#ifndef DM_MAIL
-/* #define DM_MAIL "mark@pyramid.com" */
-#endif
-
-/*
- * If you want to regenerate the spoiler.ps file, you have to recompile
- * the game with DUMP_SWITCHES defined.  If defined, it turns on the
- * -m -m2 -m3 -m4 switches.  There is probably no reason not to define
- * this
- */
-
-#define DUMP_SWITCHES
-
 /* LOGFILE specifies which file to log to when playing with the
  * -daemon option.
  */
@@ -673,10 +411,6 @@
 #define MAP_MINTIMEOUT  500
 
 /*
- * MAP_RESET tells whether map is reset after some time.  If it is defined,
- * the game uses weight variable of map object to tell, after how many seconds
- * the map will be reset.  If MAP_RESET is undefined, maps will never reset.
- *
  * MAP_MAXRESET is the maximum time a map can have before being reset.  It
  * will override the time value set in the map, if that time is longer than
  * MAP_MAXRESET.  This value is in seconds.  If you are low on space on the
@@ -689,7 +423,6 @@
  * in the map archetype.
  */
 
-#define MAP_RESET
 #define MAP_MAXRESET	7200
 
 /*
@@ -706,7 +439,7 @@
  * 25000 is about 8.5 MB
  */
 
-#define MAX_OBJECTS	25000
+#define MAX_OBJECTS	100000
 
 /*
  * Max objects low water mark (lwm).  If defined, the map swapping strategy
@@ -757,9 +490,7 @@
  * (It resides in the CONFDIR directory)
  */
 
-#ifndef MOTD
 #define MOTD "motd"
-#endif
 
 /*
  * You can restrict playing in certain times by creating a PERMIT_FILE
@@ -840,6 +571,7 @@
  */
 
 #ifdef WIN32 /* change define path city */
+
 #  define EMERGENCY_MAPPATH "\\city\\city"
 #  define EMERGENCY_X 15
 #  define EMERGENCY_Y 19
@@ -859,13 +591,13 @@
 #define ARCHETYPES	"archetypes"
 #define HIGHSCORE	"highscore"
 #define TREASURES	"treasures"
-
+#define	BANISHFILE  "banish_file"
 
 #define MAX_ERRORS	25	/* Bail out if more are received during tick */
 #define STARTMAX        500     /* How big array of objects to start with */
 #define OBJ_EXPAND      100     /* How big steps to use when expanding array */
 
-#define HIGHSCORE_LENGTH 10	/* How many entries there are room for */
+#define HIGHSCORE_LENGTH 1000	/* How many entries there are room for */
 
 #define ARCHTABLE 5003		/* Used when hashing archetypes */
 #define MAXSTRING 20
@@ -891,20 +623,8 @@
  * If you intend to run a central server, and not allow the players to
  * start their own crossfire, you won't need to define this.
  *
- * If USE_CHECKSUM is defined, a checksum will be calculated each time
- * a player saves a character.
- *
- * If ENABLE_CHECKSUM is defined, the players will get the cheat-flag set
- * if the checksums don't match.  If your player-files don't have checksums,
- * you'll want to leave "USE_CHECKSUM" defined for a while so they all
- * get checksums, and then define "ENABLE_CHECKSUM" to utilize this.
- * ENABLE_CHECKSUM should not be needed if you are on a central server and
- * players don't have access to the save files.
  */
 
-#define USE_CHECKSUM
-/* #define ENABLE_CHECKSUM */ /* Will be default in distant future versions */
-
 #ifndef PLAYERDIR
 #define PLAYERDIR "players"
 #endif
@@ -912,14 +632,16 @@
 /*
  * If you have defined SAVE_PLAYER, you might want to change this, too.
  * This is the access rights for the players savefiles.
- * I think it is usefull to restrict access to the savefiles for the
- * game admin. So if you make crossfire set-uid, use 0600.
- * If you are running the game set-gid (to a games-group, for instance),
- * you must remember to make it writeable for the group (ie 0660).
- * Kjetil W. J{\o}rgensen, jorgens@pvv.unit.no
- * (Note: something should probably be done with lock-file permission)
+ * Given that crossfire runs in a client/server model, there should
+ * be no issue setting these to be quite restrictive (600 and 700).
+ * Before client/server, multiple people might run the executable,
+ * thus requiring that the server be setuid/setgid, and more generous
+ * permisisons needed.
+ * SAVE_MODE is permissions for the files, SAVE_DIR_MODE is permission
+ * for nay directories created.
  */
 #define	SAVE_MODE	0660
+#define	SAVE_DIR_MODE	0770
 
 /* NOTE ON SAVE_INTERVAL and AUTOSAVE:  Only one of these two really
  * needs to be selected.  You can set both, and things will work fine,
@@ -981,7 +703,9 @@
  * on how the player exited the game.  But if the elapsed time is greater than
  * the value below, player will always get returned to savebed location
  * location.
+ * 
+ * Set to one hour as default
  */
 
-#define RESET_LOCATION_TIME	0
+#define RESET_LOCATION_TIME	3600
 
