@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_face_h =
- *   "$Id: face.h,v 1.2 1999/01/30 11:20:43 master Exp $";
+ *   "$Id: face.h,v 1.3 2001/12/03 07:51:40 mwedel Exp $";
  */
 
 /*
@@ -32,12 +32,11 @@
  * create a new face with that color.
  */
 typedef struct new_face_struct {
-    uint16	number;		/* This is the image id.  IT should be the */
+    uint16	number;		/* This is the image id.  It should be the */
 				/* same value as its position in the array */
-    uint8	fg, bg;
     char	*name;
     uint8	visibility;
-    uint8	magicmap;
+    uint8	magicmap;	/* Color to show this in magic map */
 } New_Face;
 
 typedef struct map_look_struct {
