@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_funcpoint_h =
- *   "$Id: funcpoint.h,v 1.6 2002/03/01 21:33:10 avogl Exp $";
+ *   "$Id: funcpoint.h,v 1.11 2003/10/27 03:44:33 temitchell Exp $";
  */
 
 /*
@@ -51,6 +51,8 @@ typedef void (*type_func_char_int)(char *, int);
 typedef void (*type_func_int_ob_ob)(int, object *, object *);
 typedef void (*type_func_player_int)(player *, int);
 typedef void (*type_func_dragon_gain)(object *who, int atnr, int level);
+typedef void (*type_func_char)(char *);
+typedef object* (*type_ob_func_ob_int)(object *, int);
 
 /*
  * These function-pointers are defined in common/glue.c
@@ -58,7 +60,6 @@ typedef void (*type_func_dragon_gain)(object *who, int atnr, int level);
  */
 
 extern void	(*move_apply_func)		(object *, object *, object *);
-extern void	(*draw_func)			(object *);
 extern void	(*draw_info_func)		(int, int, object *, const char *);
 extern void	(*emergency_save_func)		(int);
 extern void	(*fix_auto_apply_func)		(mapstruct *);
@@ -71,11 +72,14 @@ extern void	(*info_map_func)		(int, mapstruct *, char *);
 extern void	(*move_teleporter_func)		(object *);
 extern void	(*move_firewall_func)		(object *);
 extern void	(*move_creator_func)		(object *);
+extern void  (*move_trigger_marker_func)	(object *);
+extern void	(*move_duplicator_func)		(object *);
 extern void     (*trap_adjust_func)		(object *, int);
 extern void	(*esrv_send_item_func)		(object *, object *);
 extern void	(*esrv_del_item_func)		(player *, int);
 extern void	(*esrv_update_item_func)	(int, object *, object *);
 extern void	(*set_darkness_map_func)	(mapstruct *m);
 extern void     (*dragon_gain_func)             (object *, int, int);
-
+extern void	(*weather_effect_func)		(char *);
+extern object *	(*find_skill_by_number_func)	(object *, int);
 #endif
