@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_global_h =
- *   "$Id: global.h,v 1.30 2002/09/07 22:08:07 mwedel Exp $";
+ *   "$Id: global.h,v 1.47 2004/04/25 07:17:38 mwedel Exp $";
  */
 
 /*
@@ -50,26 +50,50 @@
  * The signedness for char is probably not universal, and using char
  * will probably be more portable than sint8/unit8
  */
+
  
 typedef unsigned int	uint32;
+#define UINT32_MAX	4294967295U
 typedef signed int	sint32;
+#define SINT32_MAX	2147483647
 typedef unsigned short	uint16;
+#define UINT16_MAX	65535
 typedef signed short	sint16;
+#define SINT16_MAX	32767
 typedef unsigned char	uint8;
+#define UINT8_MAX	255
 typedef signed char	sint8;
+#define SINT8_MAX	127
+
+
 typedef unsigned short Fontindex;
 
+#ifdef WIN32
+/* Python plugin stuff defines SIZEOF_LONG_LONG as 8, and besides __int64 is a 64b type on MSVC...
+ * So let's force the typedef */
+typedef unsigned __int64	uint64;
+typedef signed __int64		sint64;
+/* Needed for experience */
+#define atoll	_atoi64
 
+/* To reduce number of warnings */
+#pragma warning( disable: 4244 ) /* conversion from 'xxx' to 'yyy', possible loss of data */
+#pragma warning( disable: 4305 ) /* initializing float f = 0.05; instead of f = 0.05f; */
 
-/* global stuff used by new skill/experience system -b.t.
- * Needed before player.h
- */
-#define MAX_EXP_CAT 7 		/* This should be => # of exp obj in the game 
-				 * remember to include the "NULL" exp object  
-			         * EXP_NONE as part of the overall tally. 
-				 */
-#define EXP_NONE (MAX_EXP_CAT - 1)  /* "NULL" exp. object. This is the last 
-				     * experience obj always.*/ 
+#else // WIN32
+
+#if SIZEOF_LONG == 8
+typedef unsigned long	    uint64;
+typedef signed long	    sint64;
+#elif SIZEOF_LONG_LONG == 8
+typedef unsigned long long	uint64;
+typedef signed long long 	sint64;
+#else
+error do not know how to get a 64 bit value on this system.
+error correct and send mail to crossfire-devel on how to do this
+#endif
+
+#endif
 
 /* This blob, in this order, is needed to actually define maps */
 #include "face.h"
@@ -81,6 +105,8 @@ typedef unsigned short Fontindex;
 #include "map.h"
 #include "tod.h"
 
+#include "skills.h"
+
 /* Pull in the socket structure - used in the player structure */
 #include "newserver.h"
 
@@ -92,6 +118,7 @@ typedef unsigned short Fontindex;
 
 #include "commands.h"
 
+
 /*
  * So far only used when dealing with artifacts.
  * (now used by alchemy and other code too. Nov 95 b.t).
@@ -171,9 +198,6 @@ EXTERN long nroftreasures;		/* Only used in malloc_info() */
 EXTERN long nrofartifacts;		/* Only used in malloc_info() */
 EXTERN long nrofallowedstr;		/* Only used in malloc_info() */
 
-EXTERN short nrofexpcat;	/* Current number of experience categories in the game */
-EXTERN object *exp_cat[MAX_EXP_CAT];	/* Array of experience objects in the game */ 
-
 EXTERN archetype *empty_archetype;	/* Nice to have fast access to it */
 EXTERN archetype *map_archeytpe;
 EXTERN char first_map_path[MAX_BUF];	/* The start-level */
@@ -204,8 +228,10 @@ EXTERN int num_animations,animations_allocated, bmaps_checksum;
 #define NUM_ANIMATIONS(ob) (animations[ob->animation_id].num_animations)
 #define NUM_FACINGS(ob) (animations[ob->animation_id].facings)
 
-extern int freearr_x[SIZEOFFREE], freearr_y[SIZEOFFREE];
+extern short freearr_x[SIZEOFFREE], freearr_y[SIZEOFFREE];
 extern int maxfree[SIZEOFFREE], freedir[SIZEOFFREE];
+extern int rightof_x[9], rightof_y[9];
+extern int leftof_x[9], leftof_y[9];
 
 extern New_Face *blank_face, *empty_face;
 extern New_Face *dark_faces[];
@@ -293,8 +319,22 @@ typedef struct Settings {
     uint8   stat_loss_on_death;	/* If true, chars lose a random stat when they die */
     uint8   use_permanent_experience; /* If true, players can gain perm exp */
     uint8   balanced_stat_loss; /* If true, Death stat depletion based on level etc */
+    uint8   not_permadeth;  /* if true, death is non-permament */
     uint8   simple_exp;	    /* If true, use the simple experience system */
     int	    reset_loc_time; /* Number of seconds to put player back at home */
+    uint8   set_title;	    /* players can set thier title */
+    uint8   resurrection;   /* ressurection possible w/ permadeth on */
+    uint8   search_items;   /* search_items command */
+    uint8   spell_encumbrance; /* encumbrance effects spells */
+    uint8   spell_failure_effects; /* nasty backlash to spell failures */
+	uint16	set_friendly_fire;	/* Percent of damage done by peaceful player vs player damage */
+    uint8   casting_time;   /* it takes awhile to cast a spell */
+    uint8   real_wiz;       /* use mud-like wizards */
+    uint8   recycle_tmp_maps; /* re-use tmp maps. */
+    uint8   explore_mode;     /* explore mode allowed? */
+    uint8   spellpoint_level_depend; /* spell costs go up with level */
+    char    motd[MAX_BUF];    /* name of the motd file */
+    char    dm_mail[MAX_BUF]; /* DM's Email address */
 
     /* The meta_ is information for the metaserver.  These are set in 
      * the lib/settings file.
@@ -312,9 +352,13 @@ typedef struct Settings {
     uint32  worldmaptilesizex;	    /* number of squares wide in a wm tile */
     uint32  worldmaptilesizey;	    /* number of squares high in a wm tile */
     uint16  dynamiclevel;	    /* how dynamic is the world? */
+    uint8   fastclock;		    /* if true, clock goes warp 9 */
 
     char    *emergency_mapname;	    /* map to return players to in emergency */
     uint16  emergency_x, emergency_y;	/* coordinates to use on that map */
+
+    sint16  max_level;		    /* This is read out of exp_table */
+    float   item_power_factor;	    /* See note in setings file */
 } Settings;
 
 extern Settings settings;
