@@ -2,9 +2,11 @@
 extern void free_all_anim(void);
 extern void init_anim(void);
 extern int find_animation(char *name);
-extern void animate_object(object *op);
+extern void animate_object(object *op, int dir);
 /* arch.c */
 extern archetype *find_archetype_by_object_name(char *name);
+extern object *get_archetype_by_skill_name(char *skill, int type);
+extern archetype *get_archetype_by_type_subtype(int type, int subtype);
 extern object *get_archetype_by_object_name(char *name);
 extern object *find_best_weapon_used_match(object *pl, char *params);
 extern int item_matched_string(object *pl, object *op, char *name);
@@ -67,7 +69,6 @@ extern void set_process_active_maps(type_func_void addr);
 extern void set_update_buttons(type_func_map addr);
 extern void set_draw_info(type_func_int_int_ob_cchar addr);
 extern void set_move_apply(type_move_apply_func addr);
-extern void set_draw(type_func_ob addr);
 extern void set_monster_check_apply(type_func_ob_ob addr);
 extern void set_init_blocksview_players(type_func_void addr);
 extern void set_info_map(type_func_int_map_char addr);
@@ -75,11 +76,15 @@ extern void set_move_teleporter(type_func_ob addr);
 extern void set_move_firewall(type_func_ob addr);
 extern void set_trap_adjust(type_func_ob_int addr);
 extern void set_move_creator(type_func_ob addr);
+extern void set_move_trigger_marker(type_func_ob addr);
+extern void set_move_duplicator(type_func_ob addr);
 extern void set_esrv_send_item(type_func_ob_ob addr);
 extern void set_esrv_update_item(type_func_int_ob_ob addr);
 extern void set_esrv_del_item(type_func_player_int addr);
 extern void set_set_darkness_map(type_func_map addr);
 extern void set_dragon_gain_func(type_func_dragon_gain addr);
+extern void set_weather_effect_func(type_func_char addr);
+extern void set_find_skill_by_number_func(type_ob_func_ob_int addr);
 extern void fatal(int err);
 extern void dummy_function_int(int i);
 extern void dummy_function_int_int(int i, int j);
@@ -90,12 +95,14 @@ extern void dummy_function_ob(object *ob);
 extern void dummy_function_ob2(object *ob, object *ob2);
 extern int dummy_function_ob2int(object *ob, object *ob2);
 extern void dummy_function_ob_int(object *ob, int i);
+extern object *dummy_ob_function_ob_int(object *ob, int i);
 extern void dummy_function_txtnr(char *txt, int nr);
 extern void dummy_draw_info(int a, int b, object *ob, const char *txt);
 extern void dummy_function_mapstr(int a, mapstruct *map, char *str);
 extern void dummy_function_int_ob_ob(int n, object *ob, object *ob2);
 extern void dummy_move_apply_func(object *ob, object *ob2, object *ob3);
 extern void dummy_function_dragongain(object *ob, int a1, int a2);
+extern void dummy_function_char(char *filename);
 /* holy.c */
 extern void init_gods(void);
 extern void add_god_to_list(archetype *god_arch);
@@ -112,6 +119,9 @@ extern void bitstostring(long bits, int num, char *str);
 extern char find_color(char *name);
 extern int ReadBmapNames(void);
 extern int FindFace(char *name, int error);
+extern char *FindFaceName(int facenbr, char *error);
+extern int ReadSmooth(void);
+extern int FindSmooth(uint16 face, uint16 *smoothed);
 extern void free_all_images(void);
 /* init.c */
 extern void init_library(void);
@@ -137,7 +147,6 @@ extern char *query_base_name(object *op, int plural);
 extern char *describe_item(object *op, object *owner);
 extern int is_magical(object *op);
 extern int need_identify(object *op);
-extern int look_up_spell_name(char *spname);
 extern void identify(object *op);
 /* links.c */
 extern objectlink *get_objectlink(void);
@@ -145,9 +154,9 @@ extern oblinkpt *get_objectlinkpt(void);
 extern void free_objectlink(objectlink *ol);
 extern void free_objectlinkpt(oblinkpt *obp);
 /* living.c */
-extern void set_attr_value(living *stats, int attr, signed char value);
-extern void change_attr_value(living *stats, int attr, signed char value);
-extern signed char get_attr_value(living *stats, int attr);
+extern void set_attr_value(living *stats, int attr, sint8 value);
+extern void change_attr_value(living *stats, int attr, sint8 value);
+extern sint8 get_attr_value(living *stats, int attr);
 extern void check_stat_bounds(living *stats);
 extern int change_abil(object *op, object *tmp);
 extern void drain_stat(object *op);
@@ -157,15 +166,17 @@ extern void remove_statbonus(object *op);
 extern void add_statbonus(object *op);
 extern void fix_player(object *op);
 extern int allowed_class(object *op);
-extern uint32 level_exp(int level, double expmul);
-extern void add_exp(object *op, int exp);
 extern void set_dragon_name(object *pl, object *abil, object *skin);
 extern void dragon_level_gain(object *who);
+extern object *give_skill_by_name(object *op, char *skill_name);
 extern void player_lvl_adj(object *who, object *op);
+extern sint64 level_exp(int level, double expmul);
 extern void calc_perm_exp(object *op);
-extern int adjust_exp(object *op, int exp);
-extern int check_dm_add_exp_to_obj(object *exp_ob, int i);
+extern sint64 check_exp_loss(object *op, sint64 exp);
+extern sint64 check_exp_adjust(object *op, sint64 exp);
+extern void change_exp(object *op, sint64 exp, char *skill_name, int flag);
 extern void apply_death_exp_penalty(object *op);
+extern int did_make_save(object *op, int level, int bonus);
 /* logger.c */
 extern void LOG(LogLevel logLevel, char *format, ...);
 /* los.c */
@@ -189,14 +200,9 @@ extern char *create_overlay_pathname(char *name);
 extern int check_path(char *name, int prepend_dir);
 extern void dump_map(mapstruct *m);
 extern void dump_all_maps(void);
-extern int wall(mapstruct *m, int x, int y);
-extern int blocks_view(mapstruct *m, int x, int y);
-extern int blocks_magic(mapstruct *m, int x, int y);
-extern int blocks_cleric(mapstruct *m, int x, int y);
-extern int blocked(mapstruct *m, int x, int y);
-extern int blocked_link(object *ob, int x, int y);
-extern int arch_blocked(archetype *at, mapstruct *m, int x, int y);
-extern int arch_out_of_map(archetype *at, mapstruct *m, int x, int y);
+extern int get_map_flags(mapstruct *oldmap, mapstruct **newmap, sint16 x, sint16 y, sint16 *nx, sint16 *ny);
+extern int blocked_link(object *ob, mapstruct *m, int x, int y);
+extern int arch_blocked(archetype *at, mapstruct *m, sint16 x, sint16 y);
 extern void fix_container(object *container);
 extern void load_objects(mapstruct *m, FILE *fp, int mapflags);
 extern void save_objects(mapstruct *m, FILE *fp, FILE *fp2, int flag);
@@ -218,7 +224,7 @@ extern int change_map_light(mapstruct *m, int change);
 extern void update_position(mapstruct *m, int x, int y);
 extern void set_map_reset_time(mapstruct *map);
 extern int out_of_map(mapstruct *m, int x, int y);
-extern mapstruct *get_map_from_coord(mapstruct *m, int *x, int *y);
+extern mapstruct *get_map_from_coord(mapstruct *m, sint16 *x, sint16 *y);
 extern void get_rangevector(object *op1, object *op2, rv_vector *retval, int flags);
 extern void get_rangevector_from_mapcoord(mapstruct *m, int x, int y, object *op2, rv_vector *retval, int flags);
 extern int on_same_map(object *op1, object *op2);
@@ -253,6 +259,7 @@ extern int count_active(void);
 extern void sub_weight(object *op, signed long weight);
 extern void remove_ob(object *op);
 extern object *merge_ob(object *op, object *top);
+extern object *insert_ob_in_map_at(object *op, mapstruct *m, object *originator, int flag, int x, int y);
 extern object *insert_ob_in_map(object *op, mapstruct *m, object *originator, int flag);
 extern void replace_insert_ob_in_map(char *arch_string, object *op);
 extern object *get_split_ob(object *orig_ob, int nr);
@@ -263,7 +270,10 @@ extern int check_walk_on(object *op, object *originator);
 extern object *present_arch(archetype *at, mapstruct *m, int x, int y);
 extern object *present(unsigned char type, mapstruct *m, int x, int y);
 extern object *present_in_ob(unsigned char type, object *op);
+extern object *present_in_ob_by_name(int type, char *str, object *op);
 extern object *present_arch_in_ob(archetype *at, object *op);
+extern void flag_inv(object *op, int flag);
+extern void unflag_inv(object *op, int flag);
 extern void set_cheat(object *op);
 extern int find_free_spot(archetype *at, mapstruct *m, int x, int y, int start, int stop);
 extern int find_first_free_spot(archetype *at, mapstruct *m, int x, int y);
@@ -273,25 +283,29 @@ extern int distance(object *ob1, object *ob2);
 extern int find_dir_2(int x, int y);
 extern int absdir(int d);
 extern int dirdiff(int dir1, int dir2);
+extern int can_see_monsterP(mapstruct *m, int x, int y, int dir);
 extern int can_pick(object *who, object *item);
 extern object *ObjectCreateClone(object *asrc);
 extern int was_destroyed(object *op, tag_t old_tag);
 extern object *load_object_str(char *obstr);
+extern object *find_obj_by_type_subtype(object *who, int type, int subtype);
 /* porting.c */
 extern char *tempnam_local(char *dir, char *pfx);
 extern void remove_directory(const char *path);
 extern char *strdup_local(char *str);
 extern long strtol_local(register char *str, char **ptr, register int base);
+extern char *strcasestr_local(const char *s, const char *find);
 extern char *strerror_local(int errnum);
 extern int isqrt(int n);
 extern char *ltostr10(signed long n);
+extern char *doubletostr10(double v);
 extern void save_long(char *buf, char *name, long n);
+extern void save_long_long(char *buf, char *name, sint64 n);
 extern FILE *open_and_uncompress(char *name, int flag, int *compressed);
 extern void close_and_delete(FILE *fp, int compressed);
 extern void make_path_to_file(char *filename);
 /* player.c */
 extern void free_player(player *pl);
-extern object *find_skill(object *op, int skillnr);
 extern int atnr_is_dragon_enabled(int attacknr);
 extern int is_dragon_pl(object *op);
 /* re-cmp.c */
@@ -357,6 +371,7 @@ extern void create_all_treasures(treasure *t, object *op, int flag, int difficul
 extern void create_one_treasure(treasurelist *tl, object *op, int flag, int difficulty, int tries);
 extern void create_treasure(treasurelist *t, object *op, int flag, int difficulty, int tries);
 extern object *generate_treasure(treasurelist *t, int difficulty);
+extern int level_for_item(object *op, int difficulty, int retmult);
 extern int magic_from_difficulty(int difficulty);
 extern void set_abs_magic(object *op, int magic);
 extern void set_ring_bonus(object *op, int bonus);
@@ -379,9 +394,13 @@ extern void free_artifactlist(artifactlist *al);
 extern void free_all_treasures(void);
 /* utils.c */
 extern int random_roll(int min, int max, object *op, int goodbad);
+extern sint64 random_roll64(sint64 min, sint64 max, object *op, int goodbad);
 extern int die_roll(int num, int size, object *op, int goodbad);
 extern int rndm(int min, int max);
 extern void decay_objects(mapstruct *m);
+extern materialtype_t *name_to_material(char *name);
+extern void transmute_materialname(object *op, object *change);
+extern void set_materialname(object *op, int difficulty, materialtype_t *nmt);
 /* loader.c */
 extern int lex_load(object *op, int map_flags);
 extern void yyrestart(FILE *input_file);
@@ -393,3 +412,5 @@ extern void save_double(char *buf, char *name, double v);
 extern void init_vars(void);
 extern char *get_ob_diff(object *op, object *op2);
 extern void save_object(FILE *fp, object *op, int flag);
+extern void insert_event(object *op, int etype, char *ehook, char *eplug, char *eoptions);
+extern event *find_event(object *op, int etype);
