@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_h =
- *   "$Id: living.h,v 1.7 2002/07/15 04:57:12 mwedel Exp $";
+ *   "$Id: living.h,v 1.9 2003/09/13 05:01:33 mwedel Exp $";
  */
 
 /*
@@ -46,7 +46,6 @@
 #define NO_STAT_VAL 99    /* needed by skills code -b.t. */
 
 extern char *attacks[NROFATTACKS];
-extern char *spellpathnames[NRSPELLPATHS];
 
 extern float cha_bonus[MAX_STAT + 1];
 extern int dex_bonus[MAX_STAT + 1];
@@ -64,6 +63,8 @@ extern char *short_stat_name[NUM_STATS];
 extern char *lose_msg[NUM_STATS];
 extern float speed_bonus[MAX_STAT + 1];
 extern int weight_limit[MAX_STAT + 1];
+extern int cleric_chance[MAX_STAT + 1];
+extern int fear_bonus[MAX_STAT + 1];
 
 typedef struct liv { /* Mostly used by "alive" objects */
   sint8		Str,Dex,Con,Wis,Cha,Int,Pow;
@@ -74,7 +75,7 @@ typedef struct liv { /* Mostly used by "alive" objects */
   sint16	maxsp;		/* Max spell points. */
   sint16	grace;		/* Grace.  Used to invoke clerical prayers. */
   sint16	maxgrace;	/* Grace.  Used to invoke clerical prayers. */
-  sint32	exp;		/* Experience.  Killers gain 1/10. */
+  sint64	exp;		/* Experience.  Killers gain 1/10. */
   sint16	food;		/* How much food in stomach.  0 = starved. */
   sint16	dam;		/* How much damage this object does when hitting */
   sint8		luck;		/* Affects thaco and ac from time to time */
