@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: map.h,v 1.18 2002/07/15 04:25:40 mwedel Exp $";
+ *   "$Id: map.h,v 1.26 2004/02/16 18:05:32 ryo_saeba Exp $";
  */
 
 /*
@@ -34,6 +34,20 @@
 #ifndef MAP_H
 #define MAP_H
 
+/* We set this size - this is to make magic map work properly on
+ * tiled maps.  There is no requirement that this matches the
+ * tiled maps size - it just seemed like a reasonable value.
+ * Magic map code now always starts out putting the player in the
+ * center of the map - this makes the most sense when dealing
+ * with tiled maps.
+ * We also figure out the magicmap color to use as we process the
+ * spaces - this is more efficient as we already have up to date
+ * map pointers.
+ */
+#define MAGIC_MAP_SIZE  50
+#define MAGIC_MAP_HALF  MAGIC_MAP_SIZE/2
+
+
 #define MAP_LAYERS		4
 
 /* This is when the map will reset */
@@ -67,6 +81,9 @@
 #define MAP_WINDSPEED(m)	(m)->windspeed
 #define MAP_WINDDIRECTION(m)	(m)->winddir
 #define MAP_SKYCOND(m)		(m)->sky
+#define MAP_WORLDPARTX(m)	(m)->wpartx
+#define MAP_WORLDPARTY(m)	(m)->wparty
+#define MAP_NOSMOOTH(m)		(m)->nosmooth
 
 /* options passed to ready_map_name and load_original_map */
 #define MAP_FLUSH	    0x1
@@ -81,6 +98,11 @@
 #define MAP_LOADING 3
 #define MAP_SAVING 4
 
+/* GET_MAP_FLAGS really shouldn't be used very often - get_map_flags should
+ * really be used, as it is multi tile aware.  However, there are some cases
+ * where it is known the map is not tiled or the values are known
+ * consistent (eg, op->map, op->x, op->y)
+ */
 #define GET_MAP_FLAGS(M,X,Y)	( (M)->spaces[(X) + (M)->width * (Y)].flags )
 #define SET_MAP_FLAGS(M,X,Y,C)	( (M)->spaces[(X) + (M)->width * (Y)].flags = C )
 #define GET_MAP_LIGHT(M,X,Y)	( (M)->spaces[(X) + (M)->width * (Y)].light )
@@ -112,13 +134,23 @@
 #define P_BLOCKSVIEW	0x01
 #define P_NO_MAGIC      0x02	/* Spells (some) can't pass this object */
 #define P_NO_PASS       0x04	/* Nothing can pass (wall() is true) */
-#define P_PASS_THRU     0x08	/* */
+/*#define P_PASS_THRU     0x08	*//* */
 #define P_IS_ALIVE      0x10	/* something alive is on this space */
 #define P_NO_CLERIC     0x20	/* no clerical spells cast here */
 #define P_NEED_UPDATE	0x40	/* this space is out of date */
 #define P_NO_ERROR      0x80	/* Purely temporary - if set, update_position
                                  * does not complain if the flags are different.
                                  */
+/* The following two values are not stored in the MapLook flags, but instead
+ * used in the get_map_flags value - that function is used to return
+ * the flag value, as well as other conditions - using a more general
+ * function that does more of the work can hopefully be used to replace
+ * lots of duplicate checks currently in the code.
+ */
+#define P_OUT_OF_MAP	0x100	/* This space is outside the map */
+#define	P_NEW_MAP	0x200	/* Coordinates passed result in a new tiled map  */
+#define P_BLOCKED	(P_NO_PASS | P_IS_ALIVE)    /* convenience macro */
+#define P_WALL		P_NO_PASS   /* Just to match naming of wall function */
 
 /* Can't use MapCell as that is used in newserver.h
  * Instead of having numerous arrays that have information on a
@@ -146,9 +178,6 @@ typedef struct MapSpace {
  */
 
 typedef struct wmapdef {
-    char path[HUGE_BUF];	/* Filename of the map */
-    char	*tmpname;	/* Name of temporary file */
-    char 	*name;		/* Name of map as given by its creator */
     sint16	temp;		/* base temperature of this tile (F) */
     sint16	pressure;	/* barometric pressure (mb) */
     sint8	humid;		/* humitidy of this tile */
@@ -156,7 +185,11 @@ typedef struct wmapdef {
     sint8	winddir;	/* direction of wind */
     sint8	sky;		/* sky conditions */
     sint32	avgelev;	/* average elevation */
+    uint32	rainfall;	/* cumulative rainfall */
     uint8 	darkness;	/* indicates level of darkness of map */
+    uint8	water;		/* 0-100 percentage of water tiles */
+	/*Dynamic parts*/
+	sint16  realtemp;  /*  temperature at a given calculation step for this tile*/
 } weathermap_t;
 
 /* In general, code should always use the macros 
@@ -180,6 +213,7 @@ typedef struct mapdef {
 				 * players entering/exiting map
 				 */
     uint32 unique:1;	/* if set, this is a per player unique map */
+    uint32 nosmooth:1;  /* if set the content of this map has smoothlevel=0 forced*/
     sint32 timeout;	/* swapout is set to this */
     sint32 swap_time;	/* When it reaches 0, the map will be swapped out */
     sint16 players;	/* How many plares are on this level right now */
@@ -202,6 +236,7 @@ typedef struct mapdef {
     sint8     windspeed; /* windspeed of this tile */
     sint8     winddir;   /* direction of wind */
     sint8     sky;	/* sky conditions */
+    int     wpartx,wparty; /*Highly fasten conversion between worldmap and weathermap*/
     char    *msg;	/* Message map creator may have left */
     char    *tile_path[4];  /* path to adjoining maps */
     struct mapdef *tile_map[4];	/* Next map, linked list */
