@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_material_h =
- *   "$Id: material.h,v 1.7 2002/07/15 04:25:40 mwedel Exp $";
+ *   "$Id: material.h,v 1.9 2003/02/14 05:50:33 garbled Exp $";
  */
 
 /*
@@ -45,15 +45,28 @@
 #define M_SOFT_METAL		1024
 #define M_BONE			2048
 #define M_ICE			4096
+#define M_SPECIAL		8192	/* when displaying names, don't show the
+								   materialname */
 
-typedef struct {
-  char	*name;
-  sint8	 save[NROFATTACKS];
-} materialtype;
+typedef struct _materialtype {
+    char	*name;
+	char 	*description;
+    int		material;
+    sint8	save[NROFATTACKS];
+    sint8	mod[NROFATTACKS];
+    sint8	chance;
+    sint8	difficulty;
+    sint8	magic;
+    sint8	damage;
+    sint8	wc;
+    sint8	ac;
+	sint8	sp;
+    int		weight;
+    int		value;
+    struct _materialtype *next;
+} materialtype_t;
 
-EXTERN materialtype material[NROFMATERIALS];
-
-/* The data is initialized in common/item.c */
+EXTERN materialtype_t *materialt;
 
 #endif	/* MATERIAL_H_1 */
 
