@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newserver_h =
- *   "$Id: newserver.h,v 1.26 2002/07/15 04:57:12 mwedel Exp $";
+ *   "$Id: newserver.h,v 1.29 2003/09/13 05:01:33 mwedel Exp $";
  */
 
 /*
@@ -45,6 +45,7 @@
 
 struct MapCell {
   short faces[MAP_LAYERS];
+  uint16 smooth[MAP_LAYERS];
   int count;	/* This is really darkness in the map1 command */
 };
 
@@ -110,33 +111,57 @@ typedef struct NewSocket {
     char    *host;	    /* Which host it is connected from (ip address)*/
     Buffer  outputbuffer;   /* For undeliverable data */
     uint32  facecache:1;    /* If true, client is caching images */
-    uint8   faceset;	    /* Set the client is using, default 0 */
     uint32  sent_scroll:1;
     uint32  ext2:1;         /* enables the new sdl client/server stuff */
     uint32  ext_title_flag;  /* if 1, we should generate and send a new ext_title update */
     uint32  sound:1;	    /* does the client want sound */
-    uint32  skillexp:1;	    /* does the client want skill exp data - MT*/
-    enum MapMode mapmode;   /* Type of map commands the client wants. */
+    uint32  exp64:1;	    /* Client wants 64 bit exp data, as well as skill data */
     uint32  newmapcmd:1;    /* Send newmap command when entering new map SMACFIGGEN*/
     uint32  darkness:1;	    /* True if client wants darkness information */
     uint32  image2:1;	    /* Client wants image2/face2 commands */
-    uint32  newanim:1;      /* enable the advanced animation system - MT */
-    uint32  cs_version, sc_version; /* versions of the client */
     uint32  update_look:1;  /* If true, we need to send the look window */
     uint32  can_write:1;    /* Can we write to this socket? */
+    uint32  cs_version, sc_version; /* versions of the client */
+    enum MapMode mapmode;   /* Type of map commands the client wants. */
     uint16  look_position;  /* start of drawing of look window */
     uint8   mapx, mapy;	    /* How large a map the client wants */
     uint8   itemcmd;	    /* What version of the 'item' protocol command to use */
+    uint8   faceset;	    /* Set the client is using, default 0 */
+    uint32  ext_mapinfos:1;  /* If true client accept additionnal info on maps*/
+    /* Below are flags for extedend infos to pass to client 
+     * with S->C mapextended command */
+    uint32  EMI_smooth:1;   /* Send smooth in extendmapinfos*/
 
     /* Below here is information only relevant for old sockets */
     char    *comment;	    /* name or listen comment */
     enum Old_Mode old_mode;
 } NewSocket;
 
-
+/*Constants in the form EMI_ is for extended map infos.
+ * Even if the client select the additionnal infos it wants
+ * on the map, there may exist cases where this whole info
+ * is not given in one buch but in separate bunches. This 
+ * is done performance reasons (imagine some info related to
+ * a visible object and another info related to a 4 square
+ * width and height area). At the begin of an extended info packet
+ * is a bit field. A bit is activated for each extended info
+ * present in the data 
+ */
+/* Meanings:
+ * EMI_NOREDRAW  Take extended infos into account but don't redraw,
+ *               some additionnal datas will follow in a new packet
+ * EMI_SMOOTH    Datas about smoothing  
+ */ 
+#define EMI_NOREDRAW        0x01  
+#define EMI_SMOOTH          0x02
+/*this last one says the bitfield continue un next byte
+ * There may be several on contiguous bytes. So there is 7
+ * actual bits used per byte, and the number of bytes
+ * is not fixed in protocol
+ */
+#define EMI_HASMOREBITS     0x80
 
 #define FACE_TYPES  1
-
 #define PNG_FACE_INDEX	0
 
 typedef struct Socket_Info {
