@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_h =
- *   "$Id: player.h,v 1.21 2002/09/15 03:15:17 mwedel Exp $";
+ *   "$Id: player.h,v 1.35 2004/02/16 18:05:32 ryo_saeba Exp $";
  */
 
 /*
@@ -45,9 +45,30 @@ typedef enum rangetype {
     range_misc	    = 3, 
     range_golem     = 4,
     range_skill	    = 5,
-    range_size	    = 6
+    range_builder   = 6,
+    range_size	    = 7
 } rangetype;
 
+typedef enum _bowtype {
+    bow_normal = 0,
+    bow_threewide = 1,
+    bow_spreadshot = 2,
+    bow_n = 3, /* must stay at 3 */
+    bow_ne = 4,
+    bow_e = 5,
+    bow_se = 6,
+    bow_s = 7,
+    bow_sw = 8,
+    bow_w = 9,
+    bow_nw = 10, /* must stay at 10 */
+	bow_bestarrow = 11
+} bowtype_t;
+
+typedef enum _petmode {
+    pet_normal = 0,
+    pet_sad = 1,
+    pet_defend = 2
+} petmode_t;
 
 typedef enum usekeytype {
     key_inventory=0,
@@ -72,19 +93,17 @@ typedef struct pl {
     NewSocket	socket;		    /* Socket information for this player */
     char	maplevel[MAX_BUF];  /* On which level is the player? */
     struct mapdef *loading;	    /* When entering a map in progress of loading, not really used */
-    sint16	new_x,new_y;	    /* After entering a map being loaded */
     char	savebed_map[MAX_BUF];  /* map where player will respawn after death */
     sint16	bed_x, bed_y;	    /* x,y - coordinates of respawn (savebed) */
-    sint16	known_spells[NROFREALSPELLS]; /* Spells known by the player */
-    uint16	nrofknownspells;    /* Index in the above array */
     rangetype	shoottype;	    /* Which range-attack is being used by player */
+    bowtype_t   bowtype;	    /* which firemode? */
+    petmode_t   petmode;	    /* which petmode? */
     object	*ranges[range_size];/* object for each range.  Set up in fix player.  Note */
 				    /* That not all are used, it is just nice to map this 1:1 */
 				    /* With the range names */
+    uint32	golem_count;	    /* To track the golem */
     usekeytype	usekeys;	    /* Method for finding keys for doors */
     unapplymode	unapply;	    /* Method for auto unapply */
-    sint16	chosen_spell;	    /* Type of readied spell */
-    sint16	chosen_item_spell;  /* Type of spell that the item fires */
     uint32	count;		    /* Any numbers typed before a command */
     uint32	mode;		    /* Mode of player for pickup. */
 
@@ -99,40 +118,31 @@ typedef struct pl {
     sint8	last_level;	    /* Last level we sent to client */
 
     /* Try to put all the bitfields together - saves some small amount of memory */
-    uint32	known_spell:1;	    /* True if you know the spell of the wand */
     uint32	braced:1;	    /* Will not move if braced, only attack */
     uint32	tmp_invis:1;	    /* Will invis go away when we attack ? */
+    char	*invis_race;	    /* What race invisible to? */
     uint32	do_los:1;	    /* If true, need to call update_los() in draw(), and clear */
     uint32	fire_on:1;	    /* Player should fire object, not move */
     uint32	run_on:1;	    /* Player should keep moving in dir until run is off */
     uint32	has_hit:1;	    /* If set, weapon_sp instead of speed will count */
     uint32	name_changed:1;	    /* If true, the player has set a name. */
     uint32	peaceful:1;	    /* If set, won't attack friendly creatures */
-#ifdef EXPLORE_MODE
+    uint32	hidden:1;	    /* If True, player (DM) is hidden from view */
     uint32	explore:1;	    /* if True, player is in explore mode */
-#endif
-
-    sint32	last_skill_index;   /* this is init from init_player_exp() */
-    object	*last_skill_ob[MAX_EXP_CAT];	/* the exp object */
-    sint32	last_skill_exp[MAX_EXP_CAT];	/* shadow register. if != exp. obj update client */
-    sint32	last_skill_level[MAX_EXP_CAT];	/* same for level */
-    sint32	last_skill_id[MAX_EXP_CAT];	/* Thats the CS_STATS_ id for client STATS cmd*/
+	uint32  no_shout:1;		/* if True, player is *not* able to use shout command */
 
+    object	*last_skill_ob[NUM_SKILLS];	/* the exp object */
+    sint64	last_skill_exp[NUM_SKILLS];	/* shadow register. if != exp. obj update client */
 
     float	weapon_sp;	    /* Penalties to speed when fighting w speed >ws/10*/
     float	last_weapon_sp;	    /* if diff than weapon_sp, update client */
     uint16	last_flags;	    /* fire/run on flags for last tick */
     uint32	last_weight_limit;  /* Last weight limit transmitted to client */
-    object	*golem;		    /* Which golem is controlled */
     living	orig_stats;	    /* Permanent real stats of player */
     living	last_stats;	    /* Last stats as sent to client */
     float	last_speed;	    /* Last speed as sent to client */
     sint16	last_resist[NROFATTACKS];	/* last resist values sent to client */
-
-#ifdef USE_SWAP_STATS
     int		Swap_First;	    /* First stat player has selected to swap */
-#endif
-  
     object	*ob;		    /* The object representing the player */
     object	*last_used;	    /* Pointer to object last picked or applied */
     uint32	last_used_id;	    /* Safety measures to be sure it's the same */
@@ -166,16 +176,8 @@ typedef struct pl {
 				    /* but we will have to get password first */
 				    /* so we have to remember which party to */
 				    /* join */
-#ifdef SEARCH_ITEMS
     char    search_str[MAX_BUF];    /* Item we are looking for */
-#endif /* SEARCH_ITEMS */
-
-#ifdef SPELL_ENCUMBRANCE
     sint16  encumbrance;	    /* How much our player is encumbered  */
-				    /* Probably would be more clever to always have the */
-				    /* calculation coded in, and just disable the part that */
-				    /* that makes it do something.  OR make it a settings value */
-#endif
     Output_Buf	outputs[NUM_OUTPUT_BUFS];   /* holds output strings to client */
     uint16	outputs_sync;	    /* How often to print, no matter what */
     uint16	outputs_count;	    /* Print if this count is exceeded */
@@ -186,20 +188,18 @@ typedef struct pl {
 
 /* not really the player, but tied pretty closely */  
 typedef struct party_struct {
-  sint16 partyid;
-  char * partyleader;
-  char passwd[9];
-  struct party_struct *next;
-  char *partyname;
+    sint16 partyid;
+    char * partyleader;
+    char passwd[9];
+    struct party_struct *next;
+    char *partyname;
+
 #ifdef PARTY_KILL_LOG
-  struct party_kill
-  {
-    char killer[MAX_NAME+1],dead[MAX_NAME+1];
-    uint32 exp;
-  } party_kills[PARTY_KILL_LOG];
+    struct party_kill {
+	char killer[MAX_NAME+1],dead[MAX_NAME+1];
+	sint64 exp;
+    } party_kills[PARTY_KILL_LOG];
 #endif
-  uint32 total_exp,kills;
+    sint64 total_exp;
+    uint32  kills;
 } partylist;
-
-
-
