@@ -62,8 +62,11 @@ f_plugin HookList[NR_OF_HOOKS] =
     CFWApplyBelow,
     CFWFreeObject,
     CFWObjectCreateClone,
-    CFWTeleportObject
-
+    CFWTeleportObject,
+    CFWSetVariable,
+    CFWDecreaseObjectNR,
+    CFWFreeMemory,
+    CFWFixPlayer
 };
 
 #endif /*PLUGPROTO_H_*/
