@@ -15,13 +15,11 @@ void new_draw_info_format(int flags, int pri, object *pl, char *format, ...);
 void new_info_map_except(int color, mapstruct *map, object *op, char *str);
 void new_info_map_except2(int color, mapstruct *map, object *op1, object *op2, char *str);
 void new_info_map(int color, mapstruct *map, char *str);
-void draw(object *pl);
 void clear_win_info(object *op);
 void rangetostring(object *pl, char *obuf);
 void set_title(object *pl, char *buf);
 void magic_mapping_mark(object *pl, char *map_mark, int strength);
-void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int py);
-void draw_map(object *pl);
+void draw_magic_map(object *pl);
 void Log_Kill(const char *Who, const char *What, int WhatType, const char *With, int WithType);
 /* init.c */
 void InitConnection(NewSocket *ns, uint32 from);
@@ -55,6 +53,7 @@ void SockList_Init(SockList *sl);
 void SockList_AddChar(SockList *sl, char c);
 void SockList_AddShort(SockList *sl, uint16 data);
 void SockList_AddInt(SockList *sl, uint32 data);
+void SockList_AddInt64(SockList *sl, uint64 data);
 int GetInt_String(unsigned char *data);
 short GetShort_String(unsigned char *data);
 int SockList_ReadPacket(int fd, SockList *sl, int len);
@@ -70,6 +69,8 @@ void metaserver_update(void);
 /* request.c */
 void SetUp(char *buf, int len, NewSocket *ns);
 void AddMeCmd(char *buf, int len, NewSocket *ns);
+void ToggleExtendedInfos(char *buf, int len, NewSocket *ns);
+void AskSmooth(char *buf, int len, NewSocket *ns);
 void PlayerCmd(char *buf, int len, player *pl);
 void NewPlayerCmd(uint8 *buf, int len, player *pl);
 void ReplyCmd(char *buf, int len, player *pl);
@@ -82,10 +83,12 @@ void send_query(NewSocket *ns, uint8 flags, char *text);
 void esrv_update_stats(player *pl);
 void esrv_new_player(player *pl, uint32 weight);
 void esrv_send_animation(NewSocket *ns, short anim_num);
+int getExtendedMapInfoSize(NewSocket *ns);
 void draw_client_map1(object *pl);
 void draw_client_map(object *pl);
 void esrv_map_scroll(NewSocket *ns, int dx, int dy);
 void send_plugin_custom_message(object *pl, char *buf);
+void send_skill_info(NewSocket *ns, char *params);
 /* sounds.c */
-void play_sound_player_only(player *pl, int soundnum, int x, int y);
-void play_sound_map(mapstruct *map, int x, int y, int sound_num);
+void play_sound_player_only(player *pl, short soundnum, sint8 x, sint8 y);
+void play_sound_map(mapstruct *map, int x, int y, short sound_num);
