@@ -15,7 +15,7 @@
 /* Please note that it is still very beta - some of the functions may not    */
 /* work as expected and could even cause the server to crash.                */
 /*****************************************************************************/
-/* Version: 0.6 Beta  (also known as "Kharkov")                              */
+/* Version: 0.7 Beta  (also known as "Koursk")                               */
 /* Contact: yann.chachkoff@mailandnews.com                                   */
 /*****************************************************************************/
 /* That code is placed under the GNU General Public Licence (GPL)            */
@@ -45,7 +45,7 @@
 #endif
 
 #define PLUGIN_NAME    "Python"
-#define PLUGIN_VERSION "CFPython Plugin 0.1"
+#define PLUGIN_VERSION "CFPython Plugin 0.7 (Koursk)"
 
 /* The plugin properties and hook functions. A hook function is a pointer to */
 /* a CF function wrapper. Basically, most CF functions that could be of any  */
@@ -277,6 +277,8 @@ static PyObject* CFSetEventOptions(PyObject* self, PyObject* args);
 static PyObject* CFGetIP(PyObject* self, PyObject* args);
 static PyObject* CFGetInventory(PyObject* self, PyObject* args);
 static PyObject* CFGetInternalName(PyObject* self, PyObject* args);
+static PyObject* CFSetVariable(PyObject* self, PyObject* args);
+static PyObject* CFDecreaseObjectNR(PyObject* self, PyObject* args);
 
 /* Skill id wrappers */
 static PyObject* CFSkillStealing(PyObject* self, PyObject* args);
@@ -304,13 +306,14 @@ static PyObject* CFSkillWoodsman(PyObject* self, PyObject* args);
 static PyObject* CFSkillInscription(PyObject* self, PyObject* args);
 static PyObject* CFSkillMeleeWeapons(PyObject* self, PyObject* args);
 static PyObject* CFSkillThrowing(PyObject* self, PyObject* args);
-static PyObject* CFSkillSpellCasting(PyObject* self, PyObject* args);
+/*static PyObject* CFSkillSpellCasting(PyObject* self, PyObject* args);*/
 static PyObject* CFSkillRemoveTraps(PyObject* self, PyObject* args);
 static PyObject* CFSkillSetTraps(PyObject* self, PyObject* args);
 static PyObject* CFSkillUseMagicItem(PyObject* self, PyObject* args);
 static PyObject* CFSkillPraying(PyObject* self, PyObject* args);
 static PyObject* CFSkillClawing(PyObject* self, PyObject* args);
 static PyObject* CFSkillLevitation(PyObject* self, PyObject* args);
+#if 0
 /* Spell id wrappers */
 static PyObject* CFSpellBullet(PyObject* self, PyObject* args);
 static PyObject* CFSpellSmallFireball(PyObject* self, PyObject* args);
@@ -514,6 +517,7 @@ static PyObject* CFSpellVitriol(PyObject* self, PyObject* args);
 static PyObject* CFSpellVitriolSplash(PyObject* self, PyObject* args);
 static PyObject* CFSpellIronwoodSkin(PyObject* self, PyObject* args);
 static PyObject* CFSpellWrathfullEye(PyObject* self, PyObject* args);
+#endif
 static PyObject* CFAttackTypePhysical(PyObject* self, PyObject* args);
 static PyObject* CFAttackTypeMagic(PyObject* self, PyObject* args);
 static PyObject* CFAttackTypeFire(PyObject* self, PyObject* args);
@@ -556,6 +560,20 @@ static PyObject* CFGetObjectMoney(PyObject* self, PyObject* args);
 static PyObject* CFPayForItem(PyObject* self, PyObject* args);
 static PyObject* CFPayAmount(PyObject* self, PyObject* args);
 static PyObject* CFSendCustomCommand(PyObject* self, PyObject* args);
+static PyObject* CFGetHumidity(PyObject* self, PyObject* args);
+static PyObject* CFGetTemperature(PyObject* self, PyObject* args);
+static PyObject* CFGetPressure(PyObject* self, PyObject* args);
+static PyObject* CFGetWC(PyObject* self, PyObject* args);
+static PyObject* CFSetWC(PyObject* self, PyObject* args);
+
+/* Setting values - datapaths */
+static PyObject* CFGetMapDir(PyObject* self, PyObject* args);
+static PyObject* CFGetUniqueDir(PyObject* self, PyObject* args);
+static PyObject* CFGetTempDir(PyObject* self, PyObject* args);
+static PyObject* CFGetConfigurationDir(PyObject* self, PyObject* args);
+static PyObject* CFGetDataDir(PyObject* self, PyObject* args);
+static PyObject* CFGetLocalDir(PyObject* self, PyObject* args);
+static PyObject* CFGetPlayerDir(PyObject* self, PyObject* args);
 
 /* Those are used to handle the events. The first one is used when a player  */
 /* attacks with a "scripted" weapon. HandleEvent is used for all other events*/
@@ -798,6 +816,20 @@ static PyMethodDef CFPythonMethods[] =
         {"PayForItem",CFPayForItem,METH_VARARGS},
         {"PayAmount",CFPayAmount,METH_VARARGS},
         {"SendCustomCommand",CFSendCustomCommand,METH_VARARGS},
+        {"GetHumidity",CFGetHumidity, METH_VARARGS},
+        {"GetTemperature",CFGetTemperature, METH_VARARGS},
+        {"GetPressure",CFGetPressure, METH_VARARGS},
+        {"GetMapDirectory", CFGetMapDir, METH_VARARGS},
+        {"GetUniqueDirectory", CFGetUniqueDir, METH_VARARGS},
+        {"GetTempDirectory", CFGetTempDir, METH_VARARGS},
+        {"GetConfigurationDirectory", CFGetConfigurationDir, METH_VARARGS},
+        {"GetLocalDirectory", CFGetLocalDir, METH_VARARGS},
+        {"GetPlayerDirectory", CFGetPlayerDir, METH_VARARGS},
+        {"GetDataDirectory", CFGetDataDir, METH_VARARGS},
+        {"GetWC", CFGetWC, METH_VARARGS},
+        {"SetWC", CFSetWC, METH_VARARGS},
+        {"SetVariable", CFSetVariable, METH_VARARGS},
+        {"DecreaseObjectNr", CFDecreaseObjectNR, METH_VARARGS},
 /* Skills wrappers : */
         {"SkillStealing", CFSkillStealing, METH_VARARGS},
         {"SkillLockpicking", CFSkillLockpicking, METH_VARARGS},
@@ -824,13 +856,14 @@ static PyMethodDef CFPythonMethods[] =
         {"SkillInscription", CFSkillInscription, METH_VARARGS},
         {"SkillMeleeWeapons", CFSkillMeleeWeapons, METH_VARARGS},
         {"SkillThrowing", CFSkillThrowing, METH_VARARGS},
-        {"SkillSpellcasting", CFSkillSpellCasting, METH_VARARGS},
+/*        {"SkillSpellcasting", CFSkillSpellCasting, METH_VARARGS},*/
         {"SkillRemoveTraps", CFSkillRemoveTraps, METH_VARARGS},
         {"SkillSetTraps", CFSkillSetTraps, METH_VARARGS},
         {"SkillUseMagicItem", CFSkillUseMagicItem, METH_VARARGS},
         {"SkillPraying", CFSkillPraying, METH_VARARGS},
         {"SkillClawing", CFSkillClawing, METH_VARARGS},
         {"SkillLevitation", CFSkillLevitation, METH_VARARGS},
+#if 0
 /* Spells wrappers */
         {"SpellBullet", CFSpellBullet, METH_VARARGS},
         {"SpellSmallFireball", CFSpellSmallFireball, METH_VARARGS},
@@ -1034,6 +1067,7 @@ static PyMethodDef CFPythonMethods[] =
         {"SpellVitriolSplash", CFSpellVitriolSplash, METH_VARARGS},
         {"SpellIronwoodSkin", CFSpellIronwoodSkin, METH_VARARGS},
         {"SpellWrathfullEye", CFSpellWrathfullEye, METH_VARARGS},
+#endif
 /* Attack Types Wrappers */
         {"AttackTypePhysical", CFAttackTypePhysical, METH_VARARGS},
         {"AttackTypeMagic", CFAttackTypeMagic, METH_VARARGS},
