@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_random_map_c =
- *   "$Id: random_map.c,v 1.28 2001/10/07 06:45:40 mwedel Exp $";
+ *   "$Id: random_map.c,v 1.35 2003/09/26 05:44:43 mwedel Exp $";
  */
 
 /*
@@ -45,192 +45,195 @@ void dump_layout(char **layout,RMParms *RP) {
     }
     printf("\n");
     }}
+    printf("\n");
 }
 EXTERN FILE *logfile;
 mapstruct *generate_random_map(char *OutFileName, RMParms *RP) {
-  char **layout;
-  mapstruct *theMap;
+    char **layout, buf[HUGE_BUF];
+    mapstruct *theMap;
+    int i;
 
-  /* pick a random seed, or use the one from the input file */
-  if(RP->random_seed == 0)    SRANDOM(time(0));
-  else SRANDOM(RP->random_seed);
+    /* pick a random seed, or use the one from the input file */
+    if(RP->random_seed == 0)
+	RP->random_seed=time(0);
 
-  if(RP->difficulty==0)
-    RP->difficulty = RP->dungeon_level; /* use this instead of a map difficulty  */
-  else
-    RP->difficulty_given=1;
+    SRANDOM(RP->random_seed);
 
-  if(RP->expand2x > 0) {
-      RP->Xsize /=2;
-      RP->Ysize /=2;
+    write_map_parameters_to_string(buf, RP);
+
+    if(RP->difficulty==0)
+	RP->difficulty = RP->dungeon_level; /* use this instead of a map difficulty  */
+    else
+	RP->difficulty_given=1;
+
+    if(RP->Xsize<MIN_RANDOM_MAP_SIZE) RP->Xsize = MIN_RANDOM_MAP_SIZE + RANDOM()%25 + 5;
+    if(RP->Ysize<MIN_RANDOM_MAP_SIZE) RP->Ysize = MIN_RANDOM_MAP_SIZE + RANDOM()%25 + 5;
+
+    if(RP->expand2x > 0) {
+	RP->Xsize /=2;
+	RP->Ysize /=2;
     }
 
-  layout = layoutgen(RP);
+    layout = layoutgen(RP);
 
-  /* increment these for the current map */
-  RP->dungeon_level+=1;
-  /* allow constant-difficulty maps. */
-  /*  difficulty+=1; */
-
-  /*  rotate the layout randomly */
-  layout=rotate_layout(layout,RANDOM()%4,RP);
 #ifdef RMAP_DEBUG
-  dump_layout(layout,RP);
+    dump_layout(layout,RP);
 #endif
-  /* allocate the map and set the floor */
-  theMap = make_map_floor(layout,RP->floorstyle,RP); 
 
-  /* set the name of the map. */
-  strcpy(theMap->path,OutFileName);
+    /* increment these for the current map */
+    RP->dungeon_level+=1;
+    /* allow constant-difficulty maps. */
+    /*  difficulty+=1; */
 
-  make_map_walls(theMap,layout,RP->wallstyle,RP);
+    /*  rotate the layout randomly */
+    layout=rotate_layout(layout,RANDOM()%4,RP);
+#ifdef RMAP_DEBUG
+    dump_layout(layout,RP);
+#endif
 
-  put_doors(theMap,layout,RP->doorstyle,RP);
+    /* allocate the map and set the floor */
+    theMap = make_map_floor(layout,RP->floorstyle,RP); 
 
-  place_exits(theMap,layout,RP->exitstyle,RP->orientation,RP);
+    /* set the name of the map. */
+    strcpy(theMap->path,OutFileName);
 
-  place_specials_in_map(theMap,layout,RP);
+    make_map_walls(theMap,layout,RP->wallstyle,RP);
 
-  place_monsters(theMap,RP->monsterstyle,RP->difficulty,RP);
+    put_doors(theMap,layout,RP->doorstyle,RP);
 
-  /* treasures needs to have a proper difficulty set for
-     the map. */
-  theMap->difficulty=calculate_difficulty(theMap);
+    place_exits(theMap,layout,RP->exitstyle,RP->orientation,RP);
 
-  place_treasure(theMap,layout,RP->treasurestyle,RP->treasureoptions,RP);
+    place_specials_in_map(theMap,layout,RP);
 
-  put_decor(theMap,layout,RP->decorstyle,RP->decoroptions,RP);
+    place_monsters(theMap,RP->monsterstyle,RP->difficulty,RP);
 
-  /* generate treasures, etc. */
-  if(RP->generate_treasure_now)
-    fix_auto_apply(theMap);
+    /* treasures needs to have a proper difficulty set for the map. */
+    theMap->difficulty=calculate_difficulty(theMap);
 
-  unblock_exits(theMap,layout,RP);
-  /*  fclose(OutFile); */
-  /*new_save_map(theMap,1);*/
+    place_treasure(theMap,layout,RP->treasurestyle,RP->treasureoptions,RP);
 
-  /*free the layout */
-  { int i;
+    put_decor(theMap,layout,RP->decorstyle,RP->decoroptions,RP);
+
+    /* generate treasures, etc. */
+    if(RP->generate_treasure_now)
+	fix_auto_apply(theMap);
+
+    unblock_exits(theMap,layout,RP);
+
+    /* free the layout */
     for(i=0;i<RP->Xsize;i++)
-      free(layout[i]);
+	free(layout[i]);
     free(layout);
-  }
+
+    theMap->msg = strdup_local(buf);
   
-  return theMap;
+    return theMap;
 }
 
 
 /*  function selects the layout function and gives it whatever
     arguments it needs.  */
 char **layoutgen(RMParms *RP) {
-  char **maze=0;
-  if(RP->symmetry!=NO_SYM) {
-    if(RP->Xsize<15) RP->Xsize = 15 + RANDOM()%25;
-    if(RP->Ysize<15) RP->Ysize = 15 + RANDOM()%25;
-  }
-  else
-    {
-	/* Has to be at least 7 for square spirals to work */
-	if(RP->Xsize<7) RP->Xsize = 15 + RANDOM()%25;
-	if(RP->Ysize<7) RP->Ysize = 15 + RANDOM()%25;
-    }
-  
-  if(RP->symmetry == RANDOM_SYM) {
-    RP->symmetry_used = (RANDOM() % ( XY_SYM))+1;
+    char **maze=0;
+    int oxsize= RP->Xsize, oysize=RP->Ysize;
+
+    if(RP->symmetry == RANDOM_SYM) 
+	RP->symmetry_used = (RANDOM() % ( XY_SYM))+1;
+    else RP->symmetry_used = RP->symmetry;
+
     if(RP->symmetry_used==Y_SYM||RP->symmetry_used==XY_SYM) RP->Ysize = RP->Ysize/2+1;
     if(RP->symmetry_used==X_SYM||RP->symmetry_used==XY_SYM) RP->Xsize = RP->Xsize/2+1;
-  }
-  else RP->symmetry_used = RP->symmetry;
 
-  if(RP->symmetry==Y_SYM||RP->symmetry==XY_SYM) RP->Ysize = RP->Ysize/2+1;
-  if(RP->symmetry==X_SYM||RP->symmetry==XY_SYM) RP->Xsize = RP->Xsize/2+1;
+    if(RP->Xsize<MIN_RANDOM_MAP_SIZE) RP->Xsize = MIN_RANDOM_MAP_SIZE + RANDOM()%5;
+    if(RP->Ysize<MIN_RANDOM_MAP_SIZE) RP->Ysize = MIN_RANDOM_MAP_SIZE + RANDOM()%5;
+    RP->map_layout_style = 0;
 
-  if(strstr(RP->layoutstyle,"onion")) {
-    maze = map_gen_onion(RP->Xsize,RP->Ysize,RP->layoutoptions1,RP->layoutoptions2);
-    RP->map_layout_style = ONION_LAYOUT;
-    if(!(RANDOM()%3)&& !(RP->layoutoptions1 & OPT_WALLS_ONLY)) roomify_layout(maze,RP);
-  }
-
-  if(strstr(RP->layoutstyle,"maze")) {
-    maze = maze_gen(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-    RP->map_layout_style = MAZE_LAYOUT;
-
-    if(!(RANDOM()%2)) doorify_layout(maze,RP);
-  }
-
-  if(strstr(RP->layoutstyle,"spiral")) {
-    maze = map_gen_spiral(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-    RP->map_layout_style = SPIRAL_LAYOUT;
-    if(!(RANDOM()%2)) doorify_layout(maze,RP);
-  }
-
-  if(strstr(RP->layoutstyle,"rogue")) {
-    maze = roguelike_layout_gen(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-    RP->map_layout_style = ROGUELIKE_LAYOUT;
-    /* no doorification */
-  }
-
-  if(strstr(RP->layoutstyle,"snake")) {
-    maze = make_snake_layout(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-    RP->map_layout_style = SNAKE_LAYOUT;
-    if(RANDOM()%2) roomify_layout(maze,RP);
-    
-  }
-
-  if(strstr(RP->layoutstyle,"squarespiral")) {
-    maze = make_square_spiral_layout(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-    RP->map_layout_style = SQUARE_SPIRAL_LAYOUT;
-    if(RANDOM()%2) roomify_layout(maze,RP);
-    
-  }
-
-  if(maze == 0) /* unknown or unspecified layout type, pick one at random */
-    switch(RANDOM()%NROFLAYOUTS) {
-    case 0:
-      maze = maze_gen(RP->Xsize,RP->Ysize,RANDOM()%2);
-      RP->map_layout_style = MAZE_LAYOUT;
-      if(!(RANDOM()%2)) doorify_layout(maze,RP);
-      break;
-    case 1:
-      maze = map_gen_onion(RP->Xsize,RP->Ysize,RP->layoutoptions1,RP->layoutoptions2);
-      RP->map_layout_style = ONION_LAYOUT;
-      if(!(RANDOM()%3)&& !(RP->layoutoptions1 & OPT_WALLS_ONLY)) roomify_layout(maze,RP);
-      break;
-    case 2:
-      maze = map_gen_spiral(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-      RP->map_layout_style = SPIRAL_LAYOUT;
-      if(!(RANDOM()%2)) doorify_layout(maze,RP);
-      break;
-    case 3:
-      maze = roguelike_layout_gen(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-      RP->map_layout_style = ROGUELIKE_LAYOUT;
-      /* no doorifying...  done already */
-      break;
-    case 4:
-      maze = make_snake_layout(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-      RP->map_layout_style = SNAKE_LAYOUT;
-      if(RANDOM()%2) roomify_layout(maze,RP);
-      break;
-    case 5:
-      maze = make_square_spiral_layout(RP->Xsize,RP->Ysize,RP->layoutoptions1);
-      RP->map_layout_style = SQUARE_SPIRAL_LAYOUT;
-      if(RANDOM()%2) roomify_layout(maze,RP);
-      break;
+    /* Redo this - there was a lot of redundant code of checking for preset
+     * layout style and then random layout style.  Instead, figure out
+     * the numeric layoutstyle, so there is only one area that actually
+     * calls the code to make the maps.
+     */
+    if(strstr(RP->layoutstyle,"onion")) {
+	RP->map_layout_style = ONION_LAYOUT;
     }
 
-   
+    if(strstr(RP->layoutstyle,"maze")) {
+	RP->map_layout_style = MAZE_LAYOUT;
+    }
 
+    if(strstr(RP->layoutstyle,"spiral")) {
+	RP->map_layout_style = SPIRAL_LAYOUT;
+    }
 
-  maze = symmetrize_layout(maze, RP->symmetry_used,RP);
+    if(strstr(RP->layoutstyle,"rogue")) {
+	RP->map_layout_style = ROGUELIKE_LAYOUT;
+    }
+
+    if(strstr(RP->layoutstyle,"snake")) {
+	RP->map_layout_style = SNAKE_LAYOUT;
+    }
+
+    if(strstr(RP->layoutstyle,"squarespiral")) {
+	RP->map_layout_style = SQUARE_SPIRAL_LAYOUT;
+    }
+    /* No style found - choose one ranomdly */
+    if (RP->map_layout_style == 0) {
+	RP->map_layout_style = (RANDOM() % NROFLAYOUTS) + 1;
+    }
+
+    switch(RP->map_layout_style) {
+
+	    case ONION_LAYOUT:
+		maze = map_gen_onion(RP->Xsize,RP->Ysize,RP->layoutoptions1,RP->layoutoptions2);
+		if(!(RANDOM()%3)&& !(RP->layoutoptions1 & OPT_WALLS_ONLY)) roomify_layout(maze,RP);
+		break;
+
+	    case MAZE_LAYOUT:
+		maze = maze_gen(RP->Xsize,RP->Ysize,RANDOM()%2);
+		if(!(RANDOM()%2)) doorify_layout(maze,RP);
+		break;
+
+	    case SPIRAL_LAYOUT:
+		maze = map_gen_spiral(RP->Xsize,RP->Ysize,RP->layoutoptions1);
+		if(!(RANDOM()%2)) doorify_layout(maze,RP);
+		break;
+
+	    case ROGUELIKE_LAYOUT:
+		/* Don't put symmetry in rogue maps.  There isn't much reason to
+		 * do so in the first place (doesn't make it any more interesting),
+		 * but more importantly, the symmetry code presumes we are symmetrizing
+		 * spirals, or maps with lots of passages - making a symmetric rogue
+		 * map fails because its likely that the passages the symmetry process
+		 * creates may not connect the rooms.
+		 */
+		RP->symmetry_used = NO_SYM;
+		RP->Ysize = oysize;
+		RP->Xsize = oxsize;
+		maze = roguelike_layout_gen(RP->Xsize,RP->Ysize,RP->layoutoptions1);
+		/* no doorifying...  done already */
+		break;
+
+	    case SNAKE_LAYOUT:
+		maze = make_snake_layout(RP->Xsize,RP->Ysize,RP->layoutoptions1);
+		if(RANDOM()%2) roomify_layout(maze,RP);
+		break;
+
+	    case SQUARE_SPIRAL_LAYOUT:
+		maze = make_square_spiral_layout(RP->Xsize,RP->Ysize,RP->layoutoptions1);
+		if(RANDOM()%2) roomify_layout(maze,RP);
+		break;
+	}
+
+    maze = symmetrize_layout(maze, RP->symmetry_used,RP);
 #ifdef RMAP_DEBUG
-  dump_layout(maze,RP);
+    dump_layout(maze,RP);
 #endif
-  if(RP->expand2x) {
-    maze = expand2x(maze,RP->Xsize,RP->Ysize);
-    RP->Xsize = RP->Xsize * 2 -1;
-    RP->Ysize = RP->Ysize * 2 -1;
-  }
-  return maze; 
+    if(RP->expand2x) {
+	maze = expand2x(maze,RP->Xsize,RP->Ysize);
+	RP->Xsize = RP->Xsize * 2 -1;
+	RP->Ysize = RP->Ysize * 2 -1;
+    }
+    return maze; 
 }
 
 
@@ -628,6 +631,7 @@ void write_map_parameters_to_string(char *buf,RMParms *RP) {
     strcat(buf,small_buf);
   }
   if(RP->random_seed) {
+    /* Add one so that the next map is a bit different */
     sprintf(small_buf,"random_seed %d\n",RP->random_seed + 1);
     strcat(buf,small_buf);
   }
@@ -778,6 +782,7 @@ void write_parameters_to_string(char *buf,
     strcat(buf,small_buf);
   }
   if(random_seed_n) {
+    /* Add one so that the next map is a bit different */
     sprintf(small_buf,"random_seed %d\n",random_seed_n + 1);
     strcat(buf,small_buf);
   }
