@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_special_c =
- *   "$Id: special.c,v 1.17 2002/09/07 05:55:18 mwedel Exp $";
+ *   "$Id: special.c,v 1.19 2003/03/07 07:33:13 mwedel Exp $";
  */
 
 /*
@@ -127,7 +127,7 @@ int find_spot_for_submap(mapstruct *map,char **layout,int *ix, int *iy,int xsize
       for(l=i;l<i + xsize;l++)
         for(m=j;m<j + ysize;m++)
           if(layout[l][m]=='C' || layout[l][m]=='>' || layout[l][m]=='<')
-            is_occupied=1;
+            is_occupied|=1;
     }
   }
   if(is_occupied) return 0;
@@ -230,8 +230,8 @@ void place_special_exit(mapstruct * map, int hole_type,RMParms *RP) {
     /* Need to be at least this size, otherwise the load
      * code will generate new size values which are too large.
      */
-    if (g_xsize < 7) g_xsize = 7;
-    if (g_ysize < 7) g_ysize = 7;
+    if (g_xsize < MIN_RANDOM_MAP_SIZE) g_xsize = MIN_RANDOM_MAP_SIZE;
+    if (g_ysize < MIN_RANDOM_MAP_SIZE) g_ysize = MIN_RANDOM_MAP_SIZE;
 
     write_parameters_to_string(buf, g_xsize, g_ysize,RP->wallstyle,RP->floorstyle,mon,
                                  "none",style,decor,"none",RP->exitstyle,0,0,
