@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_chat_c =
- *   "$Id: c_chat.c,v 1.8 2002/07/15 04:25:40 mwedel Exp $";
+ *   "$Id: c_chat.c,v 1.17 2003/11/07 19:54:49 ryo_saeba Exp $";
  */
 
 /*
@@ -35,52 +35,98 @@ int command_say (object *op, char *params)
     char buf[MAX_BUF];
 
     if (!params) return 0;
-    /* This broken apart from one sprintf to avoid buffer overuns.
-     * This could be done via snprintf, but I am not sure how widely
-     * available that is. MSW 5/22/2000
-     */
-    sprintf(buf, "%s says: ",op->name);
-    strncat(buf, params, MAX_BUF - strlen(buf)-1);
-    buf[MAX_BUF-1]=0;
-
+    snprintf(buf, MAX_BUF-1, "%s says: %s",op->name, params);
     new_info_map(NDI_WHITE,op->map, buf);
     communicate(op, params);
   
     return 0;
 }
 
-int command_shout (object *op, char *params)
+
+int command_me (object *op, char *params)
 {
     char buf[MAX_BUF];
-#ifdef PLUGINS
+
+    if (!params) return 0;
+    snprintf(buf, MAX_BUF-1, "%s %s",op->name, params);
+        new_info_map(NDI_UNIQUE|NDI_BLUE,op->map, buf);
+
+    return 0;
+}
+
+
+int command_cointoss(object *op, char *params)
+{
+    char buf[MAX_BUF];
+    char buf2[MAX_BUF];
+    int i;
+
+    i = rndm(1, 2);
+    if (i == 1) {
+	snprintf(buf, MAX_BUF-1, "%s flips a coin.... Heads!", op->name);
+	snprintf(buf2, MAX_BUF-1, "You flip a coin.... Heads!");
+    } else {
+	snprintf(buf, MAX_BUF-1, "%s flips a coin.... Tails!", op->name);
+	snprintf(buf2, MAX_BUF-1, "You flip a coin.... Tails!");
+    }
+    new_draw_info(NDI_UNIQUE, 0, op, buf2);
+    new_info_map_except(NDI_WHITE, op->map, op, buf);
+    return 0;
+}
+
+char *orcknuckle[7] = {"none", "beholder", "ghost", "knight",
+		       "princess", "dragon", "orc"};
+
+int command_orcknuckle(object *op, char *params)
+{
+    char buf[MAX_BUF];
+    char buf2[MAX_BUF];
+    int i, j, k, l;
+
+    i = rndm(1, 5);
+    j = rndm(1, 5);
+    k = rndm(1, 5);
+    l = rndm(1, 6);
+
+    snprintf(buf2, MAX_BUF-1, "%s rolls %s, %s, %s, %s!", op->name,
+	orcknuckle[i], orcknuckle[j], orcknuckle[k], orcknuckle[l]);
+    snprintf(buf, MAX_BUF-1, "You roll %s, %s, %s, %s!",
+	orcknuckle[i], orcknuckle[j], orcknuckle[k], orcknuckle[l]);
+    new_draw_info(NDI_UNIQUE, 0, op, buf);
+    new_info_map_except(NDI_WHITE, op->map, op, buf2);
+    return 0;
+}
+
+int command_shout (object *op, char *params)
+{
     int evtid;
     CFParm CFP;
-#endif
+	if (op->contr->no_shout == 1){
+		new_draw_info(NDI_UNIQUE, 0,op,"You are no longer allowed to shout.");
+		return 1;
+	}else{
+	
     if (params == NULL) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Shout what?");
 	return 1;
     }
-    strcpy(buf,op->name);
-    strcat(buf," shouts: ");
-    strncat(buf, params, MAX_BUF-30);
-    buf[MAX_BUF - 30] = '\0';
-    new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_RED, 1, NULL, buf);
-#ifdef PLUGINS
+    new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_RED, 1, NULL, 
+		 "%s shouts: %s", op->name, params);
     /* GROS : Here we handle the SHOUT global event */
     evtid = EVENT_SHOUT;
     CFP.Value[0] = (void *)(&evtid);
     CFP.Value[1] = (void *)(op);
     CFP.Value[2] = (void *)(params);
     GlobalEvent(&CFP);
-#endif
     return 1;
+	}
 }
 
 int command_tell (object *op, char *params)
 {
     char buf[MAX_BUF],*name = NULL ,*msg = NULL;
-    char buf2[MAX_BUF];
     player *pl;
+
     if ( params != NULL){
         name = params;
         msg = strchr(name, ' ');
@@ -95,34 +141,31 @@ int command_tell (object *op, char *params)
 	new_draw_info(NDI_UNIQUE, 0,op,"Tell whom what?");
 	return 1;
     } else if ( msg == NULL){
-	sprintf(buf, "Tell %s what?", name);
-	new_draw_info(NDI_UNIQUE, 0,op,buf);
+	new_draw_info_format(NDI_UNIQUE, 0,op,"Tell %s what?", name);
 	return 1;
     }
 
-    sprintf(buf,"%s tells you: ",op->name);
-    strncat(buf, msg, MAX_BUF-strlen(buf)-1);
-    buf[MAX_BUF-1]=0;
+    snprintf(buf,MAX_BUF-1, "%s tells you: %s",op->name, msg);
 
-    for(pl=first_player;pl!=NULL;pl=pl->next)
-      if(strncasecmp(pl->ob->name,name,MAX_NAME)==0)
-      {
-        new_draw_info(NDI_UNIQUE | NDI_ORANGE, 0, pl->ob, buf);
-        sprintf(buf2, "You tell to %s: %s",name,msg);
-        new_draw_info(NDI_UNIQUE | NDI_ORANGE, 0, op, buf2);
+    pl = find_player_partial_name( name );
 
-        /* Update last_tell value [mids 01/14/2002] */
-        strcpy(pl->last_tell, op->name);
-        return 1;
-      }
-    new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
+    if ( pl )
+        {
+	    new_draw_info(NDI_UNIQUE | NDI_ORANGE, 0, pl->ob, buf);
+	    new_draw_info_format(NDI_UNIQUE | NDI_ORANGE, 0, op, 
+			     "You tell %s: %s", pl->ob->name, msg);
+
+	    /* Update last_tell value [mids 01/14/2002] */
+	    strcpy(pl->last_tell, op->name);
+	    return 1;
+        }
+
+    new_draw_info(NDI_UNIQUE, 0,op,"No such player or ambiguous name.");
     return 1;
-  }
+}
 
 /* Reply to last person who told you something [mids 01/14/2002] */
 int command_reply (object *op, char *params) {
-    char buf[MAX_BUF];
-    char buf2[MAX_BUF];
     player *pl;
 
     if (params == NULL) {
@@ -130,7 +173,6 @@ int command_reply (object *op, char *params) {
         return 1;
     }
 
-
     if (op->contr->last_tell[0] == '\0') {
         new_draw_info(NDI_UNIQUE, 0, op, "You can't reply to nobody.");
         return 1;
@@ -144,17 +186,13 @@ int command_reply (object *op, char *params) {
         return 1;
     }
 
-    sprintf(buf, "%s tells you: ", op->name);
-    strncat(buf, params, MAX_BUF-strlen(buf)-1);
-    buf[MAX_BUF-1] = 0;
-
     /* Update last_tell value */
     strcpy(pl->last_tell, op->name);
 
-    new_draw_info(NDI_UNIQUE | NDI_ORANGE, 0, pl->ob, buf);
-    sprintf(buf2, "You tell to %s: %s", pl->ob->name, params);
-    new_draw_info(NDI_UNIQUE | NDI_ORANGE, 0, op, buf2);
-
+    new_draw_info_format(NDI_UNIQUE | NDI_ORANGE, 0, pl->ob, 
+	"%s tells you: %s", op->name, params);
+    new_draw_info_format(NDI_UNIQUE | NDI_ORANGE, 0, op, 
+		  "You tell to %s: %s", pl->ob->name, params);
     return 1;
 }
 
@@ -692,8 +730,7 @@ static int basic_emote(object *op, char *params, int emotion)
 		return(0);
 	    }/*if self*/
 	}/*for*/
-	sprintf(buf, "%s is not around.", params);
-	new_draw_info(NDI_UNIQUE, 0, op, buf);
+	new_draw_info_format(NDI_UNIQUE, 0, op, "%s is not around.", params);
 	return(1);
     } /*else*/
 
