@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_object_c =
- *   "$Id: c_object.c,v 1.38 2002/09/11 06:21:46 mwedel Exp $";
+ *   "$Id: c_object.c,v 1.57 2004/04/26 05:07:52 mwedel Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -29,11 +29,12 @@
 
 #include <global.h>
 #include <loader.h>
+#include <skills.h>
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
 #include <living.h>
-
+#include <math.h>
 /*
  * Object id parsing functions
  */
@@ -71,31 +72,396 @@ object *find_best_object_match(object *pl, char *params)
     }
     return best;
 }
+/* Simlilar to find_best_object_match , but accepts an
+ * additional parameter for apply -u , and apply -a to
+ * only unapply applied , or apply unapplied objects
+ */
+static object *find_best_apply_object_match(object *pl, char *params, enum apply_flag aflag)
+{
+    object *tmp, *best=NULL;
+    int match_val=0,tmpmatch;
+
+    for (tmp=pl->inv; tmp; tmp=tmp->below) {
+	if (tmp->invisible) continue;
+	if ((tmpmatch=item_matched_string(pl, tmp, params))>match_val) {
+	  if ((aflag==AP_APPLY) && (QUERY_FLAG(tmp,FLAG_APPLIED))) continue;
+	  if ((aflag==AP_UNAPPLY) && (!QUERY_FLAG(tmp,FLAG_APPLIED))) continue;
+	    match_val=tmpmatch;
+	    best=tmp;
+	}
+    }
+    return best;
+}
+
+/*
+ * Notes about item creation:
+ * 1) It is similar in syntax to the dm create command.
+ * 2) It requires a player to have a buildfacility below him, a tool in his
+ *    possesion, and materials to build with.
+ * 3) The random roll is done in a loop, so if the player tries to make 100,
+ *    he makes 100 checks.
+ * 4) Exp is given only on succ. creations, but materials are used regardless.
+ * 5) The properties of the tool are stored in tooltype and weapontype.
+ * 6) The properties of the buildfacilities are stored in tooltype.
+ * 7) For now, all ingredients must be type 73 INORGANIC.
+ * 8) The player can attempt to create any arch, but item_power and value
+ *    will prevent most artifacts from being built.
+ * 9) The code allows magic bonuses up to +5.  It is not trivial to make a +5
+ *    item.
+ *10) If you ever extend it beyond +5, add more gemtypes.  Currently the code
+ *    looks for gemcost gems per item, per plus.  So a +5 item requires
+ *    gemcost pearls,rubies,emeralds,sapphires and diamonds.  Not cheap.
+ *11) There are a zillion if statements in this code.  Alot of checking takes
+ *    place here.  All of them are needed.
+ */
+
+int command_build (object *pl, char *params) {
+    return 0;
+#if 0
+    object *marked, *facility, *tool, *newobj, *tmp;
+    archetype *at;
+    int skillnr, obpow, number, bonus, mneed, nrof, magic, i, nummade, found;
+    int gemcost;
+    char *bp;
+    materialtype_t *mt;
+
+    /* NOTE THIS FUNCTION IS CURRENTLY DISABLED */
+
+/*    Given this is currently disabled, I'm not going to bother updating
+ * it with the new skill system.  IT really needs to get the skill object
+ * pointer in a better fashion than it is currently doing.
+ */
+    if (!params) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "Usage:build [nr] [+magic] <object>");
+	return 0;
+    }
+    marked = find_marked_object(pl);
+    if (marked == NULL || !marked->material || marked->materialname == NULL ||
+	marked->type != INORGANIC) {
+        new_draw_info(NDI_UNIQUE, 0, pl, "You must mark some ingredients.");
+        return 0;
+    }
+    while (*params==' ')
+        params++;
+    bp = params;
+    nrof = 1;
+    magic = 0;
+
+    if (sscanf(bp, "%d ", &nrof)) {
+	if ((bp = strchr(params, ' ')) == NULL) {
+	    new_draw_info(NDI_UNIQUE, 0, pl,
+		"Usage: build [nr] [+magic] <object>");
+	    return 0;
+	}
+	bp++;
+    }
+    if (sscanf(bp, "+%d ", &magic)) {
+        if ((bp = strchr(bp, ' ')) == NULL) {
+	    new_draw_info(NDI_UNIQUE, 0, pl,
+		"Usage: build [nr] [+magic] <object>");
+	    return 0;
+	}
+	bp++;
+    }
+    while (*bp==' ')
+	bp++;
+    at=find_archetype_by_object_name(bp);
+    if (at == NULL) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl,
+	    "You don't know how to make a %s.", bp);
+	return 0;
+    }
+    newobj = get_object();
+    copy_object(&at->clone, newobj);
+
+    skillnr = -1;
+
+    if ((IS_ARMOR(newobj) && newobj->material != M_LEATHER) ||
+	newobj->type == WEAPON)
+	skillnr = SK_SMITH;
+
+    if (IS_ARMOR(newobj) && newobj->material == M_LEATHER)
+	skillnr = SK_WOODSMAN;
+    
+    if (newobj->type == BOW || newobj->type == ARROW)
+	skillnr = SK_BOWYER;
+
+    if (skillnr == -1) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "You don't know how to create that.");
+	return 0;
+    }
+
+    if (!change_skill(pl, skillnr, 0)) {
+	new_draw_info(NDI_UNIQUE, 0, pl,
+	    "You lack the needed skill to make that item.");
+	return 0;
+    }
+    facility = NULL;
+    for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+	if (tmp->type == BUILDFAC && tmp->tooltype == newobj->type)
+	    facility=tmp;
+    if (facility == NULL) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "You lack a suitable workspace.");
+	return 0;
+    }
+    if (magic && !(IS_ARMOR(newobj) || IS_WEAPON(newobj))) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "A magical bonus is only valid with "
+	    "armour and weapons.");
+	return 0;
+    }
+
+    /* use newobj->weapontype == tool->weapontype for building weapons */
+    /* use newobj->material == tool->weapontype for building armour */
+    /* newobj->type == tool->tooltype */
+    tool = NULL;
+    for (tmp=pl->inv; tmp; tmp=tmp->below) {
+	if (tmp->type != TOOL)
+	    continue;
+	if (IS_ARMOR(newobj) && (newobj->material & tmp->weapontype) &&
+	    newobj->type == tmp->tooltype) {
+	    if (tool == NULL ||
+		(tool->level + tool->magic) < (tmp->level + tmp->magic))
+		tool = tmp;
+	} else if (IS_WEAPON(newobj) && (newobj->weapontype&tmp->weapontype) &&
+		   newobj->type == tmp->tooltype) {
+	    if (tool == NULL ||
+		(tool->level + tool->magic) < (tmp->level + tmp->magic))
+		tool = tmp;
+	}
+	/* should split off bows arrows and probably bolts around here */
+    }
+    if (tool == NULL) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "You lack the required tools.");
+	return 0;
+    }
+
+    mt = name_to_material(marked->materialname);
+    if (mt == NULL) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "Your raw materials are garbage.");
+	return 0;
+    }
+    if (magic < 0) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "You cannot create cursed objects.");
+	return 0;
+    }
+    if (magic > 0 && SK_level(pl)/20 < magic) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "You are not powerful enough to "
+	    "create such a magical item.");
+	return 0;
+    }
+
+    gemcost = 100;
+    if (newobj->type == ARROW)
+	gemcost = 1;
+    if (magic > 0) {
+	found = 0;
+	for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+	    if (tmp->type == GEM && !strcmp(tmp->arch->name, "pearl") &&
+		tmp->nrof >= gemcost*nrof*mt->value/100)
+		found++;
+	if (magic > 1)
+	    for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+		if (tmp->type == GEM && !strcmp(tmp->arch->name, "emerald") &&
+		    tmp->nrof >= gemcost*nrof*mt->value/100)
+		    found++;
+	if (magic > 2)
+	    for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+		if (tmp->type == GEM && !strcmp(tmp->arch->name, "sapphire") &&
+		    tmp->nrof >= gemcost*nrof*mt->value/100)
+		    found++;
+	if (magic > 3)
+	    for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+		if (tmp->type == GEM && !strcmp(tmp->arch->name, "ruby") &&
+		    tmp->nrof >= gemcost*nrof*mt->value/100)
+		    found++;
+	if (magic > 4)
+	    for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+		if (tmp->type == GEM && !strcmp(tmp->arch->name, "diamond") &&
+		    tmp->nrof >= gemcost*nrof*mt->value/100)
+		    found++;
+	if (found < magic) {
+	    new_draw_info(NDI_UNIQUE, 0, pl, "You did not provide a suitable "
+	        "sacrifice of gems on the ground to add this much magic.");
+	    return 0;
+	}
+	if (25*pow(3, magic)*mt->value/100 > pl->stats.sp) {
+	    new_draw_info(NDI_UNIQUE, 0, pl, "You do not have enough mana "
+		"to create this object.");
+	    return 0;
+	}
+    }
+
+    /* good lord.  Now we have a tool, facilites, materials (marked) and an
+       object we want to create.  Thats alot of if's */
+
+    obpow = (newobj->item_power + newobj->value/1000 + 1)*mt->value/100;
+    mneed = nrof*((newobj->weight * mt->weight)/80);
+    /* cost can be balanced out by cost to disassemble items for materials */
+    if ((marked->weight * MAX(1, marked->nrof)) < mneed) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl, "You do not have enough %s.",
+	    marked->name);
+	return 0;
+    }
+    if (obpow > (tool->level+tool->magic)) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl, "Your %s is not capable of "
+	    "crafting such a complex item.", tool->name);
+	return 0;
+    }
+    set_abs_magic(newobj, magic);
+    set_materialname(newobj, 1, mt);
+    for (i=0, nummade=0; i< nrof; i++) {
+	bonus = tool->level+tool->magic - obpow;
+	number = rndm(1, 3*obpow*magic);
+	LOG(llevDebug, "command_build: skill:%d obpow:%d rndm:%d tool:%s "
+	    "newobj:%s marked:%s magic:%d\n", SK_level(pl)+bonus, obpow,
+	    number, tool->name, newobj->name, marked->name, magic);
+	if (SK_level(pl)+bonus > number) {
+	    /* wow, we actually created something */
+	    newobj->x = pl->x;
+	    newobj->y = pl->y;
+	    newobj->map = pl->map;
+	    SET_FLAG(newobj, FLAG_IDENTIFIED);
+	    if (i == 0)
+		newobj = insert_ob_in_ob(newobj, pl);
+	    else
+		newobj->nrof++;
+	    esrv_send_item(pl, newobj);
+	    nummade++;
+	} else {
+	    free_object(newobj);
+	    if (bonus < rndm(1, number-SK_level(pl)+bonus)) {
+		new_draw_info_format(NDI_UNIQUE, 0, pl,
+		    "You broke your %s!\n", tool->name);
+		esrv_del_item(pl->contr, tool->count);
+		remove_ob(tool);
+		free_object(tool);
+		break;
+	    }
+	}
+	/* take away materials too */
+	tmp = get_split_ob(marked, MAX(1, mneed/marked->weight));
+	if (tmp)
+	    free_object(tmp);
+	if (marked->nrof < 1)
+	    esrv_del_item(pl->contr, marked->count);
+	else
+	    esrv_send_item(pl, marked);
+    }
+    if (magic)
+	for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+	    if (tmp->type == GEM && !strcmp(tmp->arch->name, "pearl") &&
+		tmp->nrof >= gemcost*nrof*mt->value/100)
+		tmp->nrof -= gemcost*nrof*mt->value/100;
+    if (magic > 1)
+	for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+	    if (tmp->type == GEM && !strcmp(tmp->arch->name, "emerald") &&
+		tmp->nrof >= gemcost*nrof*mt->value/100)
+		tmp->nrof -= gemcost*nrof*mt->value/100;
+    if (magic > 2)
+	for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+	    if (tmp->type == GEM && !strcmp(tmp->arch->name, "sapphire") &&
+		tmp->nrof >= gemcost*nrof*mt->value/100)
+		tmp->nrof -= gemcost*nrof*mt->value/100;
+    if (magic > 3)
+	for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+	    if (tmp->type == GEM && !strcmp(tmp->arch->name, "ruby") &&
+		tmp->nrof >= gemcost*nrof*mt->value/100)
+		tmp->nrof -= gemcost*nrof*mt->value/100;
+    if (magic > 4)
+	for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+	    if (tmp->type == GEM && !strcmp(tmp->arch->name, "diamond") &&
+		tmp->nrof >= gemcost*nrof*mt->value/100)
+		tmp->nrof -= gemcost*nrof*mt->value/100;
+    if (magic)
+	for (tmp=GET_MAP_OB(pl->map, pl->x, pl->y); tmp; tmp=tmp->above)
+	    if (tmp->type == GEM &&
+		(!strcmp(tmp->arch->name, "diamond") ||
+		 !strcmp(tmp->arch->name, "ruby") ||
+		 !strcmp(tmp->arch->name, "sapphire") ||
+		 !strcmp(tmp->arch->name, "emerald") ||
+		 !strcmp(tmp->arch->name, "pearl"))) {
+		if (tmp->nrof == 0) {
+		    remove_ob(tmp);
+		    free_object(tmp);
+		}
+		if (pl->contr)
+		    pl->contr->socket.update_look=1;
+	    }
+    pl->stats.sp -= 25*pow(3, magic)*mt->value/100;
+    fix_player(pl);
+    if (nummade > 1) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl, "You have created %d %s.",
+	    nummade, query_base_name(newobj, 1));
+    } else if (nummade == 1) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl, "You have created a %s.",
+	    query_base_name(newobj, 0));
+    } else {
+	new_draw_info_format(NDI_UNIQUE, 0, pl,
+	    "You have failed to craft a %s.", query_base_name(newobj, 0));
+	return 0;
+    }
+    if (skills[skillnr].category != EXP_NONE)
+	add_exp(pl, obpow*nummade);
+    return 1;
+#endif
+}
+
 
 
 int command_uskill ( object *pl, char *params) {
    if (!params) {
-        new_draw_info(NDI_UNIQUE, 0,pl, "Usage: use_skill <skill name>");
+        new_draw_info(NDI_UNIQUE, 0, pl, "Usage: use_skill <skill name>");
         return 0;
    }
    return use_skill(pl,params);
 }
 
 int command_rskill ( object *pl, char *params) {
-   int skillno;
+    object *skill;
 
-   if (!params) {
-        new_draw_info(NDI_UNIQUE, 0,pl, "Usage: ready_skill <skill name>");
-        return 0;
-   }
-   skillno=lookup_skill_by_name(params);
-   if (skillno==-1) {
-	new_draw_info_format(NDI_UNIQUE,0,pl,"Couldn't find the skill %s", params);
+    if (!params) {
+	new_draw_info(NDI_UNIQUE, 0, pl, "Usage: ready_skill <skill name>");
 	return 0;
-   }
-   return change_skill(pl,skillno);
+    }
+    skill = find_skill_by_name(pl, params);
+
+    if (!skill) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl, "You have no knowledge of the skill %s", params);
+	return 0;
+    }
+    return change_skill(pl,skill, 0);
 }
 
+
+/* These functions (command_search, command_disarm) are really juse wrappers for
+ * things like 'use_skill ...').  In fact, they should really be obsoleted
+ * and replaced with those.
+ */
+int command_search (object *op, char *params) {
+    return command_uskill(op, skill_names[SK_FIND_TRAPS]);
+}
+
+int command_disarm (object *op, char *params) {
+    return command_uskill(op, skill_names[SK_DISARM_TRAPS]);
+}
+
+
+/* A little special because we do want to pass the full params along
+ * as it includes the object to throw.
+ */  
+int command_throw (object *op, char *params)
+{
+    object *skop;
+
+    skop = find_skill_by_name(op, skill_names[SK_THROWING]);
+    if (skop) return do_skill(op, op, skop, op->facing,params);
+    else {
+	new_draw_info(NDI_UNIQUE, 0, op, "You have no knowledge of the skill throwing.");
+    }
+    return 0;
+}
+
+
 int command_apply (object *op, char *params)
 {
   if (!params) {
@@ -117,7 +483,7 @@ int command_apply (object *op, char *params)
     }
     while (*params==' ') params++;
 
-    inv=find_best_object_match(op, params);
+    inv=find_best_apply_object_match(op, params, aflag);
     if (inv) {
 	player_apply(op,inv,aflag,0);
     } else
@@ -137,28 +503,36 @@ int command_apply (object *op, char *params)
  * not need to use split_ob and stuff.
  */
 int sack_can_hold (object *pl, object *sack, object *op, int nrof) {
-    char buf[MAX_BUF];
-    buf[0] = 0;
 
-    if (! QUERY_FLAG (sack, FLAG_APPLIED))
-	sprintf (buf, "The %s is not active.", query_name(sack));
-    if (sack == op)
-	sprintf (buf, "You can't put the %s into itself.", query_name(sack));
-    if (sack->race && (sack->race != op->race || op->type == CONTAINER
-		       || (sack->stats.food && sack->stats.food != op->type)))
-	sprintf (buf, "You can put only %s into the %s.", sack->race,
-		 query_name(sack));
-    if (op->type == SPECIAL_KEY && sack->slaying && op->slaying)
-	sprintf (buf, "You don't want put the key into %s.", query_name(sack));
-    if (sack->weight_limit && sack->carrying + (nrof ? nrof : 1) * 
-	(op->weight + (op->type==CONTAINER?(op->carrying*op->stats.Str):0))
-	* (100 - sack->stats.Str) / 100  > sack->weight_limit)
-	sprintf (buf, "That won't fit in the %s!", query_name(sack));
-    if (buf[0]) {
-	if (pl)
-	    new_draw_info(NDI_UNIQUE, 0,pl, buf);
+    if (! QUERY_FLAG (sack, FLAG_APPLIED)) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl, 
+	     "The %s is not active.", query_name(sack));
 	return 0;
     }
+    if (sack == op) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl,
+	    "You can't put the %s into itself.", query_name(sack));
+	return 0;
+    }
+    if (sack->race && (sack->race != op->race || op->type == CONTAINER
+		       || (sack->stats.food && sack->stats.food != op->type))) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl,
+	    "You can put only %s into the %s.", sack->race,  query_name(sack));
+	return 0;
+    }
+    if (op->type == SPECIAL_KEY && sack->slaying && op->slaying) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl,
+	    "You can't want put the key into %s.", query_name(sack));
+	return 0;
+    }
+    if (sack->weight_limit && sack->carrying + (nrof ? nrof : 1) * 
+	(op->weight + (op->type==CONTAINER?(op->carrying*op->stats.Str):0))
+	* (100 - sack->stats.Str) / 100  > sack->weight_limit) {
+	new_draw_info_format(NDI_UNIQUE, 0, pl,
+	     "That won't fit in the %s!", query_name(sack));
+    return 0;
+    }
+    /* All other checks pass, must be OK */
     return 1;
 }
 
@@ -210,16 +584,11 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
     else
         effective_weight_limit = weight_limit[MAX_STAT];
     if ((pl->weight + pl->carrying + weight) > effective_weight_limit) {
-	new_draw_info(NDI_UNIQUE, 0,pl,"That item is too heavy for you to pick up.");
+	new_draw_info(0, 0,pl,"That item is too heavy for you to pick up.");
 	return;
     }
-	
-
-#ifndef REAL_WIZ
-    if(QUERY_FLAG(pl, FLAG_WAS_WIZ))
+    if (settings.real_wiz == FALSE && QUERY_FLAG(pl, FLAG_WAS_WIZ))
 	SET_FLAG(tmp, FLAG_WAS_WIZ);
-#endif
-
     if (nrof != tmp_nrof) {
 	object *tmp2 = tmp;
         tag_t tmp2_tag = tmp2->count;
@@ -370,12 +739,17 @@ void pick_up(object *op,object *alt)
 }
 
 
+/* This takes (picks up) and item.  op is the player
+ * who issued the command.  params is a string to
+ * match against the item name.  Basically, always
+ * returns zero, but that should be improved.
+ */
 int command_take (object *op, char *params)
 {
     object *tmp, *next;
 
     if (op->container) 
-	tmp=op->container;
+	tmp=op->container->inv;
     else {
 	tmp=op->above;
 	if (tmp) while (tmp->above) {
@@ -412,6 +786,10 @@ int command_take (object *op, char *params)
 	    break;
 	}
 	tmp=next;
+	/* Might as well just skip over the player immediately -
+	 * we know it can't be picked up
+	 */
+	if (tmp == op) tmp=tmp->below;
     }
     if (!params && !tmp) {
 	for (tmp=op->below; tmp!=NULL; tmp=tmp->next)
@@ -424,9 +802,6 @@ int command_take (object *op, char *params)
 	    }
 	if (!tmp) new_draw_info(NDI_UNIQUE, 0,op, "There is nothing to pick up.");
     }
-
-/* Shouldn't be needed - pick_up should update this */
-/*    draw_look(op);*/
     return 0;
 }
 
@@ -533,6 +908,7 @@ void drop_object (object *op, object *tmp, long nrof)
 {
     char buf[MAX_BUF];
     object *floor;
+    event *evt;
 
     if (QUERY_FLAG(tmp, FLAG_NO_DROP)) {
 #if 0
@@ -570,9 +946,8 @@ void drop_object (object *op, object *tmp, long nrof)
 	};
     } else
       remove_ob (tmp);
-#ifdef PLUGINS
       /* GROS: Handle for plugin drop event */
-      if(tmp->event_hook[EVENT_DROP] != NULL)
+      if ((evt = find_event(tmp, EVENT_DROP)) != NULL)
       {
         CFParm CFP;
         CFParm *CFR;
@@ -583,22 +958,21 @@ void drop_object (object *op, object *tmp, long nrof)
         CFP.Value[0] = &k;
         CFP.Value[1] = op;
         CFP.Value[2] = tmp;
-        CFP.Value[3] = &nrof;
+        CFP.Value[3] = NULL;
         CFP.Value[4] = NULL;
-        CFP.Value[5] = &m;
+        CFP.Value[5] = &nrof;
         CFP.Value[6] = &m;
         CFP.Value[7] = &m;
         CFP.Value[8] = &l;
-        CFP.Value[9] = tmp->event_hook[k];
-        CFP.Value[10]= tmp->event_options[k];
-        if (findPlugin(tmp->event_plugin[k])>=0)
+        CFP.Value[9] = evt->hook;
+        CFP.Value[10]= evt->options;
+        if (findPlugin(evt->plugin)>=0)
         {
-          CFR = ((PlugList[findPlugin(tmp->event_plugin[k])].eventfunc) (&CFP));
+          CFR = ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
           rtn_script = *(int *)(CFR->Value[0]);
           if (rtn_script!=0) return;
         }
       }
-#endif
     if (QUERY_FLAG (tmp, FLAG_STARTEQUIP)) {
       sprintf(buf,"You drop the %s.", query_name(tmp));
       new_draw_info(NDI_UNIQUE, 0,op,buf);
@@ -994,54 +1368,57 @@ void examine_monster(object *op,object *tmp) {
 	new_draw_info(NDI_UNIQUE, 0,op,"It looks very ill.");
 }
 
+
 /* tmp is the object being described, pl is who is examing it. */
 char *long_desc(object *tmp, object *pl) {
-  static char buf[VERY_BIG_BUF];
-  char *cp;
+    static char buf[VERY_BIG_BUF];
+    char *cp;
 
-  if(tmp==NULL)
-    return "";
-  buf[0]='\0';
-  switch(tmp->type) {
-  case RING:
-  case SKILL:
-  case WEAPON:
-  case ARMOUR:
-  case BRACERS:
-  case HELMET:
-  case SHIELD:
-  case BOOTS:
-  case GLOVES:
-  case AMULET:
-  case GIRDLE:
-  case BOW:
-  case ARROW:
-  case CLOAK:
-  case FOOD:
-  case DRINK:
-  case FLESH:
-    if(*(cp=describe_item(tmp, pl))!='\0') {
-	int len;
+    if(tmp==NULL)
+	return "";
 
-	strncpy(buf,query_name(tmp), VERY_BIG_BUF-1);
-	buf[VERY_BIG_BUF-1]=0;
-	len=strlen(buf);
-	if (len<VERY_BIG_BUF-5) {
-	    /* Since we know the length, we save a few cpu cycles by using
-	     * it instead of calling strcat */
-	    strcpy(buf+len," ");
-	    len++;
-	    strncpy(buf+len, cp, VERY_BIG_BUF-len-1);
-	    buf[VERY_BIG_BUF-1]=0;
-	}
+    buf[0]='\0';
+    switch(tmp->type) {
+	case RING:
+	case SKILL:
+	case WEAPON:
+	case ARMOUR:
+	case BRACERS:
+	case HELMET:
+	case SHIELD:
+	case BOOTS:
+	case GLOVES:
+	case AMULET:
+	case GIRDLE:
+	case BOW:
+	case ARROW:
+	case CLOAK:
+	case FOOD:
+	case DRINK:
+	case FLESH:
+	case SKILL_TOOL:
+	    if(*(cp=describe_item(tmp, pl))!='\0') {
+		int len;
+
+		strncpy(buf,query_name(tmp), VERY_BIG_BUF-1);
+		buf[VERY_BIG_BUF-1]=0;
+		len=strlen(buf);
+		if (len<VERY_BIG_BUF-5) {
+		    /* Since we know the length, we save a few cpu cycles by using
+		     * it instead of calling strcat */
+		    strcpy(buf+len," ");
+		    len++;
+		    strncpy(buf+len, cp, VERY_BIG_BUF-len-1);
+		    buf[VERY_BIG_BUF-1]=0;
+		}
+	    }
     }
-  }
-  if(buf[0]=='\0') {
+    if(buf[0]=='\0') {
 	strncpy(buf,query_name(tmp), VERY_BIG_BUF-1);
 	buf[VERY_BIG_BUF-1]=0;
-  }
+    }
 
-  return buf;
+    return buf;
 }
 
 void examine(object *op, object *tmp) {
@@ -1057,22 +1434,28 @@ void examine(object *op, object *tmp) {
 
     new_draw_info(NDI_UNIQUE, 0,op,buf);
     buf[0]='\0';
+
+    if(tmp->custom_name) {
+      strcpy(buf,"You name it ");
+      strncat(buf, tmp->custom_name, VERY_BIG_BUF-strlen(buf)-1);
+      buf[VERY_BIG_BUF-1]=0;
+      new_draw_info(NDI_UNIQUE, 0,op,buf);
+      buf[0]='\0';
+    }
+
     switch(tmp->type) {
 	case SPELLBOOK:
-	    if(QUERY_FLAG(tmp, FLAG_IDENTIFIED) && tmp->stats.sp >= 0 && tmp->stats.sp <= NROFREALSPELLS ) {
-		if(!strcmp(tmp->arch->name,"cleric_book"))
-		    sprintf(buf,"%s is a %d level prayer.",
-			    spells[tmp->stats.sp].name,spells[tmp->stats.sp].level);
-		else
-		    sprintf(buf,"%s is a %d level spell.",
-			    spells[tmp->stats.sp].name,spells[tmp->stats.sp].level);
+	    if(QUERY_FLAG(tmp, FLAG_IDENTIFIED) && tmp->inv ) {
+		sprintf(buf,"%s is a %d level %s spell",
+			tmp->inv->name, tmp->inv->level,
+			tmp->inv->skill);
 	    }
-	break;
+	    break;
 
 	case BOOK:
 	    if(tmp->msg!=NULL)
 		strcpy(buf,"Something is written in it.");
-	break;
+	    break;
 
 	case CONTAINER:
 	    if(tmp->race!=NULL) {
@@ -1096,15 +1479,9 @@ void examine(object *op, object *tmp) {
     if(buf[0]!='\0')
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
 
-    if(tmp->material && !tmp->msg) {
-	strcpy(buf,"It is made of: ");
-	for(i=0; i < NROFMATERIALS; i++) {
-	  if(tmp->material & (1<<i)) {
-	    strcat(buf, material[i].name);
-	    strcat(buf, " ");
-	  }
-	}
-	new_draw_info(NDI_UNIQUE, 0,op,buf);
+    if(tmp->materialname != NULL && !tmp->msg) {
+	sprintf(buf, "It is made of: %s.", tmp->materialname);
+	new_draw_info(NDI_UNIQUE, 0, op, buf);
     }
     /* Where to wear this item */
     for (i=0; i < NUM_BODY_LOCATIONS; i++) {
@@ -1131,12 +1508,12 @@ void examine(object *op, object *tmp) {
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
     }
 
-    if(tmp->value&&!QUERY_FLAG(tmp, FLAG_STARTEQUIP)) {
+    if(tmp->value && !QUERY_FLAG(tmp, FLAG_STARTEQUIP) && !QUERY_FLAG(tmp, FLAG_NO_PICK)) {
 	if(QUERY_FLAG(tmp, FLAG_UNPAID))
 	    sprintf(buf,"%s would cost you %s.",
 		    tmp->nrof>1?"They":"It",query_cost_string(tmp,op,F_BUY));
 	else
-	    sprintf(buf,"You would get %s for the %s.",
+	    sprintf(buf,"You would get %s for %s.",
 		    query_cost_string(tmp,op,F_SELL), tmp->nrof>1?"them":"it");
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
     }
@@ -1144,6 +1521,9 @@ void examine(object *op, object *tmp) {
     if(QUERY_FLAG(tmp, FLAG_MONSTER))
 	examine_monster(op,tmp);
    
+    /* Is this item buildable? */
+    if ( QUERY_FLAG( tmp, FLAG_IS_BUILDABLE ) )
+        new_draw_info( NDI_UNIQUE, 0, op, "This is a buildable item." );
 
     /* Does the object have a message?  Don't show message for all object
      * types - especially if the first entry is a match
@@ -1362,10 +1742,13 @@ void set_pickup_mode(object *op,int i) {
     }
 }
 
-#ifdef SEARCH_ITEMS
 int command_search_items (object *op, char *params)
 {
       char buf[MAX_BUF];
+
+      if (settings.search_items == FALSE)
+	  return 1;
+
   if(params == NULL) {
 	if(op->contr->search_str[0]=='\0') {
 	  new_draw_info(NDI_UNIQUE, 0,op,"Example: search magic+1");
@@ -1387,5 +1770,143 @@ int command_search_items (object *op, char *params)
       new_draw_info(NDI_UNIQUE, 0,op,buf);
   fix_player(op);
       return 1;
+}
+
+/*
+ * Changing the custom name of an item
+ * 
+ * Syntax is: rename <what object> to <new name>
+ *  if '<what object>' is omitted, marked object is used
+ *  if 'to <new name>' is omitted, custom name is cleared
+ *
+ * Names are considered for all purpose having a length <=127 (max length sent to client
+ * by server) */
+
+int command_rename_item(object *op, char *params)
+{
+  char buf[VERY_BIG_BUF];
+
+  object *item=NULL;
+  char *closebrace;
+  int counter;
+
+	if (params) {
+    /* Let's skip white spaces */
+    while(' '==*params) params++;
+
+    /* Checking the first part */
+    if ('<'==*params) {
+      /* Got old name, let's get it & find appropriate matching item */
+      closebrace=strchr(params,'>');
+      if(!closebrace) {
+        new_draw_info(NDI_UNIQUE,0,op,"Syntax error!");
+        return 1;
+      }
+      /* Sanity check for buffer overruns */
+      if((closebrace-params)>127) {
+        new_draw_info(NDI_UNIQUE,0,op,"Old name too long (up to 127 characters allowed)!");
+        return 1;
+      }
+      /* Copy the old name */
+      strncpy(buf,params+1,closebrace-params-1);
+      buf[closebrace-params-1]='\0';
+
+      /* Find best matching item */
+      item=find_best_object_match(op,buf);
+      if(!item) {
+        new_draw_info(NDI_UNIQUE,0,op,"Could not find a matching item to rename.");
+        return 1;
+      }
+
+      /* Now need to move pointer to just after > */
+      params=closebrace+1;
+      while(' '==*params) params++;
+
+    } else {
+        /* Use marked item */
+        item=find_marked_object(op);
+        if(!item) {
+          new_draw_info(NDI_UNIQUE,0,op,"No marked item to rename.");
+          return 1;
+        }
     }
-#endif /* SEARCH_ITEMS */
+
+    /* Now let's find the new name */
+    if(!strncmp(params,"to ",3)) {
+      params+=3;
+      while(' '==*params) params++;
+      if('<'!=*params) {
+        new_draw_info(NDI_UNIQUE,0,op,"Syntax error, expecting < at start of new name!");
+        return 1;
+      }
+      closebrace=strchr(params+1,'>');
+      if(!closebrace) {
+        new_draw_info(NDI_UNIQUE,0,op,"Syntax error, expecting > at end of new name!");
+        return 1;
+      }
+
+      /* Sanity check for buffer overruns */
+      if((closebrace-params)>127) {
+        new_draw_info(NDI_UNIQUE,0,op,"New name too long (up to 127 characters allowed)!");
+        return 1;
+      }
+
+      /* Copy the new name */
+      strncpy(buf,params+1,closebrace-params-1);
+      buf[closebrace-params-1]='\0';
+      
+      /* Let's check it for weird characters */
+      for(counter=0;counter<strlen(buf);counter++) {
+        if(isalnum(buf[counter])) continue;
+        if(' '==buf[counter]) continue;
+        if('\''==buf[counter]) continue;
+        if('+'==buf[counter]) continue;
+        if('_'==buf[counter]) continue;
+        if('-'==buf[counter]) continue;
+
+        /* If we come here, then the name contains an invalid character...
+        tell the player & exit */
+        new_draw_info(NDI_UNIQUE,0,op,"Invalid new name!");
+        return 1;
+      }
+
+    } else {
+      /* If param contains something, then syntax error... */
+      if(strlen(params)) {
+        new_draw_info(NDI_UNIQUE,0,op,"Syntax error, expected 'to <' after old name!");
+        return 1;
+      }
+      /* New name is empty */
+      buf[0]='\0';
+    }
+  } else {
+    /* Last case: params==NULL */
+    item=find_marked_object(op);
+    if(!item) {
+      new_draw_info(NDI_UNIQUE,0,op,"No marked item to rename.");
+      return 1;
+    }
+    buf[0]='\0';
+  }
+
+  /* Coming here, everything is fine... */
+  if(!strlen(buf)) {
+     /* Clear custom name */
+    if(item->custom_name) {
+      FREE_AND_CLEAR_STR(item->custom_name);
+
+      new_draw_info_format(NDI_UNIQUE, 0, op,"You stop calling your %s with weird names.",query_base_name(item,item->nrof>1?1:0));
+      esrv_update_item(UPD_NAME,op,item);
+    } else {
+      new_draw_info(NDI_UNIQUE,0,op,"This item has no custom name.");
+    }
+  } else {
+    /* Set custom name */
+    FREE_AND_COPY(item->custom_name,buf);
+
+    new_draw_info_format(NDI_UNIQUE, 0, op,"Your %s will now be called %s.",query_base_name(item,item->nrof>1?1:0),buf);
+    esrv_update_item(UPD_NAME,op,item);
+  }
+
+  return 1;
+}
