@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_party_c =
- *   "$Id: c_party.c,v 1.4 2002/07/15 04:25:40 mwedel Exp $";
+ *   "$Id: c_party.c,v 1.6 2003/09/13 05:02:08 mwedel Exp $";
  */
 
 /*
@@ -30,7 +30,6 @@
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
-#include <version.h>
 #include <spells.h>
 
 static partylist * firstparty=NULL; /* Keeps track of first party in list */
@@ -330,14 +329,16 @@ int command_party (object *op, char *params)
 	"Members of party: %s.",find_party(no,firstparty));
     for(pl=first_player;pl!=NULL;pl=pl->next)
       if(pl->ob->contr->party_number==no) {
-#ifdef SET_TITLE
-        if(pl->ob->contr->own_title[0]!='\0')
-          sprintf(buf,"%3d %s the %s",
-                  pl->ob->level,pl->ob->name,pl->ob->contr->own_title);
-        else
-#endif /* SET_TITLE */
-          sprintf(buf,"%3d %s the %s",
-                  pl->ob->level,pl->ob->name,pl->ob->contr->title);
+	  if (settings.set_title == TRUE) {
+	      if(pl->ob->contr->own_title[0]!='\0')
+		  sprintf(buf,"%3d %s the %s",
+		      pl->ob->level,pl->ob->name,pl->ob->contr->own_title);
+	      else
+		  sprintf(buf,"%3d %s the %s",
+		      pl->ob->level,pl->ob->name,pl->ob->contr->title);
+	  } else
+	      sprintf(buf,"%3d %s the %s",
+		  pl->ob->level,pl->ob->name,pl->ob->contr->title);
         new_draw_info(NDI_UNIQUE, 0,op,buf);
       }
     return 1;
