@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_wiz_c =
- *   "$Id: c_wiz.c,v 1.23 2002/09/07 22:08:07 mwedel Exp $";
+ *   "$Id: c_wiz.c,v 1.40 2004/03/26 21:59:28 ryo_saeba Exp $";
  */
 
 /*
@@ -37,11 +37,91 @@
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
-#include <version.h>
 #include <spells.h>
 #include <treasure.h>
 #include <skills.h>
 
+/* Enough of the DM functions seem to need this that I broke
+ * it out to a seperate function.  name is the person
+ * being saught, rq is who is looking for them.  This
+ * prints diagnostics messages, and returns the 
+ * other player, or NULL otherwise.
+ */
+static player *get_other_player_from_name(object *op, char *name)
+{
+    player *pl;
+
+    if (!name) return NULL;
+
+    for(pl=first_player;pl!=NULL;pl=pl->next) 
+	if(!strncmp(pl->ob->name, name, MAX_NAME)) 
+	    break;
+
+    if(pl==NULL) {
+	new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
+	return NULL;
+    }
+
+    if (pl->ob == op) {
+	new_draw_info(NDI_UNIQUE, 0, op, "You can't do that to yourself.");
+	return NULL;
+    }
+    if(pl->state != ST_PLAYING) {
+	new_draw_info(NDI_UNIQUE, 0,op,"That player is in no state for that right now.");
+	return NULL;
+    }
+    return pl;
+}
+
+
+int command_loadtest(object *op, char *params){
+	int x,y;
+	char buf[1024];
+	/*This command will stress server*/
+	new_draw_info(NDI_UNIQUE, 0,op, "loadtest will stress server through teleporting");
+	new_draw_info(NDI_UNIQUE, 0,op, "at different map places.");
+	new_draw_info(NDI_UNIQUE, 0,op, "use at your own risks.");
+	new_draw_info(NDI_UNIQUE, 0,op, "Very long loop used so server may have to be reset.");
+	new_draw_info(NDI_UNIQUE, 0,op, "type loadtest TRUE to run");
+	new_draw_info_format(NDI_UNIQUE, 0, op,"{%s}",params);
+	if (!params)
+		return 0;
+	if (strncmp (params,"TRUE",4))
+		return 0;
+	new_draw_info_format(NDI_UNIQUE, 0, op,"gogogo",params);
+	for (x=0; x<settings.worldmaptilesx; x++)
+	for (y=0; y<settings.worldmaptilesy; y++){
+		sprintf (buf,"/world/world_%d_%d",x+settings.worldmapstartx,y+settings.worldmapstarty);
+		/*new_draw_info_format(NDI_UNIQUE, 0, op,"going to %s",buf);*/
+		command_goto (op, buf);
+	}
+	return 0;
+}
+
+int command_hide(object *op, char *params)
+{
+    if (op->contr->hidden) {
+        op->contr->hidden=0;
+        op->invisible=1;
+        new_draw_info(NDI_UNIQUE, 0,op, "You are no longer hidden from other players");
+	    op->map->players++;
+	    new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, NULL,
+            "%s has entered the game.",op->name);
+        new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_LT_GREEN, 1, NULL,
+            "The Dungeon Master has arrived!");    }
+    else {
+        op->contr->hidden=1;
+        new_draw_info(NDI_UNIQUE, 0,op, "Other players will no longer see you.");
+	    op->map->players--;
+        new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_LT_GREEN, 1, NULL,
+	        "The Dungeon Master is gone..");
+	    new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, NULL,
+            "%s left the game.",op->name);
+    }
+    return 1;
+}
+
+
 /* This finds and returns the object which matches the name or
  * object nubmer (specified via num #whatever).
  */
@@ -58,7 +138,7 @@ static object *find_object_both(char *params)
  */
 int command_setgod(object *op, char *params)
 {
-    object *ob;
+    object *ob, *god;
     char    *str;
 
     if (!params || !(str=strchr(params,' '))) {
@@ -78,43 +158,82 @@ int command_setgod(object *op, char *params)
     	new_draw_info_format(NDI_UNIQUE,0,op,"%s is not a player - can not change its god",ob->name);
     	return 1;
     }
-    change_skill(ob,SK_PRAYING);
-    if(!ob->chosen_skill||ob->chosen_skill->stats.sp!=SK_PRAYING) {
-    	new_draw_info_format(NDI_UNIQUE,0,op,"%s doesn't have praying skill.",ob->name);
-    	return 1;
-    }
-    if(find_god(str)==NULL) {
+    god = find_god(str);
+    if(god==NULL) {
     	new_draw_info_format(NDI_UNIQUE,0,op,"No such god %s.",str);
     	return 1;
     }
-    become_follower(ob,find_god(str));
+    become_follower(ob,god);
     return 1;
 }
 
+/*Add player's IP to ban_file and kick them off the server
+* I know most people have dynamic IPs but this is more of a short term
+* solution if they have to get a new IP to play maybe they'll calm down.
+* This uses the banish_file in the local directory *not* the ban_file
+* The action is logged with a ! for easy searching. -tm
+*/
+int command_banish (object *op, char *params)
+  {
+	  player *pl;
+	  FILE *banishfile;
+	  char  buf[MAX_BUF];
+	  
+	if (!params) {
+         new_draw_info(NDI_UNIQUE, 0,op,"Usage: banish <player>.");
+         return 1;
+    }
+	
+	pl = get_other_player_from_name(op, params);
+	if (!pl) return 1;
+	
+	sprintf (buf, "%s/%s", settings.localdir, BANISHFILE);
+	
+    if ((banishfile = fopen(buf, "a")) == NULL) {
+    	LOG (llevDebug, "Could not find file banish_file.\n");
+		new_draw_info(NDI_UNIQUE,0,op,"Could not find banish_file.");
+    	return(0);
+  		}
+		
+  	fprintf(banishfile,"*@%s\n",pl->socket.host);
+	LOG (llevDebug, "! %s banned %s from IP: %s.\n", op->name, pl->ob->name, pl->socket.host);
+  	new_draw_info_format(NDI_UNIQUE | NDI_RED, 0,op,"You banish %s", pl->ob->name);
+	new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_RED, 5, op,
+			     "%s banishes %s from the land!", op->name, pl->ob->name);
+	command_kick(op, pl->ob->name);
+    fclose(banishfile);
+	return 1;
+  }
+  
 int command_kick (object *op, char *params)
 {
-  struct pl *pl;
+    struct pl *pl;
 
-  for(pl=first_player;pl!=NULL;pl=pl->next) 
-    if((params==NULL || !strcmp(pl->ob->name,params)) && pl->ob!=op)
-      {
-	object *op;
+    for(pl=first_player;pl!=NULL;pl=pl->next) 
+	if((params==NULL || !strcmp(pl->ob->name,params)) && pl->ob!=op) {
+	    object *op;
+	    int removed=0;
 
-	op=pl->ob;
-	remove_ob(op);
-	op->direction=0;
-	new_draw_info_format(NDI_UNIQUE | NDI_ALL, 5, op,
+	    op=pl->ob;
+	    if (!QUERY_FLAG(op, FLAG_REMOVED)) {
+		remove_ob(op);
+		removed=1;
+	    }
+	    op->direction=0;
+	    new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_RED, 5, op,
 			     "%s is kicked out of the game.",op->name);
-	strcpy(op->contr->killer,"left");
-	check_score(op); /* Always check score */
-	(void)save_player(op,0);
-	play_again(op);
-	op->map->players--;
+	    strcpy(op->contr->killer,"left");
+	    check_score(op); /* Always check score */
+	    if (!removed) {
+		(void)save_player(op,0);
+		op->map->players--;
+	    }
 #if MAP_MAXTIMEOUT
-	op->map->timeout = MAP_TIMEOUT(op->map);
+	    op->map->timeout = MAP_TIMEOUT(op->map);
 #endif
-      }
-  return 1;
+	    pl->socket.status = Ns_Dead;
+	}
+    return 1;
 }
 
 int command_save_overlay(object *op, char *params)
@@ -136,6 +255,38 @@ int command_save_overlay(object *op, char *params)
 
     return(1);
 } 
+/*a simple toggle for the no_shout field.
+
+*/
+int command_toggle_shout(object *op, char *params)
+{
+	player *pl;
+	
+	if (!params) {
+         new_draw_info(NDI_UNIQUE, 0,op,"Usage: toggle_shout <player>.");
+         return 1;
+    }
+
+    pl = get_other_player_from_name(op, params);
+    if (!pl) return 1;
+	
+	if (pl->ob->contr->no_shout == 0){
+		pl->ob->contr->no_shout = 1;
+    new_draw_info(NDI_UNIQUE | NDI_RED, 0, pl->ob,
+			 "You have been muzzled by the DM!");
+    new_draw_info_format(NDI_UNIQUE , 0, op,
+			 "You toggle shout for %s.", pl->ob->name);
+		return 1;
+	}else{
+		pl->ob->contr->no_shout = 0;
+    new_draw_info(NDI_UNIQUE | NDI_ORANGE, 0, pl->ob,
+			 "You are allowed to shout again.");
+    new_draw_info_format(NDI_UNIQUE , 0, op,
+			 "You toggle shout for %s.", pl->ob->name);
+		return 1;
+	}
+	
+}
 
 int command_shutdown(object *op, char *params)
 {
@@ -205,40 +356,7 @@ int command_generate (object *op, char *params)
         }
       }
       return 1;
-    }
-
-
-/* Enough of the DM functions seem to need this that I broke
- * it out to a seperate function.  name is the person
- * being saught, rq is who is looking for them.  This
- * prints diagnostics messages, and returns the 
- * other player, or NULL otherwise.
- */
-static player *get_other_player_from_name(object *op, char *name)
-{
-    player *pl;
-
-    if (!name) return NULL;
-
-    for(pl=first_player;pl!=NULL;pl=pl->next) 
-	if(!strncmp(pl->ob->name, name, MAX_NAME)) 
-	    break;
-
-    if(pl==NULL) {
-	new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
-	return NULL;
-    }
-
-    if (pl->ob == op) {
-	new_draw_info(NDI_UNIQUE, 0, op, "You can't summon yourself next to yourself.");
-	return NULL;
-    }
-    if(pl->state != ST_PLAYING) {
-	new_draw_info(NDI_UNIQUE, 0,op,"That player can't be summoned right now.");
-	return NULL;
-    }
-    return pl;
-}
+ }
 
 int command_freeze(object *op, char *params)
 {
@@ -332,7 +450,8 @@ int command_teleport (object *op, char *params) {
    EXIT_Y(dummy) = pl->ob->y + freearr_y[i];
    enter_exit(op, dummy);
    free_object(dummy);
-   new_draw_info(NDI_UNIQUE, 0, pl->ob, "You see a portal open.");
+   if (!op->contr->hidden)
+      new_draw_info(NDI_UNIQUE, 0, pl->ob, "You see a portal open.");
    new_draw_info(NDI_UNIQUE, 0, op, "OK.");
    return 1;
 }
@@ -422,9 +541,8 @@ int command_create (object *op, char *params)
 	if (set_nrof)
 	    tmp->nrof = nrof;
 	tmp->map=op->map;
-#ifndef REAL_WIZ
-	SET_FLAG(tmp, FLAG_WAS_WIZ);
-#endif
+	if (settings.real_wiz == FALSE)
+	    SET_FLAG(tmp, FLAG_WAS_WIZ);
 	if (set_magic)
 	    set_abs_magic(tmp, magic);
 	if (art)
@@ -487,9 +605,8 @@ int command_create (object *op, char *params)
 	object *prev=NULL,*head=NULL;
 	for (atmp=at; atmp!=NULL; atmp=atmp->more) {
 	    tmp=arch_to_object(atmp);
-#ifndef REAL_WIZ
-	    SET_FLAG(tmp, FLAG_WAS_WIZ);
-#endif
+	    if (settings.real_wiz == FALSE)
+		SET_FLAG(tmp, FLAG_WAS_WIZ);
 	    if(head==NULL)
 		head=tmp;
 	    tmp->x=op->x+tmp->arch->clone.x;
@@ -618,6 +735,140 @@ int command_dump (object *op, char *params)
     return 1;
   }
 
+/* When DM is possessing a monster, flip aggression on and off, to allow
+   better motion */
+
+int command_mon_aggr (object *op, char *params)
+{
+    if (op->enemy || !QUERY_FLAG(op, FLAG_UNAGGRESSIVE)) {
+	op->enemy = NULL;
+	SET_FLAG(op, FLAG_UNAGGRESSIVE);
+	new_draw_info(NDI_UNIQUE, 0, op, "Agression turned OFF");
+    } else {
+	CLEAR_FLAG(op, FLAG_FRIENDLY);
+	CLEAR_FLAG(op, FLAG_UNAGGRESSIVE);
+	new_draw_info(NDI_UNIQUE, 0, op, "Agression turned ON");
+    }
+    return 1;
+}
+
+/* DM can possess a monster.  Basically, this tricks the client into thinking
+   a given monster, is actually the player it controls.  This allows a DM
+   to inhabit a monster's body, and run around the game with it. */
+
+int command_possess (object *op, char *params)
+{
+    object *victim, *curinv, *nextinv, *tmp;
+    player *pl;
+    archetype *at;
+    int i;
+    char buf[MAX_BUF];
+
+    victim=NULL;
+    if (params != NULL) {
+	if (sscanf(params, "%d", &i))
+	    victim=find_object(i);
+	else if (sscanf(params, "%s", buf))
+	    victim=find_object_name(buf);
+    }
+    if (victim==NULL) {
+	new_draw_info(NDI_UNIQUE, 0, op, "Patch what object (nr)?");
+	return 1;
+    }
+    if (victim==op) {
+	new_draw_info(NDI_UNIQUE, 0, op, "As insane as you are, I cannot "
+		      "allow you to posess yourself.");
+	return 1;
+    }
+
+    /* clear out the old inventory */
+    curinv = op->inv;
+    while (curinv != NULL) {
+	nextinv = curinv->below;
+	esrv_del_item(op->contr, curinv->count);
+	curinv = nextinv;
+    }
+    /* make the switch */
+    pl = op->contr;
+    victim->contr = pl;
+    pl->ob = victim;
+    victim->type = PLAYER;
+    SET_FLAG(victim, FLAG_WIZ);
+
+    /* exp objects */
+    at = find_archetype("experience_agility");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("experience_charisma");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("experience_mental");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("experience_physical");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("experience_power");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("experience_wis");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    /* need to add basic skills like melee, bows, set chosen skill */
+    at = find_archetype("skill_punching");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("skill_melee_weapon");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("skill_missile_weapon");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("skill_use_magic_item");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("skill_spellcasting");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    at = find_archetype("skill_praying");
+    tmp = get_object();
+    copy_object(&at->clone, tmp);
+    tmp = insert_ob_in_ob(tmp, victim);
+    /* send the inventory to the client */
+    curinv = victim->inv;
+    while (curinv != NULL) {
+	nextinv = curinv->below;
+	esrv_send_item(victim, curinv);
+	curinv = nextinv;
+    }
+    /* basic patchup */
+    for (i=0; i<NUM_BODY_LOCATIONS; i++)
+	if (i == 1 || i == 6 || i == 8 || i == 9)
+	    victim->body_info[i] = 2;
+        else
+	    victim->body_info[i] = 1;
+
+    esrv_new_player(pl, 80); /* just pick a wieght, we don't care */
+    esrv_send_inventory(victim, victim);
+    
+    fix_player(victim);
+
+    do_some_living(victim);
+    return 1;
+}
+
+
 int command_patch (object *op, char *params)
 {
     int i;
@@ -645,9 +896,8 @@ int command_patch (object *op, char *params)
     }
     if((arg2=strchr(++arg,' ')))
       arg2++;
-#ifndef REAL_WIZ
-    SET_FLAG(tmp, FLAG_WAS_WIZ); /* To avoid cheating */
-#endif
+    if (settings.real_wiz == FALSE)
+	SET_FLAG(tmp, FLAG_WAS_WIZ); /* To avoid cheating */
     if(set_variable(tmp,arg) == -1)
       new_draw_info_format(NDI_UNIQUE, 0,op,"Unknown variable %s", arg);
     else {
@@ -660,15 +910,20 @@ int command_patch (object *op, char *params)
 int command_remove (object *op, char *params)
 {
     int i;
-  object *tmp;
+    object *tmp;
 
-  if(params==NULL || !sscanf(params, "%d", &i) || (tmp=find_object(i))==NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op,"Remove what object (nr)?");
-      return 1;
+    if(params==NULL || !sscanf(params, "%d", &i) || (tmp=find_object(i))==NULL) {
+	new_draw_info(NDI_UNIQUE, 0,op,"Remove what object (nr)?");
+	return 1;
+    }
+    if (QUERY_FLAG(tmp, FLAG_REMOVED)) {
+	new_draw_info_format(NDI_UNIQUE, 0,op,"%s is already removed!",
+			     query_name(tmp));
+	return 1;
     }
     remove_ob(tmp);
     return 1;
-  }
+}
 
 int command_free (object *op, char *params)
 {
@@ -683,47 +938,50 @@ int command_free (object *op, char *params)
     return 1;
   }
 
+/* This adds exp to a player.  We now allow adding to a specific skill.
+ */
 int command_addexp (object *op, char *params)
 {
-    char buf[MAX_BUF];
-    int i;
-    object *exp_ob,*skill;
+    char buf[MAX_BUF], skill[MAX_BUF];
+    int i, q;
+    object *skillob=NULL;
     player *pl;
 
-  if(params==NULL || sscanf(params, "%s %d", buf, &i)!=2) {
-       new_draw_info(NDI_UNIQUE, 0,op,"Usage: addexp [who] [how much].");
-       return 1;
+    if(params==NULL || ((q=sscanf(params, "%s %d", buf, &i))<2)) {
+	new_draw_info(NDI_UNIQUE, 0,op,"Usage: addexp [who] [how much].");
+	return 1;
     }
+
     for(pl=first_player;pl!=NULL;pl=pl->next) 
-      if(!strncmp(pl->ob->name,buf,MAX_NAME)) 
-        break;
+	if(!strncmp(pl->ob->name,buf,MAX_NAME)) 
+	    break;
+
     if(pl==NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
-      return 1;
+	new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
+	return 1;
     }
 
-/* In new system: for dm adding experience to a player, only can 
- * add exp if we satisfy the following: 
- * 1) there is an associated skill readied by the player 
- * 2) added exp doesnt result in exp_ob->stats.exp>MAX_EXP_IN_OBJ 
- */
-
-    if((skill = pl->ob->chosen_skill) && ((exp_ob = pl->ob->chosen_skill->exp_obj)
-       || link_player_skill(pl->ob, skill))) { 
-      i = check_dm_add_exp_to_obj(exp_ob,i);
-      exp_ob->stats.exp += i;
-    } else {
-      new_draw_info(NDI_UNIQUE, 0,op,"Can't find needed experience object.");
-      new_draw_info(NDI_UNIQUE, 0,op,"Player has no associated skill readied.");
-      return 1;
+    if (q >= 3) {
+	skillob = find_skill_by_name(pl->ob, skill);
+	if (!skillob) {
+	    new_draw_info_format(NDI_UNIQUE, 0,op,"Unable to find skill %s in %s", skill, buf);
+	    return 1;
+	}
+	i = check_exp_adjust(skillob,i);
+	skillob->stats.exp += i;
+	calc_perm_exp(skillob);
+	player_lvl_adj(pl->ob, skillob);
     }
+
+
     pl->ob->stats.exp += i;
-    add_exp(pl->ob,0);
-#ifndef REAL_WIZ
-    SET_FLAG(pl->ob, FLAG_WAS_WIZ);
-#endif
+    calc_perm_exp(pl->ob);
+    player_lvl_adj(pl->ob, NULL);
+
+    if (settings.real_wiz == FALSE)
+	SET_FLAG(pl->ob, FLAG_WAS_WIZ);
     return 1;
-  }
+}
 
 int command_speed (object *op, char *params)
 {
@@ -767,8 +1025,13 @@ int command_stats (object *op, char *params)
          sprintf(buf,"Con : %-2d        AC : %-4d  WC  : %d",
                  pl->ob->stats.Con,pl->ob->stats.ac,pl->ob->stats.wc) ;
          new_draw_info(NDI_UNIQUE, 0,op,buf);
-         sprintf(buf,"Wis : %-2d       EXP : %d",
+#ifndef WIN32
+         sprintf(buf,"Wis : %-2d       EXP : %lld",
                  pl->ob->stats.Wis,pl->ob->stats.exp);
+#else
+         sprintf(buf,"Wis : %-2d       EXP : %I64d",
+                 pl->ob->stats.Wis,pl->ob->stats.exp);
+#endif
          new_draw_info(NDI_UNIQUE, 0,op,buf);
          sprintf(buf,"Cha : %-2d      Food : %d",
                  pl->ob->stats.Cha,pl->ob->stats.food) ;
@@ -810,9 +1073,8 @@ int command_abil (object *op, char *params)
     }
     for(pl=first_player;pl!=NULL;pl=pl->next) 
        if(!strcmp(pl->ob->name,thing)){ 
-#ifndef REAL_WIZ  
-	  SET_FLAG(pl->ob, FLAG_WAS_WIZ);
-#endif
+	  if (settings.real_wiz == FALSE)
+	       SET_FLAG(pl->ob, FLAG_WAS_WIZ);
           if(!strcmp("str",thing2))
             pl->ob->stats.Str = iii,pl->orig_stats.Str = iii;
           if(!strcmp("dex",thing2))   
@@ -877,11 +1139,10 @@ int command_reset (object *op, char *params)
 
     if (m->in_memory == MAP_SWAPPED) {	
 	LOG(llevDebug,"Resetting map %s.\n",m->path);
-	clean_tmp_map(m);
-	if (m->tmpname) free(m->tmpname);
-	m->tmpname = NULL;
+
 	/* setting this effectively causes an immediate reload */
 	m->reset_time = 1;
+	flush_old_maps();
 	new_draw_info(NDI_UNIQUE, 0,op,"OK.");
 	if (tmp) {
 	    enter_exit(tmp, dummy);
@@ -905,10 +1166,10 @@ int command_nowiz (object *op, char *params) /* 'noadm' is alias */
      CLEAR_FLAG(op, FLAG_WIZ);
      CLEAR_FLAG(op, FLAG_WIZPASS);
      CLEAR_FLAG(op, FLAG_FLYING);
-#ifdef REAL_WIZ
-     CLEAR_FLAG(op, FLAG_WAS_WIZ);
-#endif
-     new_draw_info(NDI_UNIQUE | NDI_ALL, 1, NULL,
+     if (settings.real_wiz == TRUE)
+	 CLEAR_FLAG(op, FLAG_WAS_WIZ);
+    op->contr->hidden=0;
+     new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_LT_GREEN, 1, NULL,
 	"The Dungeon Master is gone..");
      return 1;
   }
@@ -964,7 +1225,7 @@ int command_dm (object *op, char *params)
       SET_FLAG(op, FLAG_WAS_WIZ);
       SET_FLAG(op, FLAG_WIZPASS);
       new_draw_info(NDI_UNIQUE, 0,op, "Ok, you are the Dungeon Master!");
-      new_draw_info(NDI_UNIQUE | NDI_ALL, 1, NULL,
+      new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_LT_GREEN, 1, NULL,
   	"The Dungeon Master has arrived!");
       SET_FLAG(op, FLAG_FLYING);
       clear_los(op);
@@ -980,29 +1241,31 @@ int command_dm (object *op, char *params)
 
 int command_invisible (object *op, char *params)
 {
-  if (!op)
+    if (op) {
+	op->invisible+=100;
+	update_object(op,UP_OBJ_FACE);
+	new_draw_info(NDI_UNIQUE, 0,op,"You turn invisible.");
+    }
     return 0;
-      op->invisible+=100;
-      update_object(op,UP_OBJ_FACE);
-      new_draw_info(NDI_UNIQUE, 0,op,"You turn invisible.");
-  return 0;
 }
 
 
 static int command_learn_spell_or_prayer (object *op, char *params,
                                           int special_prayer)
 {
-    int spell;
+    object *tmp;
 
     if (op->contr == NULL || params == NULL)
         return 0;
 
-    if ((spell = look_up_spell_name (params)) <= 0) {
-        new_draw_info (NDI_UNIQUE, 0, op, "Unknown spell.");
-        return 1;
+    tmp = get_archetype(params);
+    if (!tmp) {
+	new_draw_info_format(NDI_UNIQUE, 0, op,
+		"Could not find a spell by name of %s\n", params);
+	return 0;
     }
-
-    do_learn_spell (op, spell, special_prayer);
+    do_learn_spell (op, tmp, special_prayer);
+    free_object(tmp);
     return 1;
 }
 
@@ -1018,17 +1281,10 @@ int command_learn_special_prayer (object *op, char *params)
 
 int command_forget_spell (object *op, char *params)
 {
-    int spell;
-
     if (op->contr == NULL || params == NULL)
         return 0;
 
-    if ((spell = look_up_spell_name (params)) <= 0) {
-        new_draw_info (NDI_UNIQUE, 0, op, "Unknown spell.");
-        return 1;
-    }
-
-    do_forget_spell (op, spell);
+    do_forget_spell (op, params);
     return 1;
 }
 /* GROS */
