@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_hiscore_c =
- *   "$Id: hiscore.c,v 1.7 2002/07/15 04:25:40 mwedel Exp $";
+ *   "$Id: hiscore.c,v 1.10 2004/03/26 21:59:28 ryo_saeba Exp $";
  */
 
 /*
@@ -39,7 +39,7 @@ typedef struct scr {
   char name[BIG_NAME];      /* name  */
   char title[BIG_NAME];	    /* Title */
   char killer[BIG_NAME];    /* name (+ title) or "quit" */
-  long exp;                 /* Experience */
+  sint64 exp;               /* Experience */
   char maplevel[BIG_NAME];  /* Killed on what level */
   int maxhp,maxsp,maxgrace; /* Max hp, sp, grace when killed */
   int position;             /* Position in the highscore list */
@@ -99,8 +99,13 @@ static void copy_score(score *sc1,score *sc2) {
 
 static char *put_score(score *sc) {
   static char buf[MAX_BUF];
-  sprintf(buf,"%s:%s:%ld:%s:%s:%d:%d:%d",sc->name,sc->title,sc->exp,sc->killer,sc->maplevel,
+#ifndef WIN32
+  sprintf(buf,"%s:%s:%lld:%s:%s:%d:%d:%d",sc->name,sc->title,sc->exp,sc->killer,sc->maplevel,
           sc->maxhp,sc->maxsp,sc->maxgrace);
+#else
+  sprintf(buf,"%s:%s:%I64d:%s:%s:%d:%d:%d",sc->name,sc->title,sc->exp,sc->killer,sc->maplevel,
+          sc->maxhp,sc->maxsp,sc->maxgrace);
+#endif
   return buf;
 }
 
@@ -128,7 +133,11 @@ static score *get_score(char *bp) {
 
   if ((cp = spool(NULL, "score")) == NULL)
     return NULL;
-  sscanf(cp,"%ld",&sc.exp);
+#ifndef WIN32
+  sscanf(cp,"%lld",&sc.exp);
+#else
+  sscanf(cp,"%I64d",&sc.exp);
+#endif
 
   if ((cp = spool(NULL, "killer")) == NULL)
     return NULL;
@@ -158,15 +167,15 @@ static char * draw_one_high_score(score *sc) {
     static char retbuf[MAX_BUF];
 
     if(!strncmp(sc->killer,"quit",MAX_NAME))
-	sprintf(retbuf,"%3d %10ld %s the %s quit the game on map %s [%d][%d][%d].",
+	sprintf(retbuf,"%3d %10lld %s the %s quit the game on map %s [%d][%d][%d].",
             sc->position,sc->exp,sc->name,sc->title,sc->maplevel,sc->maxhp,sc->maxsp,
 		sc->maxgrace);
     else if(!strncmp(sc->killer,"left",MAX_NAME))
-	sprintf(retbuf,"%3d %10ld %s the %s left the game on map %s [%d][%d][%d].",
+	sprintf(retbuf,"%3d %10lld %s the %s left the game on map %s [%d][%d][%d].",
             sc->position,sc->exp,sc->name,sc->title,sc->maplevel,sc->maxhp,sc->maxsp,
 		sc->maxgrace);
     else
-	sprintf(retbuf,"%3d %10ld %s the %s was killed by %s on map %s [%d][%d][%d].",
+	sprintf(retbuf,"%3d %10lld %s the %s was killed by %s on map %s [%d][%d][%d].",
             sc->position,sc->exp,sc->name,sc->title,sc->killer,sc->maplevel,
             sc->maxhp,sc->maxsp,sc->maxgrace);
     return retbuf;
@@ -256,13 +265,11 @@ void check_score(object *op) {
 	new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
 	return;
     }
-#ifdef EXPLORE_MODE
     if (op->contr->explore) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Since you were in explore mode,");
 	new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
 	return;
     }
-#endif
     strncpy(new_score.name,op->name,BIG_NAME);
     strncpy(new_score.title,op->contr->own_title,BIG_NAME);
     if(new_score.title[0]=='\0')
