@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.26 2002/09/11 06:21:46 mwedel Exp $";
+ *   "$Id: init.c,v 1.54 2004/02/04 07:46:12 tanner Exp $";
  */
 
 /*
@@ -31,7 +31,6 @@
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
-#include <version.h>
 
 /* global weathermap */
 weathermap_t **weathermap;
@@ -80,19 +79,6 @@ void set_csport(char *val)
 #endif /* win32 */
 }
 
-static void stat_loss_on_death_true() {settings.stat_loss_on_death = 1; }
-static void stat_loss_on_death_false() {settings.stat_loss_on_death = 0; }
-
-static void use_permanent_experience_true() {settings.use_permanent_experience = 1; }
-static void use_permanent_experience_false() {settings.use_permanent_experience = 0; }
-
-static void balanced_stat_loss_true() {settings.balanced_stat_loss = 1; }
-static void balanced_stat_loss_false() {settings.balanced_stat_loss = 0; }
-
-static void simple_exp_true() {settings.simple_exp = 1; }
-static void simple_exp_false() {settings.simple_exp = 0; }
-
-
 /* Most of this is shamelessly stolen from XSysStats.  But since that is
  * also my program, no problem.
  */
@@ -124,7 +110,6 @@ struct Command_Line_Options options[] = {
 {"-d", 0, 1, set_debug},
 {"+d", 0, 1, unset_debug},
 {"-mon", 0, 1, set_mondebug},
-#ifndef SECURE
 {"-data",1,1, set_datadir},
 {"-conf",1,1, set_confdir},
 {"-local",1,1, set_localdir},
@@ -134,7 +119,6 @@ struct Command_Line_Options options[] = {
 {"-treasures", 1, 1, set_treasures},
 {"-uniquedir", 1, 1, set_uniquedir},
 {"-tmpdir", 1, 1, set_tmpdir},
-#endif
 {"-log", 1, 1, set_logfile},
 
 /* Pass 2 functions.  Most of these could probably be in pass 1, 
@@ -147,7 +131,6 @@ struct Command_Line_Options options[] = {
  * and defaults should have been set up. 
  */
 {"-o", 0, 3, compile_info},
-#ifdef DUMP_SWITCHES
 {"-m", 0, 3, set_dumpmon1},
 {"-m2", 0, 3, set_dumpmon2},
 {"-m3", 0, 3, set_dumpmon3},
@@ -159,17 +142,8 @@ struct Command_Line_Options options[] = {
 {"-m9", 0, 3, set_dumpmon9},
 {"-mt", 1, 3, set_dumpmont},
 {"-mexp", 0, 3, dump_experience},
-#endif
 {"-s", 0, 3, showscores},
-{"-score", 1, 3, showscoresparm},
-{"-simple_exp", 0, 3, simple_exp_true},
-{"+simple_exp", 0, 3, simple_exp_false},
-{"-stat_loss_on_death", 0, 3, stat_loss_on_death_true},
-{"+stat_loss_on_death", 0, 3, stat_loss_on_death_false},
-{"-balanced_stat_loss", 0, 3, balanced_stat_loss_true},
-{"+balanced_stat_loss", 0, 3, balanced_stat_loss_false},
-{"-use_permanent_experience", 0, 3, use_permanent_experience_true},
-{"+use_permanent_experience", 0, 3, use_permanent_experience_false}
+{"-score", 1, 3, showscoresparm}
 };
 
 
@@ -218,6 +192,124 @@ static void parse_args(int argc, char *argv[], int pass)
     }
 }
 
+materialtype_t *materialt;
+
+static materialtype_t *get_empty_mat() {
+    materialtype_t *mt;
+    int i;
+
+    mt = (materialtype_t *)malloc(sizeof(materialtype_t));
+    if (mt == NULL)
+	fatal(OUT_OF_MEMORY);
+    mt->name = NULL;
+    mt->description = NULL;
+    for (i=0; i < NROFATTACKS; i++) {
+	mt->save[i] = 0;
+	mt->mod[i] = 0;
+    }
+    mt->chance = 0;
+    mt->difficulty = 0;
+    mt->magic = 0;
+    mt->damage = 0;
+    mt->wc = 0;
+    mt->ac = 0;
+    mt->sp = 0;
+    mt->weight = 100;
+    mt->value = 100;
+    mt->next = NULL;
+    return mt;
+}
+
+static void load_materials()
+{
+    char buf[MAX_BUF], filename[MAX_BUF], *cp, *next;
+    FILE *fp;
+    materialtype_t *mt;
+    int i, value;
+
+    sprintf(filename, "%s/materials", settings.datadir);
+    LOG(llevDebug, "Reading material type data from %s...", filename);
+    if ((fp = fopen(filename, "r")) == NULL) {
+        LOG(llevError, "Cannot open %s for reading\n", filename);
+        mt = get_empty_mat();
+        mt->next = NULL;
+        materialt = mt;
+        return;
+    }
+    mt = get_empty_mat();
+    materialt = mt;
+    while (fgets(buf, MAX_BUF, fp) != NULL) {
+	if (*buf=='#')
+	    continue;
+	if ((cp=strchr(buf, '\n'))!=NULL)
+	    *cp='\0';
+	cp=buf;
+	while(*cp==' ') /* Skip blanks */
+	    cp++;
+	if (!strncmp(cp, "name", 4)) {
+	    /* clean up the previous entry */
+	    if (mt->next != NULL) {
+		if (mt->description == NULL)
+		    mt->description = add_string(mt->name);
+		mt = mt->next;
+	    }
+	    mt->next = get_empty_mat();
+	    mt->name = add_string(strchr(cp, ' ') + 1);
+	} else if (!strncmp(cp, "description", 11)) {
+	    mt->description = add_string(strchr(cp, ' ') + 1);
+	} else if (sscanf(cp, "material %d", &value)) {
+	    mt->material = value;
+	} else if (!strncmp(cp, "saves", 5)) {
+	    cp = strchr(cp, ' ') + 1;
+	    for (i=0; i < NROFATTACKS; i++) {
+		if (cp == NULL) {
+		    mt->save[i] = 0;
+		    continue;
+		}
+		if ((next=strchr(cp,',')) != NULL)
+		    *(next++) = '\0';
+		sscanf(cp, "%d", &value);
+		mt->save[i] = (sint8)value;
+		cp = next;
+	    }
+	} else if (!strncmp(cp, "mods", 4)) {
+	    	cp = strchr(cp, ' ') + 1;
+		for (i=0; i < NROFATTACKS; i++) {
+		    if (cp == NULL) {
+			mt->save[i] = 0;
+			continue;
+		    }
+		    if ((next=strchr(cp,',')) != NULL)
+			*(next++) = '\0';
+		    sscanf(cp, "%d", &value);
+		    mt->mod[i] = (sint8)value;
+		    cp = next;
+		}
+	} else if (sscanf(cp, "chance %d\n", &value)) {
+	    mt->chance = (sint8)value;
+	} else if (sscanf(cp, "diff %d\n", &value)) {
+	    mt->difficulty = (sint8)value;
+	} else if (sscanf(cp, "magic %d\n", &value)) {
+	    mt->magic = (sint8)value;
+	} else if (sscanf(cp, "damage %d\n", &value)) {
+	    mt->damage= (sint8)value;
+	} else if (sscanf(cp, "wc %d\n", &value)) {
+	    mt->wc = (sint8)value;
+	} else if (sscanf(cp, "ac %d\n", &value)) {
+	    mt->ac = (sint8)value;
+	} else if (sscanf(cp, "sp %d\n", &value)) {
+	    mt->sp = (sint8)value;
+	} else if (sscanf(cp, "weight %d\n", &value)) {
+	    mt->weight = value;
+	} else if (sscanf(cp, "value %d\n", &value)) {
+	    mt->value = value;
+	}
+    }
+    mt->next = NULL;
+    LOG(llevDebug, "Done.\n");
+    fclose(fp);
+}
+
 /* This loads the settings file.  There could be debate whether this should
  * be here or in the common directory - but since only the server needs this
  * information, having it here probably makes more sense.
@@ -269,6 +361,16 @@ static void load_settings()
 	    if (has_val) strcpy(settings.meta_server, cp);
 	    else 
 		LOG(llevError,"load_settings: metaserver_server must have a value.\n");
+	} else if (!strcasecmp(buf,"motd")) {
+	    if (has_val)
+		strcpy(settings.motd, cp);
+	    else 
+		LOG(llevError,"load_settings: motd must have a value.\n");
+	} else if (!strcasecmp(buf,"dm_mail")) {
+	    if (has_val)
+		strcpy(settings.dm_mail, cp);
+	    else 
+		LOG(llevError,"load_settings: dm_mail must have a value.\n");
 	} else if (!strcasecmp(buf,"metaserver_host")) {
 	    if (has_val) strcpy(settings.meta_host, cp);
 	    else 
@@ -338,8 +440,141 @@ static void load_settings()
 		    "at least 0, %d is invalid\n", lev);
 	    else
 		settings.dynamiclevel = lev;
-	}
-	else if (!strcasecmp(buf,"simple_exp")) {
+	} else if (!strcasecmp(buf, "fastclock")) {
+	    int lev = atoi(cp);
+
+	    if (lev < 0)
+		LOG(llevError, "load_settings: fastclock must be at least 0"
+		    ", %d is invalid\n", lev);
+	    else
+		settings.fastclock = lev;
+	} else if (!strcasecmp(buf, "not_permadeth")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.not_permadeth=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.not_permadeth=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for not_permadeth"
+		    ": %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "resurrection")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.resurrection=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.resurrection=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for resurrection"
+		    ": %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "set_title")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.set_title=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.set_title=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for set_title"
+		    ": %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "search_items")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.search_items=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.search_items=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for search_items"
+		    ": %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "spell_encumbrance")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.spell_encumbrance=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.spell_encumbrance=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for "
+		    "spell_encumbrance: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "spell_failure_effects")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.spell_failure_effects=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.spell_failure_effects=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "spell_failure_effects: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "casting_time")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.casting_time=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.casting_time=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "casting_time: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "real_wiz")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.real_wiz=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.real_wiz=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "real_wiz: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "recycle_tmp_maps")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.recycle_tmp_maps=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.recycle_tmp_maps=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "recycle_tmp_maps: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "explore_mode")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.explore_mode=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.explore_mode=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "explore_mode: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "spellpoint_level_depend")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.spellpoint_level_depend=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.spellpoint_level_depend=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "spellpoint_level_depend: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "stat_loss_on_death")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.stat_loss_on_death=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.stat_loss_on_death=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "stat_loss_on_death: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "use_permanent_experience")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.use_permanent_experience=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.use_permanent_experience=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "use_permanent_experience: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf, "balanced_stat_loss")) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.balanced_stat_loss=TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.balanced_stat_loss=FALSE;
+	    } else {
+		LOG(llevError, "load_settings: Unkown value for"
+		    "balanced_stat_loss: %s\n", cp);
+	    }
+	} else if (!strcasecmp(buf,"simple_exp")) {
 	    if (!strcasecmp(cp,"on") || !strcasecmp(cp,"true")) {
 		settings.simple_exp=TRUE;
 	    } else if (!strcasecmp(cp,"off") || !strcasecmp(cp,"false")) {
@@ -348,6 +583,21 @@ static void load_settings()
 		LOG(llevError,"load_settings: Unkown value for simple_exp: %s\n",
 		    cp);
 	    }
+	} else if (!strcasecmp(buf, "item_power_factor")) {
+	    float tmp = atof(cp);
+	    if (tmp < 0)
+		LOG(llevError, "load_settings: item_power_factor must be a postive number (%f < 0)\n",
+		    tmp);
+	    else
+		settings.item_power_factor = tmp;
+	} else if (!strcasecmp(buf, "set_friendly_fire")) {
+	    int val = atoi(cp);
+
+	    if (val < 1 || val >100)
+		LOG(llevError, "load_settings: set_friendly_fire must be between 1 an 100"
+		    ", %d is invalid\n", val);
+	    else
+		settings.set_friendly_fire = val;	
 	} else {
 	    LOG(llevError,"Unknown value in settings file: %s\n", buf);
 	}
@@ -362,8 +612,6 @@ static void load_settings()
 
 void init(int argc, char **argv) {
 
-    (void) umask(0);	/* We don't want to be affected by players' umask */
-
     init_done=0;		/* Must be done before init_signal() */
     logfile=stderr;
     parse_args(argc, argv, 1);	/* First arg pass - right now it does
@@ -373,15 +621,16 @@ void init(int argc, char **argv) {
     init_library();	/* Must be called early */
     load_settings();	/* Load the settings file */
     init_weather();
+    load_materials();
     parse_args(argc, argv, 2);
     fprintf(logfile,"Welcome to CrossFire, v%s\n",VERSION);
     fprintf(logfile,"Copyright (C) 1994 Mark Wedel.\n");
     fprintf(logfile,"Copyright (C) 1992 Frank Tore Johansen.\n");
 
-#ifdef DM_MAIL
-    fprintf(logfile,"Maintained locally by: %s\n",DM_MAIL);
-    fprintf(logfile,"Questions and bugs should be mailed to above address.\n");
-#endif
+    if (strcmp(settings.dm_mail, "") != 0) {
+	fprintf(logfile,"Maintained locally by: %s\n", settings.dm_mail);
+	fprintf(logfile,"Questions and bugs should be mailed to above address.\n");
+    }
     SRANDOM(time(NULL));
 
     init_startup();	/* Write (C), check shutdown/forbid files */
@@ -389,6 +638,7 @@ void init(int argc, char **argv) {
     setup_library();	/* Set up callback function pointers */
     init_commands();	/* Sort command tables */
     read_map_log();	/* Load up the old temp map files */
+    init_skills();
 
     parse_args(argc, argv, 3);
 
@@ -426,16 +676,7 @@ void help() {
     printf(" -o          Prints out info on what was defined at compile time.\n");
     printf(" -s          Display the high-score list.\n");
     printf(" -score <name or class> Displays all high scores with matching name/class.\n");
-    printf(" -stat_loss_on_death - if set, player loses stat when they die\n");
-    printf(" +stat_loss_on_death - if set, player does not lose a stat when they die\n");
-    printf(" -use_permanent_experience - if set, player may gain permanent experience\n");
-    printf(" +use_permanent_experience - if set, player does not gain permanent experience\n");
-    printf(" -balanced_stat_loss - if set, death stat depletion is balanced by level etc\n");
-    printf(" +balanced_stat_loss - if set, ordinary death stat depletion is used\n");
     printf(" -v          Print version and contributors.\n");
-
-#ifndef SECURE
-    printf("\nThe following options are only available if a secure server was not compiled.\n");
     printf(" -data       Sets the lib dir (archetypes, treasures, etc.)\n");
     printf(" -local      Read/write local data (hiscore, unique items, etc.)\n");
     printf(" -maps       Sets the directory for maps.\n");
@@ -444,13 +685,9 @@ void help() {
     printf(" -treasures	 Sets the treasures file to use.\n");
     printf(" -uniquedir  Sets the unique items/maps directory.\n");
     printf(" -tmpdir     Sets the directory for temporary files (mostly maps.)\n");
-#endif
-
-#ifdef DUMP_SWITCHES
-    printf("\nThe following are only available in DUMP_SWITCHES was compiled in.\n");
     printf(" -m          Lists out suggested experience for all monsters.\n");
     printf(" -m2         Dumps out abilities.\n");
-    printf(" -m3         Dumps out artificat information.\n");
+    printf(" -m3         Dumps out artifactt information.\n");
     printf(" -m4         Dumps out spell information.\n");
     printf(" -m5         Dumps out skill information.\n");
     printf(" -m6         Dumps out race information.\n");
@@ -458,7 +695,6 @@ void help() {
     printf(" -m8         Dumps out gods information.\n");
     printf(" -m9         Dumps out more alchemy information (formula checking).\n");
     printf(" -mt <name>  Dumps out list of treasures for a monster.\n");
-#endif
     exit(0);
 }
 
@@ -470,12 +706,8 @@ void init_beforeplay() {
   init_races();	   /* overwrite race designations using entries in lib/races file */ 
   init_gods();	/* init linked list of gods from archs*/ 
   init_readable();	/* inits useful arrays for readable texts */
-#ifdef ALCHEMY
   init_formulae();  /* If not called before, reads formulae from file */
-#endif
-  init_new_exp_system();    /* If not called before, inits experience system */
 
-#ifdef DUMP_SWITCHES
   switch(settings.dumpvalues) {
   case 1:
     print_monsters();
@@ -490,7 +722,6 @@ void init_beforeplay() {
     dump_spells();
     exit(0);
   case 5:
-    dump_skills();
     exit(0);
   case 6:
     dump_races();
@@ -508,7 +739,6 @@ void init_beforeplay() {
     dump_monster_treasure(settings.dumparg);
     exit(0);
   }
-#endif
 }
 
 void init_startup() {
@@ -563,11 +793,6 @@ void compile_info() {
 #endif
   if(!i)
     printf("(none)\n");
-#ifdef SECURE
-  printf("Secure:\t\t<true>\n");
-#else
-  printf("Secure:\t\t<false>\n");
-#endif
   printf("Datadir:\t\t%s\n",settings.datadir);
   printf("Localdir:\t\t%s\n",settings.localdir);
 #ifdef PERM_FILE
@@ -580,18 +805,8 @@ void compile_info() {
   printf("Save mode:\t%4.4o\n",SAVE_MODE);
   printf("Playerdir:\t<VAR>/%s\n",settings.playerdir);
   printf("Itemsdir:\t<VAR>/%s\n", settings.uniquedir);
-#ifdef USE_CHECKSUM
-  printf("Use checksum:\t<true>\n");
-#else
-  printf("Use checksum:\t<false>\n");
-#endif
   printf("Tmpdir:\t\t%s\n",settings.tmpdir);
   printf("Map max timeout:\t%d\n",MAP_MAXTIMEOUT);
-#ifdef MAP_RESET
-  printf("Map reset:\t<true>\n");
-#else
-  printf("Map reset:\t<false>\n");
-#endif
   printf("Max objects:\t%d\n",MAX_OBJECTS);
 #ifdef USE_CALLOC
   printf("Use_calloc:\t<true>\n");
@@ -599,29 +814,10 @@ void compile_info() {
   printf("Use_calloc:\t<false>\n");
 #endif
 
-#ifdef USE_SWAP_STATS
-  printf("Use_swap_stats:\t<true>\n");
-#else
-  printf("Use_swap_stats:\t<false>\n");
-#endif
-
-#ifdef DM_MAIL
-  printf("DM mail:\t%s\n",DM_MAIL);
-#endif
-
 #ifdef X_EDITOR
   printf("Editor:\t\t%s\n",X_EDITOR);
 #endif
-#ifdef EXPLORE_MODE
-  printf("Explore mode:\t<true>\n");
-#else
-  printf("Explore mode:\t<false>\n");
-#endif
-#ifdef SHOP_LISTINGS
-  printf("Shop listings:\t<true>\n");
-#else
-  printf("Shop listings:\t<false>\n");
-#endif
+
   printf("Max_time:\t%d\n",MAX_TIME);
 
 #ifdef WIN32 /* ***win32 compile_info(): remove execl... */
@@ -705,6 +901,7 @@ void init_signals() {
 #ifndef WIN32 /* init_signals() remove signals */
   signal(SIGHUP,rec_sighup);
   signal(SIGINT,rec_sigint);
+#ifndef DEBUG
   signal(SIGQUIT,rec_sigquit);
   signal(SIGSEGV,rec_sigsegv);
   signal(SIGPIPE,rec_sigpipe);
@@ -712,6 +909,7 @@ void init_signals() {
   signal(SIGBUS,rec_sigbus);
 #endif
   signal(SIGTERM,rec_sigterm);
+#endif
 #endif /* win32 */
 }
 
@@ -728,19 +926,21 @@ void setup_library() {
   set_update_buttons(update_buttons);
   set_draw_info(new_draw_info);
   set_move_apply(move_apply);
-  set_draw(draw);
   set_monster_check_apply(monster_check_apply);
   set_move_teleporter(move_teleporter);
   set_move_firewall(move_firewall);
   set_move_creator(move_creator);
+  set_move_trigger_marker(move_marker);
+  set_move_duplicator(move_duplicator);
   set_trap_adjust(trap_adjust);
   set_esrv_send_item(esrv_send_item);
   set_esrv_del_item(esrv_del_item);
   set_esrv_update_item(esrv_update_item);
-/*  set_init_blocksview_players(init_blocksview_players); */
   set_info_map(new_info_map);
   set_set_darkness_map(set_darkness_map);
   set_dragon_gain_func(dragon_ability_gain);
+  set_weather_effect_func(weather_effect);
+  set_find_skill_by_number_func(find_skill_by_number);
 }
 
 /* init_races() - reads the races file in the lib/ directory, then
@@ -866,4 +1066,3 @@ racelink * find_racelink( char *name ) {
  
   return test;
 }
-
