@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_login_c =
- *   "$Id: login.c,v 1.33 2002/07/15 04:57:13 mwedel Exp $";
+ *   "$Id: login.c,v 1.46 2004/03/12 19:04:15 ryo_saeba Exp $";
  */
 
 /*
@@ -34,7 +34,6 @@
 #include <loader.h>
 #include <define.h>
 
-extern spell spells[NROFREALSPELLS];
 extern void sub_weight (object *, signed long);
 extern void add_weight (object *, signed long);
 extern long pticks;
@@ -164,6 +163,11 @@ int check_name(player *me,char *name) {
 	new_draw_info(NDI_UNIQUE, 0,me->ob,"That name contains illegal characters.");
 	return 0;
     }
+    if (strlen(name) >= MAX_NAME) {
+	new_draw_info(NDI_UNIQUE, 0,me->ob,"That name is too long.");
+	return 0;
+    }
+
     return 1;
 }
 
@@ -177,11 +181,7 @@ int create_savedir_if_needed(char *savedir)
   } else {
     stat(savedir, buf);
     if ((buf->st_mode & S_IFDIR) == 0)
-#if defined(_IBMR2) || defined(___IBMR2)
-      if (mkdir(savedir, S_ISUID|S_ISGID|S_IRUSR|S_IWUSR|S_IXUSR))
-#else
-      if (mkdir(savedir, S_ISUID|S_ISGID|S_IREAD|S_IWRITE|S_IEXEC))
-#endif
+      if (mkdir(savedir, SAVE_DIR_MODE))
 	{
 	perror("Unable to create player savedir,");
 	perror(savedir);
@@ -267,21 +267,20 @@ int save_player(object *op, int flag) {
    }
 
   fprintf(fp,"password %s\n",pl->password);
-#ifdef SET_TITLE
-  if(pl->own_title[0]!='\0')
-    fprintf(fp,"title %s\n",pl->own_title);
-#endif /* SET_TITLE */
+  if (settings.set_title == TRUE)
+      if(pl->own_title[0]!='\0')
+	  fprintf(fp,"title %s\n",pl->own_title);
 
-#ifdef EXPLORE_MODE
   fprintf(fp,"explore %d\n",pl->explore);
-#endif
   fprintf(fp,"gen_hp %d\n",pl->gen_hp);
   fprintf(fp,"gen_sp %d\n",pl->gen_sp);
   fprintf(fp,"gen_grace %d\n",pl->gen_grace);
   fprintf(fp,"listening %d\n",pl->listening);
-  fprintf(fp,"spell %d\n",pl->chosen_spell);
   fprintf(fp,"shoottype %d\n",pl->shoottype);
+  fprintf(fp,"bowtype %d\n",pl->bowtype);
+  fprintf(fp,"petmode %d\n",pl->petmode);
   fprintf(fp,"peaceful %d\n",pl->peaceful);
+  fprintf(fp,"no_shout %d\n",pl->no_shout);
   fprintf(fp,"digestion %d\n",pl->digestion);
   fprintf(fp,"pickup %d\n", pl->mode);
   fprintf(fp,"outputs_sync %d\n", pl->outputs_sync);
@@ -321,8 +320,6 @@ int save_player(object *op, int flag) {
     fprintf(fp,"%d\n",pl->levsp[i]);
 	 fprintf(fp,"%d\n",pl->levgrace[i]);
   }
-  for(i=0;i<pl->nrofknownspells;i++)
-    fprintf(fp,"known_spell %s\n",spells[pl->known_spells[i]].name);
   fprintf(fp,"endplst\n");
 
   SET_FLAG(op, FLAG_NO_FIX_PLAYER);
@@ -358,7 +355,7 @@ int save_player(object *op, int flag) {
 	free(tmpfilename);
 	return 0;
   }
-  checksum = calculate_checksum(tmpfilename, 0);
+  checksum = 0;
   sprintf(backupfile, "%s.tmp", filename);
   rename(filename, backupfile);
   fp = fopen(filename,"w");
@@ -392,36 +389,6 @@ int save_player(object *op, int flag) {
   return 1;
 }
 
-/*
- * calculate_checksum:
- * Evil scheme to avoid tampering with the player-files 8)
- * The cheat-flag will be set if the file has been changed.
- */
-
-long calculate_checksum(char *filename, int checkdouble) {
-#ifdef USE_CHECKSUM
-  long checksum = 0;
-  int offset = 0;
-  FILE *fp;
-  char buf[MAX_BUF], *cp;
-  if ((fp = fopen(filename,"r")) == NULL)
-    return 0;
-  while(fgets(buf,MAX_BUF,fp)) {
-    if(checkdouble && !strncmp(buf,"checksum",8))
-      continue;
-    for(cp=buf;*cp;cp++) {
-      if(++offset>28)
-        offset = 0;
-      checksum^=(*cp<<offset);
-    }
-  }
-  fclose(fp);
-  return checksum;
-#else
-  return 0;
-#endif
-}
-
 void copy_file(char *filename, FILE *fpout) {
   FILE *fp;
   char buf[MAX_BUF];
@@ -432,19 +399,6 @@ void copy_file(char *filename, FILE *fpout) {
   fclose(fp);
 }
 
-#if 1
-static int spell_sort(const void *a1,const void *a2)
-{
-  return strcmp(spells[(int)*(sint16 *)a1].name,spells[(int)*(sint16 *)a2].name);
-}
-#else
-static int spell_sort(const char *a1,const char *a2)
-{
-   fprintf(stderr, "spell1=%d, spell2=%d\n", *(sint16*)a1, *(sint16*)a2);
-  return strcmp(spells[(int )*a1].name,spells[(int )*a2].name);
-}
-#endif
-
 
 void check_login(object *op) {
     FILE *fp;
@@ -456,10 +410,9 @@ void check_login(object *op) {
     int correct = 0;
     time_t    elapsed_save_time=0;
     struct stat	statbuf;
-#ifdef PLUGINS
     CFParm CFP;
     int evtid;
-#endif
+
     strcpy (pl->maplevel,first_map_path);
 
     /* First, lets check for newest form of save */
@@ -522,10 +475,8 @@ void check_login(object *op) {
     pl->last_save_time=time(NULL);
 #endif /* SAVE_INTERVAL */
     pl->party_number = (-1);
-
-#ifdef SEARCH_ITEMS
-    pl->search_str[0]='\0';
-#endif /* SEARCH_ITEMS */
+    if (settings.search_items == TRUE)
+	pl->search_str[0]='\0';
     pl->name_changed=1;
     pl->orig_stats.Str=0;
     pl->orig_stats.Dex=0;
@@ -542,29 +493,28 @@ void check_login(object *op) {
 	sscanf(bufall,"%s %d\n",buf,&value);
         if (!strcmp(buf,"endplst"))
           break;
-#ifdef SET_TITLE
-	else if (!strcmp(buf,"title"))
+	else if (!strcmp(buf,"title") && settings.set_title == TRUE)
 	    sscanf(bufall,"title %[^\n]",pl->own_title);
-#endif /* SET_TITLE */
-
-#ifdef EXPLORE_MODE
 	else if (!strcmp(buf,"explore"))
 	    pl->explore = value;
-#endif
 	else if (!strcmp(buf,"gen_hp"))
 	    pl->gen_hp=value;
         else if (!strcmp(buf,"shoottype"))
 	    pl->shoottype=(rangetype)value;
+	else if (!strcmp(buf,"bowtype"))
+	    pl->bowtype=(bowtype_t)value;
+	else if (!strcmp(buf,"petmode"))
+	    pl->petmode=(petmode_t)value;
         else if (!strcmp(buf,"gen_sp"))
 	    pl->gen_sp=value;
         else if (!strcmp(buf,"gen_grace"))
 	    pl->gen_grace=value;
-        else if (!strcmp(buf,"spell"))
-	    pl->chosen_spell=value;
         else if (!strcmp(buf,"listening"))
 	    pl->listening=value;
         else if (!strcmp(buf,"peaceful"))
 	    pl->peaceful=value;
+		else if (!strcmp(buf,"no_shout"))
+		pl->no_shout=value;
         else if (!strcmp(buf,"digestion"))
 	    pl->digestion=value;
 	else if (!strcmp(buf,"pickup"))
@@ -630,6 +580,10 @@ void check_login(object *op) {
 	 * objects.
 	 */
 	} else if (!strcmp(buf,"known_spell")) {
+#if 0
+	    /* Logic is left here in case someone wants to try
+	     * and write code to update to spell objects.
+	     */
 	    char *cp=strchr(bufall,'\n');
 	    *cp='\0';
 	    cp=strchr(bufall,' ');
@@ -641,10 +595,13 @@ void check_login(object *op) {
 		}
 	    if(i==NROFREALSPELLS)
 		LOG(llevDebug, "Error: unknown spell (%s)\n",cp);
+#endif
 	}
 	/* Remove confkeys, pushkey support - very old */
     } /* End of loop loading the character file */
     leave_map(op);
+    op->speed=0;
+    update_ob_speed(op);
     reset_object(op);
     op->contr = pl;
     pl->ob = op;
@@ -694,9 +651,7 @@ void check_login(object *op) {
      * Moved ahead of the esrv functions, so proper weights will be
      * sent to the client.
      */
-
-    (void) init_player_exp(op);
-    (void) link_player_skills(op);
+    link_player_skills(op);
 
     if ( ! legal_range (op, op->contr->shoottype))
         op->contr->shoottype = range_none;
@@ -705,7 +660,7 @@ void check_login(object *op) {
     
     /* if it's a dragon player, set the correct title here */
     if (is_dragon_pl(op) && op->inv != NULL) {
-        object *tmp, *abil, *skin;
+        object *tmp, *abil=NULL, *skin=NULL;
         for (tmp=op->inv; tmp!=NULL; tmp=tmp->below) {
 	    if (tmp->type == FORCE) {
 	        if (strcmp(tmp->arch->name, "dragon_ability_force")==0)
@@ -718,26 +673,15 @@ void check_login(object *op) {
     }
     
     new_draw_info(NDI_UNIQUE, 0,op,"Welcome Back!");
-    new_draw_info_format(NDI_UNIQUE | NDI_ALL, 5, NULL,
+    new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, NULL,
 	     "%s has entered the game.",pl->ob->name);
-#ifdef PLUGINS
     /* GROS : Here we handle the LOGIN global event */
     evtid = EVENT_LOGIN;
     CFP.Value[0] = (void *)(&evtid);
     CFP.Value[1] = (void *)(pl);
     CFP.Value[2] = (void *)(pl->socket.host);
     GlobalEvent(&CFP);
-#endif
     op->contr->socket.update_look=1;
-    LOG(llevDebug,"Checksums: %x %x\n",
-                checksum,calculate_checksum(filename,1));
-#ifdef ENABLE_CHECKSUM
-    if(calculate_checksum(filename,1) != checksum) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Since your savefile has been tampered with,");
-	new_draw_info(NDI_UNIQUE, 0,op,"you will not be able to save again.");
-	set_cheat(op);
-    }
-#endif
     /* If the player should be dead, call kill_player for them
      * Only check for hp - if player lacks food, let the normal
      * logic for that to take place.  If player is permanently
@@ -758,12 +702,6 @@ void check_login(object *op) {
     esrv_new_player(op->contr,op->weight+op->carrying);
     esrv_send_inventory(op, op);
 
-    /* This seems to compile without warnings now.  Don't know if it works
-     * on SGI's or not, however.
-     */
-    qsort((void *)pl->known_spells,pl->nrofknownspells,
-	sizeof(pl->known_spells[0]),(int (*)())spell_sort);
-
     CLEAR_FLAG(op, FLAG_FRIENDLY);
 
     /* can_use_shield is a new flag.  However, the can_use.. seems to largely come
