@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_main_c =
- *    "$Id: main.c,v 1.65 2002/09/07 22:08:07 mwedel Exp $";
+ *    "$Id: main.c,v 1.78 2004/04/03 08:05:25 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2001-2003 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@
     The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
-#include <version.h>
 #include <global.h>
 #include <object.h>
 #include <tod.h>
@@ -141,7 +140,7 @@ void start_info(object *op) {
   new_draw_info(NDI_UNIQUE, 0,op,buf);
   new_draw_info(NDI_UNIQUE, 0,op,"Press `?' for help");
   new_draw_info(NDI_UNIQUE, 0,op," ");
-  new_draw_info_format(NDI_UNIQUE | NDI_ALL, 5, op,
+  new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, op,
 	"%s entered the game.",op->name);
   if(!op->contr->name_changed) {
     new_draw_info(NDI_UNIQUE, 0,op,"Note that you must set your name with the name");
@@ -274,7 +273,8 @@ void leave_map(object *op)
     remove_ob(op);
 
     if (oldmap) {
-	oldmap->players--;
+	if (!op->contr->hidden)
+	    oldmap->players--;
 	if (oldmap->players <= 0) { /* can be less than zero due to errors in tracking this */
 	    set_map_timeout(oldmap);
 	}
@@ -290,16 +290,20 @@ void leave_map(object *op)
  */
 static void enter_map(object *op, mapstruct *newmap, int x, int y) {
     mapstruct *oldmap = op->map;
-#ifdef PLUGINS
     int evtid;
     CFParm CFP;
-#endif
 
     if (out_of_map(newmap, x, y)) {
 	LOG(llevError,"enter_map: supplied coordinates are not within the map! (%s: %d, %d)\n",
 	    newmap->path, x, y);
 	x=MAP_ENTER_X(newmap);
 	y=MAP_ENTER_Y(newmap);
+	if (out_of_map(newmap, x, y)) {
+	    LOG(llevError,"enter_map: map %s provides invalid default enter location (%d, %d) > (%d, %d)\n",
+		newmap->path, x, y, MAP_WIDTH(newmap), MAP_HEIGHT(newmap));
+	    new_draw_info(NDI_UNIQUE, 0, op, "The exit is closed");
+	    return;
+	}
     }
     /* try to find a spot for the player */
     if (arch_blocked(op->arch, newmap, x, y)) {	/* First choice blocked */
@@ -332,7 +336,6 @@ static void enter_map(object *op, mapstruct *newmap, int x, int y) {
      */
     if(!QUERY_FLAG(op, FLAG_REMOVED))
 	remove_ob(op);
-#ifdef PLUGINS
     if (op->map!=NULL)
     {
     	/* GROS : Here we handle the MAPLEAVE global event */
@@ -341,23 +344,21 @@ static void enter_map(object *op, mapstruct *newmap, int x, int y) {
     	CFP.Value[1] = (void *)(op);
     	GlobalEvent(&CFP);
     };
-#endif
     /* remove_ob clears these so they must be reset after the remove_ob call */
     op->x = x;
     op->y = y;
     op->map = newmap;
     insert_ob_in_map(op,op->map,NULL,INS_NO_WALK_ON);
 
-#ifdef PLUGINS
     /* GROS : Here we handle the MAPENTER global event */
     evtid = EVENT_MAPENTER;
     CFP.Value[0] = (void *)(&evtid);
     CFP.Value[1] = (void *)(op);
     GlobalEvent(&CFP);
-#endif
 
+    if (!op->contr->hidden)
+	newmap->players++;
 
-    newmap->players++;
     newmap->timeout=0;
     op->enemy = NULL;
 
@@ -367,24 +368,28 @@ static void enter_map(object *op, mapstruct *newmap, int x, int y) {
     }
 
     /* Update any golems */
-    if(op->type == PLAYER && op->contr->golem != NULL) {
-	int i = find_free_spot(op->contr->golem->arch,newmap, x, y, 1, SIZEOFFREE+1);
+    if(op->type == PLAYER && op->contr->ranges[range_golem] != NULL) {
+	int i = find_free_spot(op->contr->ranges[range_golem]->arch,newmap,
+			       x, y, 1, SIZEOFFREE+1);
 
-	remove_ob(op->contr->golem);
+	remove_ob(op->contr->ranges[range_golem]);
 	if (i==-1) {
-	    remove_friendly_object(op->contr->golem);
-	    free_object(op->contr->golem);
-	    op->contr->golem=NULL;
+	    remove_friendly_object(op->contr->ranges[range_golem]);
+	    free_object(op->contr->ranges[range_golem]);
+	    op->contr->ranges[range_golem]=NULL;
+	    op->contr->golem_count=0;
 	}
 	else {
 	    object *tmp;
-	    for (tmp=op->contr->golem; tmp!=NULL; tmp=tmp->more) {
+	    for (tmp=op->contr->ranges[range_golem]; tmp!=NULL; tmp=tmp->more) {
 		tmp->x = x + freearr_x[i]+ (tmp->arch==NULL?0:tmp->arch->clone.x);
 		tmp->y = y + freearr_y[i]+ (tmp->arch==NULL?0:tmp->arch->clone.y);
 		tmp->map = newmap;
 	    }
-	    insert_ob_in_map(op->contr->golem, newmap, NULL,0);
-	    op->contr->golem->direction = find_dir_2(op->x - op->contr->golem->x, op->y - op->contr->golem->y);
+	    insert_ob_in_map(op->contr->ranges[range_golem], newmap, NULL,0);
+	    op->contr->ranges[range_golem]->direction = 
+		find_dir_2(op->x - op->contr->ranges[range_golem]->x, 
+			   op->y - op->contr->ranges[range_golem]->y);
 	}
     }
     op->direction=0;
@@ -483,7 +488,7 @@ char *unclean_path(char *src)
 static void enter_random_map(object *pl, object *exit_ob)
 {
     mapstruct *new_map;
-    char newmap_name[HUGE_BUF];
+    char newmap_name[HUGE_BUF], *cp;
     static int reference_number = 0;
     RMParms rp;
 
@@ -496,10 +501,27 @@ static void enter_random_map(object *pl, object *exit_ob)
     rp.generate_treasure_now = 1;
     strcpy(rp.origin_map, pl->map->path);
 
-    /* pick a new pathname for the new map.  Currently, we just
-     * use a static variable and increment the counter one each time.
+    /* If we have a final_map, use it as a base name to give some clue
+     * as where the player is.  Otherwise, use the origin map.
+     * Take the last component (after the last slash) to give
+     * shorter names without bogus slashes.
      */
-    sprintf(newmap_name,"/random/%016d",reference_number++);
+    if (rp.final_map[0]) {
+	cp = strrchr(rp.final_map, '/');
+	if (!cp) cp = rp.final_map;
+    } else {
+	char buf[HUGE_BUF];
+
+	cp = strrchr(rp.origin_map, '/');
+	if (!cp) cp = rp.origin_map;
+	/* Need to strip of any trailing digits, if it has them */
+	strcpy(buf, cp);
+	while (isdigit(buf[strlen(buf) - 1]))
+	    buf[strlen(buf) - 1] = 0;
+	cp = buf;
+    }
+
+    sprintf(newmap_name,"/random/%s%04d",cp+1, reference_number++);
 
     /* now to generate the actual map. */
     new_map=(mapstruct *)generate_random_map(newmap_name,&rp);
@@ -632,7 +654,17 @@ void enter_exit(object *op, object *exit_ob) {
 		 * map.
 		 */
 		if (!newmap && !strncmp(EXIT_PATH(exit_ob),"/random/",8)) {
-		    enter_random_map(op, exit_ob);
+		    /* Maps that go down have a message set.  However, maps that go
+		     * up, don't.  If the going home has reset, there isn't much
+		     * point generating a random map, because it won't match the maps.
+		     */
+		    if (exit_ob->msg) {
+			enter_random_map(op, exit_ob);
+		    } else {
+			new_draw_info_format(NDI_UNIQUE, 0, op, "The %s is closed.", exit_ob->name);
+			return;
+		    }
+
 		    /* For exits that cause damages (like pits).  Don't know if any
 		     * random maps use this or not.
 		     */
@@ -748,7 +780,7 @@ void process_active_maps() {
     if(enough_elapsed_time()) {
 	for(map=first_map;map!=NULL;map=map->next) {
 	    if(map->in_memory == MAP_IN_MEMORY) {
-		if(players_on_map(map))
+		if(players_on_map(map,TRUE))
 		    process_events(map);
 	    }
 	}
@@ -786,7 +818,7 @@ void process_players1(mapstruct *map)
 		 * tick time so it will be about 10 seconds before we try and save
 		 * again.
 		 */
-		if (blocks_cleric(pl->ob->map, pl->ob->x, pl->ob->y)) {
+		if (get_map_flags(pl->ob->map, NULL, pl->ob->x, pl->ob->y, NULL, NULL) & P_NO_CLERIC) {
 		    pl->last_save_tick += 100;
 		} else {
 		    save_player(pl->ob,1);
@@ -805,18 +837,18 @@ void process_players1(mapstruct *map)
 	} /* end of for loop for all the players */
     } /* for flag */
     for(pl=first_player;pl!=NULL;pl=pl->next) {
-	if (map!=NULL && (pl->ob == NULL || pl->ob->map!=map)) continue;
-#ifdef CASTING_TIME
-	if (pl->ob->casting > 0){
-	    pl->ob->casting--;
-	    pl->ob->start_holding = 1;
+	if (map!=NULL && (pl->ob == NULL || pl->ob->map!=map))
+	    continue;
+	if (settings.casting_time == TRUE) {
+	    if (pl->ob->casting_time > 0){
+		pl->ob->casting_time--;
+		pl->ob->start_holding = 1;
+	    }
+	    /* set spell_state so we can update the range in stats field */
+	    if ((pl->ob->casting_time == 0) && (pl->ob->start_holding ==1)){
+		pl->ob->start_holding = 0;
+	    }
 	}
-	/* set spell_state so we can update the range in stats field */
-	if ((pl->ob->casting == 0) && (pl->ob->start_holding ==1)){
-	    pl->ob->start_holding = 0;
-	    pl->ob->spell_state = 1;
-	}
-#endif
 	do_some_living(pl->ob);
 /*	draw(pl->ob);*/	/* updated in socket code */
     }
@@ -921,7 +953,7 @@ void process_events (mapstruct *map)
 	 * is based on ticks, and not the creatures speed?
 	 */
 	if (op->anim_speed && op->last_anim >= op->anim_speed) {
-	    animate_object (op);
+	    animate_object (op, op->direction);
 	    op->last_anim = 1;
 	} else {
 	    op->last_anim++;
@@ -933,11 +965,8 @@ void process_events (mapstruct *map)
 	    if (was_destroyed (op, tag))
 		continue;
 	}
-
-#ifdef CASTING_TIME
-	if (op->casting > 0)
-	    op->casting--;
-#endif
+	if (settings.casting_time == TRUE && op->casting_time > 0)
+	    op->casting_time--;
 	if (op->speed_left <= 0)
 	    op->speed_left += FABS (op->speed);
     }
@@ -961,20 +990,23 @@ void clean_tmp_files() {
    * just make a special function that only saves the unique items.
    */
   for(m=first_map;m!=NULL;m=next) {
-    next=m->next;
-    if (m->in_memory == MAP_IN_MEMORY)
-/* If we want to reuse the temp maps, swap it out (note that will also
- * update the log file.  Otherwise, save the map (mostly for unique item
- * stuff).  Note that the clean_tmp_map is called after the end of
- * the for loop but is in the #else bracket.  IF we are recycling the maps,
- * we certainly don't want the temp maps removed.
- */
-#ifdef RECYCLE_TMP_MAPS
-	swap_map(m);
-#else
-	new_save_map(m, 0); /* note we save here into a overlay map */
-    clean_tmp_map(m);
-#endif
+      next=m->next;
+      if (m->in_memory == MAP_IN_MEMORY) {
+	/* If we want to reuse the temp maps, swap it out (note that will also
+	 * update the log file.  Otherwise, save the map (mostly for unique item
+         * stuff).  Note that the clean_tmp_map is called after the end of
+	 * the for loop but is in the #else bracket.  IF we are recycling the maps,
+         * we certainly don't want the temp maps removed.
+	 */
+
+	  /* XXX The above comment is dead wrong */
+	  if (settings.recycle_tmp_maps == TRUE)
+	      swap_map(m);
+	  else {
+	      new_save_map(m, 0); /* note we save here into a overlay map */
+	      clean_tmp_map(m);
+	  }
+      }
   }
   write_todclock(); /* lets just write the clock here */
 }
@@ -1005,16 +1037,14 @@ void cleanup()
 
 void leave(player *pl, int draw_exit) {
     char buf[MAX_BUF];
-#ifdef PLUGINS
     int evtid;
     CFParm CFP;
-#endif
+
     if (pl!=NULL) {
 	/* We do this so that the socket handling routine can do the final
 	 * cleanup.  We also leave that loop to actually handle the freeing
 	 * of the data.
 	 */
-#ifdef PLUGINS
         if (draw_exit==0)
         {
             /* GROS : Here we handle the LOGOUT global event */
@@ -1024,7 +1054,6 @@ void leave(player *pl, int draw_exit) {
             CFP.Value[2] = (void *)(pl->socket.host);
             GlobalEvent(&CFP);
         };
-#endif
 	pl->socket.status=Ns_Dead;
 	LOG(llevInfo,"LOGOUT: Player named %s from ip %s\n", pl->ob->name,
 	    pl->socket.host);
@@ -1038,7 +1067,7 @@ void leave(player *pl, int draw_exit) {
 	}
 	pl->ob->type = DEAD_OBJECT; /* To avoid problems with inventory window */
     }
-  if (pl!=NULL && draw_exit) new_draw_info(NDI_UNIQUE | NDI_ALL, 5, NULL, buf);
+  if (pl!=NULL && draw_exit) new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, NULL, buf);
 }
 
 int forbid_play()
@@ -1132,10 +1161,8 @@ void do_specials() {
 
 int main(int argc, char **argv)
 {
-#ifdef PLUGINS
   int evtid;
   CFParm CFP;
-#endif
 #ifdef WIN32 /* ---win32 this sets the win32 from 0d0a to 0a handling */
 	_fmode = _O_BINARY ;
 #endif
@@ -1147,21 +1174,17 @@ int main(int argc, char **argv)
   settings.argc=argc;
   settings.argv=argv;
   init(argc, argv);
-#ifdef PLUGINS
   initPlugins();        /* GROS - Init the Plugins */
-#endif
   for(;;) {
     nroferrors = 0;
 
     doeric_server();
     process_events(NULL);    /* "do" something with objects with speed */
     cftimer_process_timers();/* Process the crossfire Timers */    
-#ifdef PLUGINS
     /* GROS : Here we handle the CLOCK global event */
     evtid = EVENT_CLOCK;
     CFP.Value[0] = (void *)(&evtid);
     GlobalEvent(&CFP);
-#endif
     check_active_maps(); /* Removes unused maps after a certain timeout */
     do_specials();       /* Routines called from time to time. */
 
