@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_move_c =
- *    "$Id: move.c,v 1.23 2002/09/07 05:55:18 mwedel Exp $";
+ *    "$Id: move.c,v 1.32 2004/04/16 06:23:43 mwedel Exp $";
  */
 
 /*
@@ -55,26 +55,32 @@ int move_object(object *op, int dir) {
   */
 int move_ob (object *op, int dir, object *originator)
 {
-    int newx = op->x+freearr_x[dir];
-    int newy = op->y+freearr_y[dir];
+    sint16 newx = op->x+freearr_x[dir];
+    sint16 newy = op->y+freearr_y[dir];
     object *tmp;
+    mapstruct *m;
+    int mflags;
+
+    m = op->map;
+    mflags = get_map_flags(m, &m, newx, newy, &newx, &newy);
 
     if(op==NULL) {
 	LOG(llevError,"Trying to move NULL.\n");
 	return 0;
     }
 
-    /* Is this space blocked?  Players with wizpass are immune to
-     * this condition.
-     */
-    if(blocked_link(op, newx, newy) &&
-       !QUERY_FLAG(op, FLAG_WIZPASS))
-	return 0;
-
     /* If the space the player is trying to is out of the map,
      * bail now - we know it can't work.
      */
-    if (out_of_map(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir])) return 0;
+    if (mflags & P_OUT_OF_MAP) return 0;
+
+
+    /* Is this space blocked?  Players with wizpass are immune to
+     * this condition.
+     */
+    if(blocked_link(op, m, newx, newy) &&
+       !QUERY_FLAG(op, FLAG_WIZPASS))
+	return 0;
 
     /* 0.94.2 - we need to set the direction for the new animation code.
      * it uses it to figure out face to use - I can't see it
@@ -86,9 +92,9 @@ int move_ob (object *op, int dir, object *originator)
     op->direction = dir;
 
     if(op->will_apply&4)
-	check_earthwalls(op,newx,newy);
+	check_earthwalls(op,m, newx,newy);
     if(op->will_apply&8)
-	check_doors(op,newx,newy);
+	check_doors(op,m, newx,newy);
 
     /* 0.94.1 - I got a stack trace that showed it crash with remove_ob trying
      * to remove a removed object, and this function was the culprit.  A possible
@@ -117,6 +123,7 @@ int move_ob (object *op, int dir, object *originator)
     for(tmp = op; tmp != NULL; tmp = tmp->more)
 	tmp->x+=freearr_x[dir], tmp->y+=freearr_y[dir];
 
+    /* insert_ob_in_map will deal with any tiling issues */
     insert_ob_in_map(op, op->map, originator,0);
 
     /* Hmmm.  Should be possible for multispace players now */
@@ -144,23 +151,28 @@ int move_ob (object *op, int dir, object *originator)
 
 int transfer_ob (object *op, int x, int y, int randomly, object *originator)
 {
-  int i;
-  object *tmp;
+    int i;
+    object *tmp;
 
-  if (randomly)
-    i = find_free_spot (op->arch,op->map,x,y,0,SIZEOFFREE);
-  else
-    i = find_first_free_spot(op->arch,op->map,x,y);
-  if (i==-1)
-    return 0;	/* No free spot */
+    if (randomly)
+	i = find_free_spot (op->arch,op->map,x,y,0,SIZEOFFREE);
+    else
+	i = find_first_free_spot(op->arch,op->map,x,y);
 
-  if(op->head!=NULL)
-    op=op->head;
-  remove_ob(op);
-  for(tmp=op;tmp!=NULL;tmp=tmp->more)
-    tmp->x=x+freearr_x[i]+(tmp->arch==NULL?0:tmp->arch->clone.x),
-    tmp->y=y+freearr_y[i]+(tmp->arch==NULL?0:tmp->arch->clone.y);
-  return insert_ob_in_map(op,op->map,originator,0) == NULL;
+    if (i==-1)
+	return 0;	/* No free spot */
+
+    if(op->head!=NULL)
+	op=op->head;
+    remove_ob(op);
+    for(tmp=op;tmp!=NULL;tmp=tmp->more)
+	tmp->x=x+freearr_x[i]+(tmp->arch==NULL?0:tmp->arch->clone.x),
+	tmp->y=y+freearr_y[i]+(tmp->arch==NULL?0:tmp->arch->clone.y);
+
+    tmp = insert_ob_in_map(op,op->map,originator,0);
+    if (op && op->type == PLAYER) MapNewmapCmd(op->contr);
+    if (tmp) return 0;
+    else return 1;
 }
 
 /*
@@ -193,7 +205,8 @@ int teleport (object *teleporter, uint8 tele_type, object *user)
 	for(j= -5;j<6;j++) {
 	    if(i==0&&j==0)
 		continue;
-	    if(out_of_map(teleporter->map,teleporter->x+i,teleporter->y+j))
+	    /* Perhaps this should be extended to support tiled maps */
+	    if(OUT_OF_REAL_MAP(teleporter->map,teleporter->x+i,teleporter->y+j))
 		continue;
 	    other_teleporter=get_map_ob(teleporter->map,
                                   teleporter->x+i,teleporter->y+j);
@@ -214,8 +227,30 @@ int teleport (object *teleporter, uint8 tele_type, object *user)
     other_teleporter=altern[RANDOM()%nrofalt];
     k=find_free_spot(user->arch,other_teleporter->map,
                         other_teleporter->x,other_teleporter->y,1,9);
-    if (k==-1)
-	return 0;
+
+    /* if k==-1, unable to find a free spot.  If this is shop
+     * mat that the player is using, find someplace to move
+     * the player - otherwise, player can get trapped in the shops
+     * that appear in random dungeons.  We basically just make
+     * sure the space isn't no pass (eg wall), and don't care
+     * about is alive.
+     */
+    if (k==-1) {
+	if (tele_type == SHOP_MAT && user->type == PLAYER) {
+	    for (k=1; k<9; k++) {
+		if (!(get_map_flags(other_teleporter->map, NULL, 
+			other_teleporter->x + freearr_x[k],
+			other_teleporter->y + freearr_y[k], NULL,NULL) &
+		      (P_NO_PASS | P_OUT_OF_MAP))) break;
+	    }
+	    if (k==9) {
+		LOG(llevError,"Shop mat %s (%d, %d) is in solid rock?\n",
+		    other_teleporter->name, other_teleporter->x, other_teleporter->y);
+		return 0;
+	    }
+	}
+	else return 0;
+    }
 
     remove_ob(user);
 
@@ -239,7 +274,7 @@ void recursive_roll(object *op,int dir,object *pusher) {
   }
   (void) move_ob(pusher,dir,pusher);
   new_draw_info_format(NDI_BLACK, 0, pusher,
-	"You roll the %s.",query_name(op));
+	"You move the %s.",query_name(op));
   return;
 }
 
@@ -251,20 +286,25 @@ void recursive_roll(object *op,int dir,object *pusher) {
  * This is currently only used for the boulder roll code.
  */
 
-int try_fit (object *op, int x, int y) 
+int try_fit (object *op, mapstruct *m, int x, int y) 
 {
     object *tmp, *more;
-    int tx, ty;
+    sint16 tx, ty;
+    int mflags;
+
     if (op->head) 
 	op = op->head;
 
     for (more = op; more ; more = more->more) {
 	tx = x + more->x - op->x;
 	ty = y + more->y - op->y;
-	if (out_of_map(op->map,tx,ty))
+
+	mflags = get_map_flags(m, &m, tx, ty, &tx, &ty);
+
+	if (mflags & P_OUT_OF_MAP)
 	    return 1;
 
-	for (tmp = get_map_ob (more->map, tx, ty); tmp; tmp=tmp->above) {
+	for (tmp = get_map_ob (m, tx, ty); tmp; tmp=tmp->above) {
 	    if (tmp->head == op || tmp == op)
 		continue;
 
@@ -283,33 +323,44 @@ int try_fit (object *op, int x, int y)
 /*
  * this is not perfect yet. 
  * it does not roll objects behind multipart objects properly.
+ * Support for rolling multipart objects is questionable.
  */
 
 int roll_ob(object *op,int dir, object *pusher) {
     object *tmp;
-    int x, y;
+    sint16 x, y;
+    int flags;
+    mapstruct *m;
+
     if (op->head) 
 	op = op->head;
+
     x=op->x+freearr_x[dir];
     y=op->y+freearr_y[dir];
+
     if(!QUERY_FLAG(op,FLAG_CAN_ROLL) || 
        (op->weight &&
        random_roll(0, op->weight/50000-1, pusher, PREFER_LOW) > pusher->stats.Str))
 	return 0;
 
-    if(out_of_map(op->map,x,y))
+    m = op->map;
+    flags = get_map_flags(m, &m, x, y, &x, &y);
+
+    if (flags & (P_OUT_OF_MAP | P_IS_ALIVE))
 	return 0;
 
-    for(tmp=get_map_ob(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]);
-	tmp!=NULL;tmp=tmp->above) {
+    
+
+    /* If the target space is not blocked, no need to look at the objects on it */
+    if (flags & P_BLOCKED) {
+	for (tmp=get_map_ob(m, x, y); tmp!=NULL; tmp=tmp->above) {
 	if (tmp->head == op)
 	    continue;
-	if(QUERY_FLAG(tmp,FLAG_ALIVE))
-	    return 0;
 	if (QUERY_FLAG(tmp,FLAG_NO_PASS) && !roll_ob(tmp,dir,pusher))
 	    return 0;
+	}
     }
-    if (try_fit (op, x, y))
+    if (try_fit (op, m, x, y))
 	return 0;
 
     remove_ob(op);
@@ -335,14 +386,22 @@ int push_ob(object *who, int dir, object *pusher) {
     /* TODO: allow multi arch pushing. Can't be very difficult */
     if (who->more == NULL && owner == pusher) {
 	int temp;
+	mapstruct *m;
+
 	remove_ob(who);
 	remove_ob(pusher);
 	temp = pusher->x;
 	pusher->x = who->x;
 	who->x = temp;
+
 	temp = pusher->y;
 	pusher->y = who->y;
 	who->y = temp;
+
+	m = pusher->map;
+	pusher->map = who->map;
+	who->map = m;
+
 	insert_ob_in_map (who,who->map,pusher,0);
 	insert_ob_in_map (pusher,pusher->map,pusher,0);
 
@@ -376,7 +435,7 @@ int push_ob(object *who, int dir, object *pusher) {
 	else 
 	{
 	    new_draw_info_format(NDI_UNIQUE, 0, pusher,
-				 "You avoid to attack %s .",who->name);
+				 "You avoid attacking %s .",who->name);
 	}
     }
 
