@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_plugins_c =
- *   "$Id: plugins.c,v 1.17 2002/09/07 22:08:07 mwedel Exp $";
+ *   "$Id: plugins.c,v 1.24 2004/04/25 07:17:40 mwedel Exp $";
  */
 
 /*****************************************************************************/
@@ -222,7 +222,7 @@ void initOnePlugin(char* pluginfile)
         HookParm = (CFParm *)(malloc(sizeof(CFParm)));
         HookParm->Value[0]=(int *)(malloc(sizeof(int)));
 
-        for(j=1; j<=NR_OF_HOOKS;j++)
+        for(j=1; j<NR_OF_HOOKS;j++)
         {
             memcpy(HookParm->Value[0], &j, sizeof(int));
             HookParm->Value[1] = HookList[j];
@@ -543,7 +543,7 @@ void initOnePlugin(char* pluginfile)
                 HookParm = (CFParm *)(malloc(sizeof(CFParm)));
                 HookParm->Value[0]=(int *)(malloc(sizeof(int)));
 
-                for(j=1; j<=NR_OF_HOOKS;j++)
+                for(j=1; j<NR_OF_HOOKS;j++)
                 {
                     memcpy(HookParm->Value[0], &j, sizeof(int));
                     HookParm->Value[1] = HookList[j];
@@ -772,11 +772,8 @@ CFParm* CFWSpringTrap(CFParm* PParm)
 /* 0 - op;                                                                   */
 /* 1 - caster;                                                               */
 /* 2 - direction;                                                            */
-/* 3 - type of casting;                                                      */
-/* 4 - is it an ability or a wizard spell ?                                  */
-/* 5 - spelltype;                                                            */
-/* 6 - optional args;                                                        */
-/* 7 - type of firing;                                                       */
+/* 3 - spell object                                                          */
+/* 4 - optional args;                                                        */
 /*                                                                           */
 /*****************************************************************************/
 CFParm* CFWCastSpell(CFParm* PParm)
@@ -784,12 +781,10 @@ CFParm* CFWCastSpell(CFParm* PParm)
     static int val;
     CFParm *CFP;
     CFP = (CFParm*)(malloc(sizeof(CFParm)));
-/*int cast_spell(object *op, object *caster, int dir, int type, int ability, */
-/*SpellTypeFrom item, char *stringarg); */
+    /* int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *stringarg)*/
     val = cast_spell((object *)(PParm->Value[0]),(object *)(PParm->Value[1]),
-        *(int *)(PParm->Value[2]),*(int *)(PParm->Value[3]),
-        *(int *)(PParm->Value[4]),*(SpellTypeFrom *)(PParm->Value[5]),
-        (char *)(PParm->Value[6])/*,*(int *) (PParm->Value[7])*/);
+        *(int*)(PParm->Value[2]), (object*)(PParm->Value[3]),
+        (char *)(PParm->Value[4]));
     CFP->Value[0] = (void *)(&val);
     return CFP;
 };
@@ -1009,7 +1004,7 @@ CFParm* CFWDoForgetSpell(CFParm* PParm)
 {
     do_forget_spell(
         (object *)(PParm->Value[0]),
-        *(int *)(PParm->Value[1])
+        (char *)(PParm->Value[1])
     );
     return NULL;
 };
@@ -1025,7 +1020,7 @@ CFParm* CFWDoLearnSpell(CFParm* PParm)
 {
     do_learn_spell(
         (object *)(PParm->Value[0]),
-        *(int *)(PParm->Value[1]),
+        (object *)(PParm->Value[1]),
         *(int *)(PParm->Value[2])
     );
     return NULL;
@@ -1040,13 +1035,14 @@ CFParm* CFWDoLearnSpell(CFParm* PParm)
 CFParm* CFWCheckSpellKnown(CFParm* PParm)
 {
     CFParm *CFP;
-    static int val;
+    static object *val;
+
     CFP = (CFParm*)(malloc(sizeof(CFParm)));
     val = check_spell_known(
         (object *)(PParm->Value[0]),
-        *(int *)(PParm->Value[1])
+        (char *)(PParm->Value[1])
         );
-    CFP->Value[0] = &val;
+    CFP->Value[0] = val;
     return CFP;
 };
 
@@ -1210,12 +1206,16 @@ CFParm* CFWReadyMapName(CFParm* PParm)
 /*****************************************************************************/
 /* 0 - object to increase experience of.                                     */
 /* 1 - amount of experience to add.                                          */
+/* 2 - skill name to add it to						     */
+/* 3 - flags for this							     */
 /*****************************************************************************/
 CFParm* CFWAddExp(CFParm* PParm)
 {
-    add_exp(
+    change_exp(
         (object *)(PParm->Value[0]),
-        *(int *)(PParm->Value[1])
+        *(int *)(PParm->Value[1]), 
+	(char*)(PParm->Value[2]),
+	*(int *)(PParm->Value[3])
     );
     return(PParm);
 };
@@ -1367,6 +1367,10 @@ CFParm* CFWQueryCost(CFParm* PParm)
     whatptr = (object *)(PParm->Value[0]);
     whoptr = (object *)(PParm->Value[1]);
     flag = *(int*)(PParm->Value[2]);
+    /* This is now broken since query_cost returns 64 bit -
+     * not positive how well/what needs to be changed for the plugin
+     * to know this and act accordingly.
+     */
     val=query_cost (whatptr,whoptr,flag);
     CFP->Value[0] = (void*) &val;
     return CFP;
@@ -1519,7 +1523,7 @@ CFParm* CFWSetAnimation (CFParm* PParm)
         SET_ANIMATION (op,face);
         }
     update_object(op, UP_OBJ_FACE);
-    return(PParm);    
+    return(PParm);
 }
 /*****************************************************************************/
 /* communicate wrapper.                                                      */
@@ -1581,6 +1585,31 @@ CFParm* CFWFreeObject (CFParm* PParm)
     if (op) free_object(op);
     return NULL;
 }
+
+/*****************************************************************************/
+/* fix_player wrapper.                                                      */
+/*****************************************************************************/
+/* 0 - object                                                                */
+/*****************************************************************************/
+CFParm* CFWFixPlayer (CFParm* PParm)
+{
+    object* op=(object*)PParm->Value[ 0 ];
+    if (op) fix_player(op);
+    return NULL;
+}
+
+/*****************************************************************************/
+/* free wrapper.                                                      */
+/*****************************************************************************/
+/* 0 - object                                                                */
+/*****************************************************************************/
+CFParm* CFWFreeMemory(CFParm* PParm)
+{
+    void* op = PParm->Value[ 0 ];
+    if (op) free(op);
+    return NULL;
+}
+
 /*****************************************************************************/
 /* ObjectCreateClone object_copy wrapper.                                    */
 /*****************************************************************************/
@@ -1602,6 +1631,30 @@ CFParm* CFWObjectCreateClone (CFParm* PParm)
         }
     return CFP;
 }
+/*****************************************************************************/
+/* Sets a variable in a given object.                                        */
+/*****************************************************************************/
+/* 0 - object to change                                                      */
+/* 1 - string describing the change to make.                                 */
+/*****************************************************************************/
+CFParm* CFWSetVariable(CFParm* PParm)
+{
+    set_variable((object*)(PParm->Value[0]), (char *)(PParm->Value[1]));
+    return NULL;
+}
+
+CFParm* CFWDecreaseObjectNR(CFParm* PParm)
+{
+    CFParm* CFP=(CFParm*)malloc(sizeof (CFParm));
+    object* op=(object*)PParm->Value[0];
+    int i=*(int*)PParm->Value[1];
+    object* ob;
+
+    ob = decrease_ob_nr (op, i);
+    CFP->Value[0] = ob;
+    return CFP;
+}
+
 /*****************************************************************************/
 /* teleport an object to another map                                         */
 /*****************************************************************************/
@@ -1615,7 +1668,7 @@ CFParm* CFWObjectCreateClone (CFParm* PParm)
 CFParm* CFWTeleportObject (CFParm* PParm)
 {
     object* current;
-/*    char * mapname; not used 
+/*    char * mapname; not used
     int mapx;
     int mapy;
     int unique; not used */
