@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_swap_c =
- *    "$Id: swap.c,v 1.9 2002/04/21 05:23:20 mwedel Exp $";
+ *    "$Id: swap.c,v 1.14 2003/03/03 05:13:08 mwedel Exp $";
  */
 
 /*
@@ -32,7 +32,6 @@
 #endif
 #include <object.h>
 
-#ifdef RECYCLE_TMP_MAPS
 /* This writes out information on all the temporary maps.  It is called by
  * swap_map below.
  */
@@ -67,7 +66,6 @@ static void write_map_log()
     }
     fclose(fp);
 }
-#endif
 
 void read_map_log() 
 {
@@ -75,6 +73,8 @@ void read_map_log()
     mapstruct *map;
     char buf[MAX_BUF],*cp,*cp1;
     int do_los, darkness, lock;
+    long sec =  seconds();
+
 
     sprintf(buf,"%s/temp.maps", settings.localdir);
     if (!(fp=fopen(buf,"r"))) {
@@ -103,18 +103,30 @@ void read_map_log()
 		    &lock, &map->difficulty, &do_los,
 		    &darkness);
 
+
 	map->in_memory=MAP_SWAPPED;
 	map->darkness=darkness;
+
+	/* When the reset time is saved out, it is adjusted so that
+	 * the current time is subtracted (thus, it is saved as number
+         * of seconds from current time that it should reset).  We need
+	 * to add in the current seconds for this to work right.
+	 * On metalforge, strange behavior was observed with really high
+	 * reset times - I don't know how they got to that state, 
+	 * but easy enough to do some sanity checking here.
+	 */
+	map->reset_time += sec;
+	if (map->reset_time > (sec + MAP_MAXRESET))
+	    map->reset_time = 0;
+
     }
     fclose(fp);
 }
 
 void swap_map(mapstruct *map) {
     player *pl;
-#ifdef PLUGINS
     int evtid;
     CFParm CFP;
-#endif
 
     if(map->in_memory != MAP_IN_MEMORY) {
 	LOG(llevError,"Tried to swap out map which was not in memory.\n");
@@ -141,13 +153,11 @@ void swap_map(mapstruct *map) {
 	mapstruct *oldmap = map;
 
 	LOG(llevDebug,"Resetting map %s.\n",map->path);
-#ifdef PLUGINS
 	/* GROS : Here we handle the MAPRESET global event */
 	evtid = EVENT_MAPRESET;
 	CFP.Value[0] = (void *)(&evtid);
 	CFP.Value[1] = (void *)(map->path);
 	GlobalEvent(&CFP);
-#endif
 	map = map->next;
 	delete_map(oldmap);
 	return;
@@ -164,9 +174,8 @@ void swap_map(mapstruct *map) {
 	free_map(map,1);
     }
 
-#ifdef RECYCLE_TMP_MAPS
-    write_map_log();
-#endif
+    if (settings.recycle_tmp_maps == TRUE)
+	write_map_log();
 }
 
 void check_active_maps() {
@@ -231,13 +240,16 @@ void swap_below_max(char *except_level) {
 /*
  * players_on_map(): will be replaced by map->players when I'm satisfied
  * that the variable is always correct.
+ * If show_all is true, we show everyone.  If not, we don't show hidden
+ * players (dms)
  */
 
-int players_on_map(mapstruct *m) {
+int players_on_map(mapstruct *m, int show_all) {
   player *pl;
   int nr=0;
   for(pl=first_player;pl!=NULL;pl=pl->next)
-    if(pl->ob != NULL && !QUERY_FLAG(pl->ob,FLAG_REMOVED) && pl->ob->map==m)
+    if(pl->ob != NULL && !QUERY_FLAG(pl->ob,FLAG_REMOVED) && pl->ob->map==m &&
+       (show_all || !pl->hidden))
       nr++;
   return nr;
 }
@@ -252,10 +264,8 @@ void flush_old_maps() {
 
     mapstruct *m, *oldmap;
     long sec;
-#ifdef PLUGINS
     int evtid;
     CFParm CFP;
-#endif
     sec = seconds();
 
     m= first_map;
@@ -264,7 +274,7 @@ void flush_old_maps() {
 	 * is not set so it isn't swapped out.
 	 */
 	if ((m->in_memory == MAP_IN_MEMORY) && (m->timeout==0) &&
-	    !players_on_map(m)) {
+	    !players_on_map(m,TRUE)) {
 	    set_map_timeout(m);
 	}
 
@@ -284,25 +294,21 @@ void flush_old_maps() {
 	    m = m->next;
 	    delete_map(oldmap);
 	}
-#ifdef MAP_RESET /* No need to flush them if there are no resets */
 	else if(m->in_memory != MAP_SWAPPED || m->tmpname == NULL ||
 	   sec < m->reset_time) {
 	    m = m->next;
 	}
 	else {
 	    LOG(llevDebug,"Resetting map %s.\n",m->path);
-#ifdef PLUGINS
 	    /* GROS : Here we handle the MAPRESET global event */
 	    evtid = EVENT_MAPRESET;
 	    CFP.Value[0] = (void *)(&evtid);
 	    CFP.Value[1] = (void *)(m->path);
 	    GlobalEvent(&CFP);
-#endif    
 	clean_tmp_map(m);
 	oldmap = m;
 	m = m->next;
 	delete_map(oldmap);
 	}
-#endif
     }
 }
