@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_sock_info_c =
- *   "$Id: info.c,v 1.8 2002/07/15 04:57:13 mwedel Exp $";
+ *   "$Id: info.c,v 1.17 2004/01/18 10:36:10 ryo_saeba Exp $";
  */
 
 /*
@@ -26,7 +26,13 @@
     The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
-/* This file implements some of the simpler output functions to the
+/**
+ * \file
+ * Basic client output functions.
+ *
+ * \date 2003-12-02
+ *
+ * This file implements some of the simpler output functions to the
  * client.  Basically, things like sending text strings along
  */
 
@@ -36,8 +42,8 @@
 #include <spells.h>
 #include <skills.h>
 
-/*
- * esrv_print_msg draws a normal message on the client.  It is pretty 
+/**
+ * Draws a normal message on the client.  It is pretty 
  * much the same thing as the draw_info above, but takes a color
  * parameter.  the esrv_drawinfo functions should probably be
  * replaced with this, just using black as the color.
@@ -55,19 +61,17 @@ static void esrv_print_msg(NewSocket *ns,int color, const char *str)
     Write_String_To_Socket(ns, buf, strlen(buf));
 }
 
-/****************************************************************************
-print_message : This routine prints out the character string in tmp on the
-                info window. If in color mode, then the text will show up in
-                a color specified by the variable colr. If Black and white
-                mode, then it prints a line of "="s before and after the
-                messsage.
 
-		This has been changed around - this is now a front end to
-		draw_info.  draw_info should never be called directly, except
-		by this functin.  Likewise, this function should only be
-		called by a few functions above (new_draw_info, 
-		check_output_buffers)
-****************************************************************************/
+/**
+ * Frontend for esrv_print_msg
+ * \param colr message color
+ * \param pl player to send to. Can be NULL
+ * \param tmp message to send. Can be NULL
+ *
+ * If pl is NULL or without contr set, writes message to log.
+ *
+ * Else sends message to player via esrv_print_msg
+ */
 
 static void print_message(int colr, object *pl,const char *tmp) {
 
@@ -86,7 +90,8 @@ static void print_message(int colr, object *pl,const char *tmp) {
 }
 
 
-/* Following prints out the contents of one of the buffer structures,
+/**
+ * Prints out the contents of specified buffer structures,
  * and clears the string.
  */
 
@@ -102,8 +107,19 @@ void flush_output_element(object *pl, Output_Buf *outputs)
     outputs->first_update=0;	/* This way, it will be reused */
 }
 
-/* Following checks the various buffers in the player structure and
- * other things, and stores/prints/whatever's the data, as appropriate.
+/**
+ * Sends message to player through output buffers.
+ * \param pl player to send message
+ * \param buf message to send
+ *
+ * If player's output_count is 1, sends message directly.
+ *
+ * Else checks output buffers for specified message.
+ *
+ * If found, checks if message should be sent now.
+ *
+ * If message not already in buffers, flushes olders buffer,
+ * and adds message to queue.
  */
 
 void check_output_buffers(object *pl, char *buf)
@@ -144,8 +160,8 @@ void check_output_buffers(object *pl, char *buf)
 	    
 
 
-/*
- * new_draw_info:
+/**
+ * Sends message to player(s).
  *
  * flags is various flags - mostly color, plus a few specials.
  *
@@ -157,9 +173,10 @@ void check_output_buffers(object *pl, char *buf)
  * pl can be passed as NULL - in fact, this will be done if NDI_ALL is set
  * in the flags.
  *
+ * If message is black, and not NDI_UNIQUE, gets sent through output buffers.
+ *
  */
 
-
 void new_draw_info(int flags,int pri, object *pl, const char *buf)
 {
 
@@ -171,7 +188,7 @@ void new_draw_info(int flags,int pri, object *pl, const char *buf)
 		new_draw_info((flags & ~NDI_ALL), pri, tmppl->ob, buf);
 
 	for (i=1; i<socket_info.allocated_sockets; i++) {
-	    if (init_sockets[i].status == Ns_Old && init_sockets[i].old_mode != Old_Listen) {
+	    if (init_sockets[i].status == Ns_Old && init_sockets[i].old_mode != Old_Listen && pri< 10) {
 		cs_write_string(&init_sockets[i], buf, strlen(buf));
 		/* Most messages don't have a newline, so add one */
 		cs_write_string(&init_sockets[i], "\n", 1);
@@ -197,9 +214,12 @@ void new_draw_info(int flags,int pri, object *pl, const char *buf)
     }
 }
 
-/* This is a pretty trivial function, but it allows us to use printf style
+/**
+ * Wrapper for new_draw_info printf-like.
+ *
+ * This is a pretty trivial function, but it allows us to use printf style
  * formatting, so instead of the calling function having to do it, we do
- * it here.  IT may also have advantages in the future for reduction of
+ * it here.  It may also have advantages in the future for reduction of
  * client/server bandwidth (client could keep track of various strings
  */
 
@@ -217,8 +237,8 @@ void new_draw_info_format(int flags, int pri,object *pl, char *format, ...)
     new_draw_info(flags, pri, pl, buf);
 }
 
-/*
- * write to everyone on the map *except* op.  This is useful for emotions.
+/**
+ * Writes to everyone on the map *except* op.  This is useful for emotions.
  */
 
 void new_info_map_except(int color, mapstruct *map, object *op, char *str) {
@@ -230,8 +250,8 @@ void new_info_map_except(int color, mapstruct *map, object *op, char *str) {
 	}
 }
 
-/*
- * write to everyone on the map except op1 and op2
+/**
+ * Writes to everyone on the map except op1 and op2
  */
 
 void new_info_map_except2(int color, mapstruct *map, object *op1, object *op2,
@@ -245,8 +265,8 @@ void new_info_map_except2(int color, mapstruct *map, object *op1, object *op2,
 	}
 }
 
-/*
- * write to everyone on the current map
+/**
+ * Writes to everyone on the specified map
  */
 
 void new_info_map(int color, mapstruct *map, char *str) {
@@ -258,21 +278,9 @@ void new_info_map(int color, mapstruct *map, char *str) {
 	}
 }
 
-void draw(object *pl) {
 
-    if(pl->map == NULL || pl->map->in_memory != MAP_IN_MEMORY)
-	return;
-
-    if(pl->contr->do_los) {
-	update_los(pl);
-	pl->contr->do_los = 0;
-    }
-
-    draw_client_map(pl);
-}
-
-
-/* This does nothing now.  However, in theory, we should probably send
+/**
+ * This does nothing now.  However, in theory, we should probably send
  * something to the client and let the client figure out how it might want
  * to handle this
  */
@@ -280,73 +288,82 @@ void clear_win_info(object *op)
 {
 }
 
+/**
+ * Get player's current range attack in obuf.
+ */
 void rangetostring(object *pl,char *obuf)
 {
-  int chosen_spell;
+    switch(pl->contr->shoottype) {
+	case range_none:
+	    strcpy(obuf,"Range: nothing");
+	    break;
 
-  chosen_spell = (pl->contr->shoottype==range_magic)? pl->contr->chosen_spell :
-      pl->contr->chosen_item_spell;
-  switch(pl->contr->shoottype) {
-   case range_none:
-    strcpy(obuf,"Range: nothing");
-    break;
-   case range_bow: {
-     char *s;
-     object *op;
-     for (op = pl->inv; op; op=op->below)
-       if (op->type == BOW && QUERY_FLAG (op, FLAG_APPLIED))
-       break;
-     if(op==NULL) break;
-     s = query_name(op);
-#if 1                         /* Hack to remove (readied) from a bow description */
-     if (strcmp (s + strlen (s) - 10, " (readied)") == 0)
-       s[strlen (s) - 10] = 0;
-#endif
-     sprintf (obuf, "Range: %s (%s)", s, 
-            op && op->race ? op->race : "nothing");
-   }
-    break;
-   case range_magic:
-#ifdef CASTING_TIME
-    if (pl->casting > -1) {
-      if (pl->casting == 0)
-      sprintf(obuf,"Range: Holding spell (%s)",
-              pl->spell->name);
-      else
-      sprintf(obuf,"Range: Casting spell (%s)",
-              pl->spell->name);
+	case range_bow:
+	    {
+	    object *op;
+
+	    for (op = pl->inv; op; op=op->below)
+		if (op->type == BOW && QUERY_FLAG (op, FLAG_APPLIED))
+		    break;
+	    if(op==NULL) break;
+
+	    sprintf (obuf, "Range: %s (%s)", query_base_name(op, 0), 
+		     op->race ? op->race : "nothing");
+	    }
+	    break;
+
+	case range_magic:
+	    if (settings.casting_time == TRUE) {
+		if (pl->casting_time > -1) {
+		    if (pl->casting_time == 0)
+			sprintf(obuf,"Range: Holding spell (%s)",
+				pl->spell->name);
+		    else
+			sprintf(obuf,"Range: Casting spell (%s)",
+				pl->spell->name);
+		} else
+		    sprintf(obuf,"Range: spell (%s)", 
+			    pl->contr->ranges[range_magic]->name);
+	    } else
+		sprintf(obuf,"Range: spell (%s)", 
+			pl->contr->ranges[range_magic]->name);
+	    break;
+
+	case range_misc:
+	    sprintf(obuf,"Range: %s", 
+		    pl->contr->ranges[range_misc]?
+		    query_base_name(pl->contr->ranges[range_misc],0): "none");
+	    break;
+
+	/* range_scroll is only used for controlling golems.  If the
+	 * the player does not have a golem, reset some things.
+	 */
+	case range_golem:
+	    if (pl->contr->ranges[range_golem]!=NULL)
+		sprintf(obuf,"Range: golem (%s)",pl->contr->ranges[range_golem]->name);
+	    else {
+		pl->contr->shoottype = range_none;
+		strcpy(obuf,"Range: nothing");
+	    }
+	    break;
+
+	case range_skill:
+	    sprintf(obuf,"Skill: %s", pl->chosen_skill!=NULL ?
+		    pl->chosen_skill->name : "none");
+	    break;
+
+    case range_builder:
+        sprintf( obuf, "Builder: %s", query_base_name( pl->contr->ranges[ range_builder ], 0 ) );
+        break;
+
+	default:
+	    strcpy(obuf,"Range: illegal");
     }
-    else
-#endif
-      sprintf(obuf,"Range: spell (%s)", 
-              spells[pl->contr->chosen_spell].name);
-    break;
-
-   case range_misc:
-    sprintf(obuf,"Range: %s", 
-	    pl->contr->ranges[range_misc]?query_base_name(pl->contr->ranges[range_misc],0): "none");
-    break;
-
-    /* range_scroll is only used for controlling golems.  If the
-     * the player does not have a golem, reset some things.
-     */
-   case range_golem:
-    if (pl->contr->golem!=NULL)
-      sprintf(obuf,"Range: golem (%s)",pl->contr->golem->name);
-    else {
-      pl->contr->shoottype = range_none;
-      strcpy(obuf,"Range: nothing");
-    }
-    break;
-   case range_skill:
-       sprintf(obuf,"Skill: %s", pl->chosen_skill!=NULL ?
-		skills[pl->chosen_skill->stats.sp].name : "none");
-    break;
-   default:
-    strcpy(obuf,"Range: illegal");
-  }
 }
 
+/**
+ * Sets player title.
+ */
 void set_title(object *pl,char *buf)
 {
     /* Eneq(@csd.uu.se): Let players define their own titles. */
@@ -357,12 +374,68 @@ void set_title(object *pl,char *buf)
 }
 
 
+/**
+ * Helper for magic map creation.
+ *
+ * Takes a player, the map_mark array and an x and y starting position.
+ * pl is the player.
+ * px, py are offsets from the player.
+ *
+ * This function examines all the adjacant spaces next to px, py.
+ * It updates the map_mark arrow with the color and high bits set
+ * for various code values.
+ */
+static void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int py)
+{
+    int x, y, dx, dy,mflags;
+    sint16 nx, ny;
+    mapstruct *mp;
+    New_Face *f;
 
-/* Note:  For improved magic mapping display, the space that blocks
+    for (dx = -1; dx <= 1; dx++) {
+	for (dy = -1; dy <= 1; dy++) {
+	    x = px + dx;
+	    y = py + dy;
+
+	    if (FABS(x) >= MAGIC_MAP_HALF || FABS(y) >= MAGIC_MAP_HALF) continue;
+
+	    mp = pl->map;
+	    nx = pl->x + x;
+	    ny = pl->y + y;
+
+	    mflags = get_map_flags(pl->map, &mp, nx, ny, &nx, &ny);
+	    if (mflags & P_OUT_OF_MAP) continue;
+
+	    if (map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] == 0) {
+		f= GET_MAP_FACE(mp, nx, ny, 0);
+		if (f == blank_face)
+		    f= GET_MAP_FACE(mp, nx, ny, 1);
+		if (f == blank_face)
+		    f= GET_MAP_FACE(mp, nx, ny, 2);
+
+		/* Should probably have P_NO_MAGIC here also, but then shops don't
+		 * work.
+		 */
+		if (mflags & P_BLOCKSVIEW)
+		    map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] = FACE_WALL | (f?f->magicmap:0);
+		else {
+		    map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] = FACE_FLOOR | (f?f->magicmap:0);
+		    magic_mapping_mark_recursive(pl, map_mark, x, y);
+		}
+	    }
+	}
+    }
+}
+
+
+/**
+ * Creates magic map for player.
+ *
+ * Note:  For improved magic mapping display, the space that blocks
  * the view is now marked with value 2.  Any dependencies of map_mark
  * being nonzero have been changed to check for 1.  Also, since
  * map_mark is a char value, putting 2 in should cause no problems.
- * Mark Wedel
+ *
  * This function examines the map the player is on, and determines what
  * is visible.  2 is set for walls or objects that blocks view.  1
  * is for open spaces.  map_mark should already have been initialized
@@ -373,149 +446,91 @@ void set_title(object *pl,char *buf)
 
 void magic_mapping_mark(object *pl, char *map_mark, int strength)
 {
-  int x, y;
-  int xmin = pl->x - strength + 1 < 0 ? 0 : pl->x - strength + 1;
-  int xmax = pl->x + strength - 1 > MAP_WIDTH(pl->map) - 1 ? 
-    MAP_WIDTH(pl->map) - 1 : pl->x + strength - 1;
-  int ymin = pl->y - strength + 1 < 0 ? 0 : pl->y - strength + 1;
-  int ymax = pl->y + strength - 1 > MAP_HEIGHT(pl->map) - 1 ? 
-    MAP_HEIGHT(pl->map) - 1 : pl->y + strength - 1;
+    int x, y, mflags;
+    sint16 nx, ny;
+    mapstruct *mp;
+    New_Face *f;
 
-  for (x = xmin; x <= xmax; x++) {
-    for (y = ymin; y <= ymax; y++) {
-      if (wall(pl->map, x, y) || blocks_view(pl->map, x, y))
-	map_mark[x + MAP_WIDTH(pl->map) * y] = 2;
-      else {
-	map_mark[x + MAP_WIDTH(pl->map) * y] = 1;
-	magic_mapping_mark_recursive(pl, map_mark, x, y);
-      }
-    }
-  }
-}
-
-/* Takes a player, the map_mark array and an x and y starting position.
- * pl could be replaced by the map, since that is all taht pl is
- * used for.
- * This function examines all the adjacant spaces next to px, py.
- * If there is a wall or it otherwise blocks view on a space, we set
- * map_mark to 2.
- * If the space is otherwise open, we set it to 1, and this function
- * is called again to examine those spaces.
- */
-void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int py)
-{
-  int x, y, dx, dy;
-
-  for (dx = -1; dx <= 1; dx++) {
-    for (dy = -1; dy <= 1; dy++) {
-      x = px + dx;
-      y = py + dy;
-      if (x >= 0 && x < MAP_WIDTH(pl->map) && y >= 0 && y < MAP_HEIGHT(pl->map)
-	&& (map_mark[x + MAP_WIDTH(pl->map) * y] ==0) ) {
-            if (blocks_view(pl->map, x, y))
-		map_mark[x + MAP_WIDTH(pl->map) * y] = 2;
+    for (x = -strength; x <strength; x++) {
+	for (y = -strength; y <strength; y++) {
+	    mp = pl->map;
+	    nx = pl->x + x;
+	    ny = pl->y + y;
+	    mflags = get_map_flags(pl->map, &mp, nx, ny, &nx, &ny);
+	    if (mflags & P_OUT_OF_MAP)
+		continue;
 	    else {
-		if (wall(pl->map, x, y))
-		    map_mark[x + MAP_WIDTH(pl->map) * y] = 2;
-		else
-		    map_mark[x + MAP_WIDTH(pl->map) * y] = 1;
+		f= GET_MAP_FACE(mp, nx, ny, 0);
+		if (f == blank_face)
+		    f= GET_MAP_FACE(mp, nx, ny, 1);
+		if (f == blank_face)
+		    f= GET_MAP_FACE(mp, nx, ny, 2);
+	    }
+
+	    if (mflags & P_BLOCKSVIEW)
+		map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] = FACE_WALL | (f?f->magicmap:0);
+	    else {
+		map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] = FACE_FLOOR | (f?f->magicmap:0);
 		magic_mapping_mark_recursive(pl, map_mark, x, y);
 	    }
 	}
     }
-  }
 }
 
 
-
-/* The following function is a lot messier than it really should be,
+/**
+ * Creates and sends magic map to player.
+ *
+ * The following function is a lot messier than it really should be,
  * but there is no real easy solution.
  *
- * One of the main causes is uses the crossfire font to draw the stipple
- * pattern.  This then means that the excess needs to be erased.  As things
- * stand now, the excess is erased, and things look ok.
- *
- * Also, display on black and white system is still not as good (useful)
- * as on a color system.  However, things are not too bad.  At present, there
- * are 4 possible outputs:  White, meaning a wall, black, meaning
- * nothing (or only floor), grey (stippled pattern), for any other objects
- * that do not have a black foreground, and another stippled patern for
- * objects that do have a black foreground.
- *
- * Display of the stipples is not perfect.  One of the stipples is just
- * a checkerboard pattern.  IF the resolution is odd, and two of these
- * are placed together, little imperfections in the matching shows up.
- * However, it doesn't affect the usefulness of the display much, and
- * I don't want to add more code to deal with making the stipple perfect.
- * This is because the second stipple pattern used has a different
- * repeat rate
- *
  * Mark Wedel
  */
 
-void draw_map(object *pl) 
+void draw_magic_map(object *pl) 
 {
     int x,y;
-    char *map_mark = (char *) malloc(MAP_WIDTH(pl->map) * MAP_HEIGHT(pl->map));
-    int xmin = MAP_WIDTH(pl->map), xmax = 0, ymin = MAP_HEIGHT(pl->map), ymax = 0;
+    char *map_mark = (char *) calloc(MAGIC_MAP_SIZE*MAGIC_MAP_SIZE, 1);
+    int xmin, xmax, ymin, ymax;
     SockList sl;
 
     if (pl->type!=PLAYER) {
-	LOG(llevError,"Non player objectg called draw_map.\n");
+	LOG(llevError,"Non player object called draw_map.\n");
 	return;
     }
    
     /* First, we figure out what spaces are 'reachable' by the player */
-    memset(map_mark, 0, MAP_WIDTH(pl->map) * MAP_HEIGHT(pl->map));
     magic_mapping_mark(pl, map_mark, 3);
-    for(x = 0; x < MAP_WIDTH(pl->map); x++) {
-      for(y = 0; y < MAP_HEIGHT(pl->map); y++) {
-        if (map_mark[x + MAP_WIDTH(pl->map) * y]==1) {
-	  xmin = x < xmin ? x : xmin;
-	  xmax = x > xmax ? x : xmax;
-	  ymin = y < ymin ? y : ymin;
-	  ymax = y > ymax ? y : ymax;
-        }
-      }
+
+    /* We now go through and figure out what spaces have been
+     * marked, and thus figure out rectangular region we send
+     * to the client (eg, if only a 10x10 area is visible, we only
+     * want to send those 100 spaces.)
+     */
+    xmin = MAGIC_MAP_SIZE;
+    ymin = MAGIC_MAP_SIZE;
+    xmax = 0;
+    ymax = 0;
+    for(x = 0; x < MAGIC_MAP_SIZE ; x++) {
+	for(y = 0; y < MAGIC_MAP_SIZE; y++) {
+	    if (map_mark[x + MAP_WIDTH(pl->map) * y] | FACE_FLOOR) {
+		xmin = x < xmin ? x : xmin;
+		xmax = x > xmax ? x : xmax;
+		ymin = y < ymin ? y : ymin;
+		ymax = y > ymax ? y : ymax;
+	    }
+	}
     }
 
-    xmin--;
-    xmin = xmin < 0 ? 0 : xmin;
-    xmax++;
-    xmax = xmax > MAP_WIDTH(pl->map) - 1 ? MAP_WIDTH(pl->map) - 1: xmax;
-    ymin--;
-    ymin = ymin < 0 ? 0 : ymin;
-    ymax++;
-    ymax = ymax > MAP_HEIGHT(pl->map) - 1? MAP_HEIGHT(pl->map) - 1: ymax;
-    
     sl.buf=malloc(MAXSOCKBUF);
     sprintf((char*)sl.buf,"magicmap %d %d %d %d ", (xmax-xmin+1), (ymax-ymin+1),
-	    pl->x - xmin, pl->y - ymin);
+	    MAGIC_MAP_HALF - xmin, MAGIC_MAP_HALF - ymin);
     sl.len=strlen((char*)sl.buf);
     
-    /* Reversed ordering of X and Y in 0.93.2.  This way the order should
-     * match up the way we said it would
-     */
     for (y = ymin; y <= ymax; y++) {
-      for (x = xmin; x <= xmax; x++) {
-	    int mark;
-	    
-	    if ((mark=map_mark[x+MAP_WIDTH(pl->map)*y])==0)
-		sl.buf[sl.len++]=0;
-	    else {
-	        /* get map face and assign the proper magicmap value */
-		New_Face *f = GET_MAP_FACE(pl->map, x, y, 0);
-		if (f==NULL) {
-		    /* this map spot is completely empty */
-		    sl.buf[sl.len++]=0;
-		}
-		else if (mark==2)
-		    sl.buf[sl.len++]=f->magicmap | FACE_WALL;
-		else
-		    sl.buf[sl.len++]=f->magicmap;
-	    }
-
-      } /* x loop */
+	for (x = xmin; x <= xmax; x++) {
+	    sl.buf[sl.len++]= map_mark[x+MAGIC_MAP_SIZE*y] & ~FACE_FLOOR;
+	} /* x loop */
     } /* y loop */
     
     Send_With_Handling(&pl->contr->socket, &sl);
@@ -524,7 +539,7 @@ void draw_map(object *pl)
 }
 
 
-/*
+/**
  * Send a kill log record to sockets
  */
 
