@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *    "$Id: init.c,v 1.27 2002/08/26 07:14:11 mwedel Exp $";
+ *    "$Id: init.c,v 1.33 2003/12/18 20:40:44 ryo_saeba Exp $";
  */
 
 /*
@@ -26,7 +26,13 @@
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/* socket.c mainly deals with initialization and higher level socket
+/**
+ * \file
+ * Socket general functions
+ *
+ * \date 2003-12-02
+ *
+ * Mainly deals with initialization and higher level socket
  * maintenance (checking for lost connections and if data has arrived.)
  * The reading of data is handled in ericserver.c
  */
@@ -56,7 +62,8 @@
 Socket_Info socket_info;
 NewSocket *init_sockets;
 
-/* Initializes a connection - really, it just sets up the data structure,
+/**
+ * Initializes a connection. Really, it just sets up the data structure,
  * socket setup is handled elsewhere.  We do send a version to the
  * client.
  */
@@ -101,10 +108,9 @@ void InitConnection(NewSocket *ns, uint32 from)
     ns->sound = 0;
     ns->ext2 = 0;
     ns->ext_title_flag = 1;
-    ns->skillexp = 0;
+    ns->exp64 = 0;
     ns->mapmode = Map0Cmd;
     ns->darkness = 1;
-    ns->newanim = 0;
     ns->status = Ns_Add;
     ns->comment = NULL;
     ns->old_mode = 0;
@@ -112,6 +118,10 @@ void InitConnection(NewSocket *ns, uint32 from)
     ns->mapy = 11;
     ns->newmapcmd= 0;
     ns->itemcmd = 1;	/* Default is version item1 command*/
+    ns->ext_mapinfos=0; /*extendedmapinfo datas*/
+    ns->EMI_smooth=0; 
+    ns->look_position = 0;
+    ns->update_look = 0;
 
     /* we should really do some checking here - if total clients overflows
      * we need to do something more intelligent, because client id's will start
@@ -119,12 +129,12 @@ void InitConnection(NewSocket *ns, uint32 from)
      * just open and close connections could get this total up.
      */
     ns->inbuf.len=0;
-    ns->inbuf.buf=malloc(MAXSOCKBUF);
+    ns->inbuf.buf=malloc( sizeof( unsigned char ) * MAXSOCKBUF );
     /* Basic initialization. Needed because we do a check in
      * HandleClient for oldsocketmode without checking the
      * length of data.
      */
-    ns->inbuf.buf[0] = 0;
+    memset( ns->inbuf.buf, 0, sizeof( unsigned char ) * MAXSOCKBUF );
     memset(&ns->lastmap,0,sizeof(struct Map));
     memset(&ns->faces_sent,0,sizeof(ns->faces_sent));
     memset(&ns->anims_sent,0,sizeof(ns->anims_sent));
@@ -156,7 +166,7 @@ void InitConnection(NewSocket *ns, uint32 from)
 }
 
 
-/* This sets up the socket and reads all the image information into memory. */
+/** This sets up the socket and reads all the image information into memory. */
 void init_ericserver()
 {
     struct sockaddr_in	insock;
@@ -229,7 +239,11 @@ void init_ericserver()
         defined(__sun__) || defined(linux) || defined(SVR4) || defined(__FreeBSD__) || \
 	defined(__OpenBSD__) || defined(WIN32) /* ---win32 add this here */
     {
+#ifdef WIN32
+    char tmp = 1;
+#else
 	int tmp =1;
+#endif
 
 	if(setsockopt(init_sockets[0].fd,SOL_SOCKET,SO_REUSEADDR, &tmp, sizeof(tmp))) {
 	    perror("error on setsockopt REUSEADDR");
@@ -276,7 +290,7 @@ void init_ericserver()
  *
  ******************************************************************************/
 
-/* Free's all the memory that ericserver allocates. */
+/** Free's all the memory that ericserver allocates. */
 void free_all_newserver()
 {  
     LOG(llevDebug,"Freeing all new client/server information.\n");
@@ -284,7 +298,9 @@ void free_all_newserver()
     free(init_sockets);
 }
 
-/* basically, all we need to do here is free all data structures that
+/**
+ * Frees a socket.
+ * Basically, all we need to do here is free all data structures that
  * might be associated with the socket.  It is up to the caller to
  * update the list
  */
@@ -312,6 +328,7 @@ void free_newsocket(NewSocket *ns)
     free(ns->inbuf.buf);
 }
 
+/** Sends the 'goodbye' command to the player, and closes connection. */
 void final_free_player(player *pl)
 {
     cs_write_string(&pl->socket, "goodbye", 8);
