@@ -1,13 +1,13 @@
 
 /*
  * static char *rcsid_loop_c =
- *    "$Id: loop.c,v 1.19 2002/07/25 06:57:14 mwedel Exp $";
+ *    "$Id: loop.c,v 1.26 2003/12/02 18:51:44 ryo_saeba Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & The Crossfire Development Team
+    Copyright (C) 2002-2003 Mark Wedel & The Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,13 @@
     The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
-/* socket.c mainly deals with initialization and higher level socket
+/**
+ * \file
+ * Main client/server loops.
+ *
+ * \date 2003-12-02
+ *
+ * loop.c mainly deals with initialization and higher level socket
  * maintenance (checking for lost connections and if data has arrived.)
  * The reading of data is handled in ericserver.c
  */
@@ -84,7 +90,9 @@ struct PlCmdMapping {
     uint8   flag;
 };
 
-/*
+/**
+ * Dispatch table for the server.
+ *
  * CmdMapping is the dispatch table for the server, used in HandleClient,
  * which gets called when the client has input.  All commands called here 
  * use the same parameter form (char* data, int len, int clientnum.
@@ -109,6 +117,7 @@ static struct PlCmdMapping plcommands[] = {
     { NULL, NULL}	/* terminator */
 };
 
+/** Face-related commands */
 static struct NsCmdMapping nscommands[] = {
     { "addme",		AddMeCmd },
     { "askface",	SendFaceCmd},	/* Added: phil */
@@ -117,10 +126,13 @@ static struct NsCmdMapping nscommands[] = {
     { "setsound",	SetSound},
     { "setup",		SetUp},
     { "version",	VersionCmd },
+    { "toggleextendedinfos", ToggleExtendedInfos}, /*Added: tchize*/
+    { "asksmooth", AskSmooth},   /*Added: tchize (smoothing technologies)*/ 
     { NULL, NULL}	/* terminator */
 };
 
-/* RequestInfo is sort of a meta command - there is some specific
+/**
+ * RequestInfo is sort of a meta command. There is some specific
  * request of information, but we call other functions to provide
  * that information.
  */
@@ -149,9 +161,13 @@ void RequestInfo(char *buf, int len, NewSocket *ns)
 	}
     if (!strcmp(buf, "image_info")) send_image_info(ns, params);
     else if (!strcmp(buf,"image_sums")) send_image_sums(ns, params);
+    else if (!strcmp(buf,"skill_info")) send_skill_info(ns, params);
     else Write_String_To_Socket(ns, bigbuf, len);
 }
 
+/**
+ * Handles old socket format.
+ */
 void Handle_Oldsocket(NewSocket *ns)
 {
     int stat,i;
@@ -188,6 +204,8 @@ void Handle_Oldsocket(NewSocket *ns)
 	}
     } while (ns->inbuf.buf[ns->inbuf.len++]!='\n');
 
+    ns->inbuf.buf[ns->inbuf.len]=0;
+
     cp = strchr(ns->inbuf.buf, ' ');
     if (cp) {
 	/* Replace the space with a null, skip any more spaces */
@@ -277,7 +295,10 @@ void Handle_Oldsocket(NewSocket *ns)
 }
 
 
-/* HandleClient is actually not named really well - we only get here once
+/**
+ * Handle client input.
+ *
+ * HandleClient is actually not named really well - we only get here once
  * there is input, so we don't do exception or other stuff here.
  * sock is the output socket information.  pl is the player associated
  * with this socket, null if no player (one of the init_sockets for just
@@ -380,8 +401,9 @@ void HandleClient(NewSocket *ns, player *pl)
  ******************************************************************************/
 
 #ifdef WATCHDOG
-/*
+/**
  * Tell watchdog that we are still alive
+ *
  * I put the function here since we should hopefully already be getting
  * all the needed include files for socket support 
  */
@@ -410,6 +432,7 @@ void watchdog(void)
 
 extern unsigned long todtick;
 
+/** Waits for new connection */
 static void block_until_new_connection()
 {
 
@@ -433,8 +456,13 @@ static void block_until_new_connection()
 	}
 	FD_ZERO(&readfs);
 	FD_SET((uint32)init_sockets[0].fd, &readfs);
-	Timeout.tv_sec=60;
-	Timeout.tv_usec=0;
+	if (settings.fastclock > 0) {
+	    Timeout.tv_sec=0;
+	    Timeout.tv_usec=50;
+	} else {
+	    Timeout.tv_sec=60;
+	    Timeout.tv_usec=0;
+	}
 #ifdef WATCHDOG
 	watchdog();
 #endif
@@ -446,8 +474,10 @@ static void block_until_new_connection()
 }
 
 
-/* This checks the sockets for input and exceptions, does the right thing.  A 
- * bit of this code is grabbed out of socket.c
+/**
+ * This checks the sockets for input and exceptions, does the right thing.
+ *
+ * A bit of this code is grabbed out of socket.c
  * There are 2 lists we need to look through - init_sockets is a list
  * 
  */
@@ -628,7 +658,8 @@ void doeric_server()
 		 * sending them whenever they change, and probably just as useful
 		 */
 		esrv_update_stats(pl);
-		draw(pl->ob);
+		if (pl->ob->map && pl->ob->map->in_memory==MAP_IN_MEMORY)
+		    draw_client_map(pl->ob);
 		if (pl->socket.update_look) esrv_draw_look(pl->ob);
 	    }
 	}
