@@ -1,24 +1,34 @@
 /*
  * static char *rcsid_sound_c =
- *   "$Id: sounds.c,v 1.2 1999/07/13 06:03:06 cvs Exp $";
+ *   "$Id: sounds.c,v 1.5 2003/12/14 16:28:47 ryo_saeba Exp $";
  */
 
 /* Send bug reports to Raphael Quinet (quinet@montefiore.ulg.ac.be) */
 
+/**
+ * \file
+ * Sound-related functions.
+ *
+ * \date 2003-12-02
+ */
+
 #include <global.h>
 #include <sproto.h>
 #include <sounds.h>
 
-int volume_table[NROF_SOUNDS];
-
-/* This is only used for new client/server sound.  If the sound source
+/**
+ * Maximum distance a player may hear a sound from.
+ * This is only used for new client/server sound.  If the sound source
  * on the map is farther away than this, we don't sent it to the client.
  */
 #define MAX_SOUND_DISTANCE 10
 
-void play_sound_player_only(player *pl, int soundnum,  int x, int y)
+/**
+ * Plays a sound for specified player only
+ */
+void play_sound_player_only(player *pl, short soundnum,  sint8 x, sint8 y)
 {
-    int soundtype;
+    char soundtype;
     SockList sl;
 
     if (!pl->socket.sound) return;
@@ -42,8 +52,8 @@ void play_sound_player_only(player *pl, int soundnum,  int x, int y)
 
 #define POW2(x) ((x) * (x))
 
-/* Plays some sound on map at x,y.  */
-void play_sound_map(mapstruct *map, int x, int y, int sound_num)
+/** Plays some sound on map at x,y.  */
+void play_sound_map(mapstruct *map, int x, int y, short sound_num)
 {
     player *pl;
 
@@ -57,7 +67,7 @@ void play_sound_map(mapstruct *map, int x, int y, int sound_num)
 	    int distance=isqrt(POW2(pl->ob->x - x) + POW2(pl->ob->y - y));
 
 	    if (distance<=MAX_SOUND_DISTANCE) {
-		play_sound_player_only(pl, sound_num, x-pl->ob->x, y-pl->ob->y);
+		play_sound_player_only(pl, sound_num, ( sint8 )( x-pl->ob->x ), ( sint8 )( y-pl->ob->y ));
 	    }
 	}
     }
